/*
<:copyright-gpl
 Copyright 2002 Broadcom Corp. All Rights Reserved.

 This program is free software; you can distribute it and/or modify it
 under the terms of the GNU General Public License (Version 2) as
 published by the Free Software Foundation.

 This program is distributed in the hope it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
:>
*/
/*
 ***************************************************************************
 * File Name  : bcm63xx_flash.c
 *
 * Description: This file contains the flash device driver APIs for bcm63xx board. 
 *
 * Created on :  8/10/2002  seanl:  use cfiflash.c, cfliflash.h (AMD specific)
 *
 ***************************************************************************/


/* Includes. */
#include <linux/fs.h>
#include <linux/capability.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <asm/uaccess.h>

#include <bcm_map_part.h>
#include <board.h>
#define  BCMTAG_EXE_USE
#include <bcmTag.h>
#include "cfiflash.h"
#include "boardparms.h"

//#define DEBUG_FLASH

static FLASH_ADDR_INFO fInfo;
static int flashInitialized = 0;

/*Michael: for fix crash when push reset button */
void *retriedKmallocRESET(size_t size)
{
	void *pBuf;
    int tryCount = 0;

    // try 1000 times before quit
/*Michael: for fix crash when push reset button */
/* if we kill pppd and do reset button push, it goes fine. This method provided by viclin */
//    while (((pBuf = kmalloc(size, GFP_KERNEL)) == NULL) && (tryCount++ < 1000)){
    while (((pBuf = kmalloc(size, GFP_ATOMIC)) == NULL) && (tryCount++ < 1000)){
		current->state   = TASK_INTERRUPTIBLE;
		schedule_timeout(HZ/10);
	}
    if (tryCount >= 1000)
        pBuf = NULL;
    else
	memset(pBuf, 0, size);

    return pBuf;
}


void *retriedKmalloc(size_t size)
{
	void *pBuf;
    int tryCount = 0;

    // try 1000 times before quit
    while (((pBuf = kmalloc(size, GFP_KERNEL)) == NULL) && (tryCount++ < 1000)){
		current->state   = TASK_INTERRUPTIBLE;
		schedule_timeout(HZ/10);
	}
    if (tryCount >= 1000)
        pBuf = NULL;
    else
	    memset(pBuf, 0, size);

    return pBuf;
}




void retriedKfree(void *pBuf)
{
	kfree(pBuf);
}

/***************************************************************************
// Function Name: getCrc32
// Description  : caculate the CRC 32 of the given data.
// Parameters   : pdata - array of data.
//                size - number of input data bytes.
//                crc - either CRC32_INIT_VALUE or previous return value.
// Returns      : crc.
****************************************************************************/
UINT32 getCrc32(byte *pdata, UINT32 size, UINT32 crc) 
{
    while (size-- > 0)
        crc = (crc >> 8) ^ Crc32_table[(crc ^ *pdata++) & 0xff];

    return crc;
}

// get the nvram start addr
//
unsigned long get_nvram_start_addr(void)
{
    return ((unsigned long) 
        (flash_get_memptr(fInfo.flash_nvram_start_blk) + fInfo.flash_nvram_blk_offset));
}

// get the scratch_pad start addr
//
unsigned long get_scratch_pad_start_addr(void)
{
    return ((unsigned long) 
        (flash_get_memptr(fInfo.flash_scratch_pad_start_blk) + fInfo.flash_scratch_pad_blk_offset));
}


#if defined(CONFIG_BCM96348)  
/*  *********************************************************************
    *  kerSysImageTagGet()
    *   Get the image tag
    *  Input parameters:
    *      none
    *  Return value:
    *      point to tag -- Found
    *      NULL -- failed
    ********************************************************************* */
PFILE_TAG kerSysImageTagGet(void)
{
    int i,startblock;
    int totalBlks = flash_get_numsectors();
    int totalSize = flash_get_total_size();
    UINT32 crc;
    unsigned char *sectAddr;
    PFILE_TAG pTag;
    unsigned char   nbootflag,*bootflag;

    bootflag=flash_get_memptr((byte) 0);
    
    if (totalSize == 0x800000) {nbootflag=*bootflag;}
    else nbootflag=1;
    
    if (nbootflag == 2) nbootflag = 0; /*binfs is valid to boot from bfc00000*/
    startblock = (nbootflag?1:(totalBlks>>1)+1);
#if defined(DEBUG_FLASH)
    printk("totalblks in tagGet=%d\n", totalBlks);
#endif


    // start blk, assume 1st one is always CFE
    for (i = startblock; i < startblock + (totalBlks>>1); i++)
    {
        sectAddr =  flash_get_memptr((byte) i);
        crc = CRC32_INIT_VALUE;
        crc = getCrc32(sectAddr, (UINT32)TAG_LEN-TOKEN_LEN, crc);      
        pTag = (PFILE_TAG) sectAddr;

#if defined(DEBUG_FLASH)
        printk("Check Tag crc on blk [%d]\n", i);
#endif

        if (crc == (UINT32)(*(UINT32*)(pTag->tagValidationToken)))
            return pTag;
    }

    return (PFILE_TAG) NULL;
}

PFILE_TAG kerSysImageTagGet2(void)
{
    int i,startblock;
    int totalBlks = flash_get_numsectors();
    int totalSize = flash_get_total_size();
    UINT32 crc;
    unsigned char *sectAddr;
    PFILE_TAG pTag;
    unsigned char   nbootflag,*bootflag;

    bootflag=flash_get_memptr((byte) 0);
    
    if (totalSize == 0x800000)  nbootflag=1; /*always get from bf800000 */
    else return (PFILE_TAG) NULL;
    
    startblock = (nbootflag?1:(totalBlks>>1)+1);
#if defined(DEBUG_FLASH)
    printk("totalblks in tagGet=%d\n", totalBlks);
#endif


    // start blk, assume 1st one is always CFE
    for (i = startblock; i < startblock + (totalBlks>>1); i++)
    {
        sectAddr =  flash_get_memptr((byte) i);
        crc = CRC32_INIT_VALUE;
        crc = getCrc32(sectAddr, (UINT32)TAG_LEN-TOKEN_LEN, crc);      
        pTag = (PFILE_TAG) sectAddr;

        if (*((unsigned char*) pTag->kernelLen) == 0x30 ) {
        	
#if defined(DEBUG_FLASH)
        printk("Check Tag crc on blk [%d]\n", i);
#endif

            if (crc == (UINT32)(*(UINT32*)(pTag->tagValidationToken)))
                return pTag;
        }
    }

    return  (PFILE_TAG) NULL;
}


#endif

#if defined(CONFIG_BCM96338) || defined(CONFIG_BCM96345)
PFILE_TAG kerSysImageTagGet(void)
{
    int i,startblock;
    int totalBlks = flash_get_numsectors();
    int totalSize = flash_get_total_size();
    UINT32 crc;
    unsigned char *sectAddr;
    PFILE_TAG pTag;
    unsigned char bootf1,bootf2,nbootflag=0;

    
if (totalSize == 0x400000) {
    bootf1 = *(unsigned char*) (0xbfe00000 -1); /*see design note 4MB flash layout*/
    bootf2 = *(unsigned char*) (0xbfff0000 -1);
    if ((bootf1 >> 2) == 0 && bootf1 ) {
        if ((bootf2 >> 2) == 0 && bootf2 ) { /* both are valid */
          if ((bootf1+1) == bootf2 || ((bootf1+1) >>2 == bootf2) ) nbootflag=1; /*bank2*/
          if ((bootf2+1) == bootf1 || ((bootf2+1) >>2 == bootf1) ) nbootflag=0; /*bank1*/ 
        } else nbootflag = 0; /* bank2 invalid bank1 valid */
    }
    else  if ((bootf2 >> 2) == 0 && bootf2) nbootflag =1; /*bank 1 invalid,bank 2 valid*/
    else  nbootflag=0; /* bank1 bank2 both invalid */
    
    printk("bootf1=%x bootf2=%x\n", bootf1,bootf2);
}
else nbootflag=0; 
    
    startblock = nbootflag?flash_get_blk(0xbfe00000):1;
#if defined(DEBUG_FLASH)
    printk("totalblks in tagGet=%d\n", totalBlks);
#endif


    // start blk, assume 1st one is always CFE
    for (i = startblock; i < startblock + (totalBlks>>1); i++)
    {
        sectAddr =  flash_get_memptr((byte) i);
        crc = CRC32_INIT_VALUE;
        crc = getCrc32(sectAddr, (UINT32)TAG_LEN-TOKEN_LEN, crc);      
        pTag = (PFILE_TAG) sectAddr;

#if defined(DEBUG_FLASH)
        printk("Check Tag crc on blk [%d]\n", i);
#endif

        if (crc == (UINT32)(*(UINT32*)(pTag->tagValidationToken)))
            return pTag;
    }

    return (PFILE_TAG) NULL;	
}  
#endif
// Initialize the flash and fill out the fInfo structure
void kerSysFlashInit( void )
{
    int i = 0;
    int totalBlks = 0;
    int totalSize = 0;
    int startAddr = 0;
    int usedBlkSize = 0;
    NVRAM_DATA nvramData;
    UINT32 crc = CRC32_INIT_VALUE, savedCrc;
    PFILE_TAG pTag = NULL;
    unsigned long kernelEndAddr = 0;
    unsigned long spAddr = 0;

    if (flashInitialized)
        return;

    flashInitialized = 1;
    flash_init();

    totalBlks = flash_get_numsectors();
    totalSize = flash_get_total_size();

    /* nvram is always at the end of flash */
    fInfo.flash_nvram_length = FLASH45_LENGTH_NVRAM;

printk("FLASH_BASE %x,blk %x\n",FLASH_BASE,flash_get_blk(FLASH_BASE));
    fInfo.flash_nvram_start_blk = flash_get_blk(FLASH_BASE);	/* denny 09/13/04' modified, set the nvram locate at boot loader sector */
    fInfo.flash_nvram_number_blk = 1; /*always fits in the first block */
    fInfo.flash_nvram_blk_offset = NVRAM_DATA_OFFSET;
 
    printk("Total Flash size: %dK with %d sectors NVRAM @%d block\n", totalSize/1024, totalBlks, fInfo.flash_nvram_start_blk);

    // check nvram CRC
    memcpy((char *)&nvramData, (char *)get_nvram_start_addr(), sizeof(NVRAM_DATA));
    savedCrc = nvramData.ulCheckSum;
    nvramData.ulCheckSum = 0;
    crc = getCrc32((char *)&nvramData, (UINT32) sizeof(NVRAM_DATA), crc);   

    BpSetBoardId( nvramData.szBoardId );

    fInfo.flash_persistent_length = NVRAM_PSI_DEFAULT;
    if (savedCrc != crc)
    {
        printk("***Board is not initialized****: Using the default PSI size: %d\n",
            fInfo.flash_persistent_length);
    } else {
        unsigned long ulPsiSize;
        if( BpGetPsiSize( &ulPsiSize ) == BP_SUCCESS ) {
            fInfo.flash_persistent_length = ulPsiSize;
        } else {
            printk("***Board id is not set****: Using the default PSI size: %d\n",
                fInfo.flash_persistent_length);
        }
    }

    fInfo.flash_persistent_length *= ONEK;
    startAddr = totalSize - fInfo.flash_persistent_length;
    fInfo.flash_persistent_start_blk = flash_get_blk(startAddr+FLASH_BASE_ADDR_REG);
    fInfo.flash_persistent_number_blk = totalBlks - fInfo.flash_persistent_start_blk;
    // save abs SP address (Scratch Pad). it is before PSI 
    spAddr = startAddr - SP_MAX_LEN ;
    // find out the offset in the start_blk
    usedBlkSize = 0;
    for (i = fInfo.flash_persistent_start_blk; 
        i < (fInfo.flash_persistent_start_blk + fInfo.flash_persistent_number_blk); i++)
    {
        usedBlkSize += flash_get_sector_size( i);
    }
    fInfo.flash_persistent_blk_offset =  usedBlkSize - fInfo.flash_persistent_length;

    // get the info for sp
    if (!(pTag = kerSysImageTagGet())) {
        printk("Failed to read image tag from flash\n");
        return;
    }
    kernelEndAddr = (unsigned long) simple_strtoul(pTag->kernelAddress, NULL, 10) + \
        (unsigned long) simple_strtoul(pTag->kernelLen, NULL, 10);

    // make suer sp does not share kernel block
    fInfo.flash_scratch_pad_start_blk = flash_get_blk(spAddr+FLASH_BASE_ADDR_REG);
    if (fInfo.flash_scratch_pad_start_blk != flash_get_blk(kernelEndAddr))
    {
        fInfo.flash_scratch_pad_length = SP_MAX_LEN;
        if (fInfo.flash_persistent_start_blk == fInfo.flash_scratch_pad_start_blk)  // share blk
        {
#if 1 /* do not used scratch pad unless it's in its own sector */
            printk("Scratch pad is not used for this flash part.\n");  
            fInfo.flash_scratch_pad_length = 0;     // no sp
#else /* allow scratch pad to share a sector with another section such as PSI */
            fInfo.flash_scratch_pad_number_blk = 1;
            fInfo.flash_scratch_pad_blk_offset = fInfo.flash_persistent_blk_offset - fInfo.flash_scratch_pad_length;
#endif
        }else {// on different blk
            fInfo.flash_scratch_pad_number_blk = fInfo.flash_persistent_start_blk- fInfo.flash_scratch_pad_start_blk;
            // find out the offset in the start_blk
            usedBlkSize = 0;
            for (i = fInfo.flash_scratch_pad_start_blk; 
                i < (fInfo.flash_scratch_pad_start_blk + fInfo.flash_scratch_pad_number_blk); i++)
                usedBlkSize += flash_get_sector_size( i);
                fInfo.flash_scratch_pad_blk_offset =  usedBlkSize - fInfo.flash_scratch_pad_length;
        }
    }else{
        printk("No flash for scratch pad!\n");  
        fInfo.flash_scratch_pad_length = 0;     // no sp
    }

    /*==== add by Andrew (2004/09/14) ====*/
#if defined(CFG_XFER_TO_FACDEFLT)
    // save abs factdeflt address (Factory Default). it is before SP
    unsigned long dfAddr;

   	if(fInfo.flash_scratch_pad_length>0) {
        dfAddr = spAddr - NVRAM_PSI_DEFAULT*ONEK;
        // make sure factdeflt does not share kernel block
        fInfo.flash_factdeflt_start_blk = flash_get_blk(dfAddr+FLASH_BASE_ADDR_REG);
        if (fInfo.flash_factdeflt_start_blk != flash_get_blk(kernelEndAddr)) {
            fInfo.flash_factdeflt_length = NVRAM_PSI_DEFAULT*ONEK;
            if (fInfo.flash_factdeflt_start_blk == fInfo.flash_scratch_pad_start_blk)  // share blk
            {
                fInfo.flash_factdeflt_number_blk = 1;
                fInfo.flash_factdeflt_blk_offset = fInfo.flash_scratch_pad_blk_offset - fInfo.flash_factdeflt_length;
            } else { // on different blk
                fInfo.flash_factdeflt_number_blk = fInfo.flash_scratch_pad_start_blk - fInfo.flash_factdeflt_start_blk;
                // find out the offset in the start_blk
                usedBlkSize = 0;
                for (i = fInfo.flash_factdeflt_start_blk; 
                    i < (fInfo.flash_factdeflt_start_blk + fInfo.flash_factdeflt_number_blk); i++)
                    usedBlkSize += flash_get_sector_size( i);
                fInfo.flash_factdeflt_blk_offset =  usedBlkSize - fInfo.flash_factdeflt_length;
            }
        } else {
            printk("Not enough flash space to store factory default setting!\n");  
            fInfo.flash_factdeflt_length = 0;     // no df
        }
    } else {
   	    dfAddr = startAddr - NVRAM_PSI_DEFAULT*ONEK;
        // make sure factdeflt does not share kernel block
        fInfo.flash_factdeflt_start_blk = flash_get_blk(dfAddr+FLASH_BASE_ADDR_REG);
        if (fInfo.flash_factdeflt_start_blk != flash_get_blk(kernelEndAddr)) {
            fInfo.flash_factdeflt_length = NVRAM_PSI_DEFAULT*ONEK;
            if (fInfo.flash_factdeflt_start_blk == fInfo.flash_persistent_start_blk)  // share blk
            {
                fInfo.flash_factdeflt_number_blk = 1;
                fInfo.flash_factdeflt_blk_offset = fInfo.flash_persistent_blk_offset - fInfo.flash_factdeflt_length;
            } else { // on different blk
                fInfo.flash_factdeflt_number_blk = fInfo.flash_persistent_start_blk - fInfo.flash_factdeflt_start_blk;
                // find out the offset in the start_blk
                usedBlkSize = 0;
                for (i = fInfo.flash_factdeflt_start_blk; 
                    i < (fInfo.flash_factdeflt_start_blk + fInfo.flash_factdeflt_number_blk); i++)
                    usedBlkSize += flash_get_sector_size( i);
                fInfo.flash_factdeflt_blk_offset =  usedBlkSize - fInfo.flash_factdeflt_length;
            }
        } else {
            printk("Not enough flash space to store factory default setting!\n");  
            fInfo.flash_factdeflt_length = 0;     // no df
        }
    }
#endif
    /*==== end ====*/
#if defined(DEBUG_FLASH)
  #if defined(CFG_XFER_TO_FACDEFLT)
    printk("dfAddr = %x\n", dfAddr+FLASH_BASE_ADDR_REG);
    printk("fInfo.flash_factdeflt_start_blk = %d\n", fInfo.flash_factdeflt_start_blk);
    printk("fInfo.flash_factdeflt_number_blk = %d\n", fInfo.flash_factdeflt_number_blk);
    printk("fInfo.flash_factdeflt_length = 0x%x\n", fInfo.flash_factdeflt_length);
    printk("fInfo.flash_factdeflt_blk_offset = 0x%x\n", (unsigned int)fInfo.flash_factdeflt_blk_offset);
  #endif
    printk("fInfo.flash_scratch_pad_start_blk = %d\n", fInfo.flash_scratch_pad_start_blk);
    printk("fInfo.flash_scratch_pad_number_blk = %d\n", fInfo.flash_scratch_pad_number_blk);
    printk("fInfo.flash_scratch_pad_length = 0x%x\n", fInfo.flash_scratch_pad_length);
    printk("fInfo.flash_scratch_pad_blk_offset = 0x%x\n", (unsigned int)fInfo.flash_scratch_pad_blk_offset);

    printk("fInfo.flash_nvram_start_blk = %d\n", fInfo.flash_nvram_start_blk);
    printk("fInfo.flash_nvram_blk_offset = 0x%x\n", (unsigned int)fInfo.flash_nvram_blk_offset);
    printk("fInfo.flash_nvram_number_blk = %d\n", fInfo.flash_nvram_number_blk);

    printk("psi startAddr = %x\n", startAddr+FLASH_BASE_ADDR_REG);
    printk("fInfo.flash_persistent_start_blk = %d\n", fInfo.flash_persistent_start_blk);
    printk("fInfo.flash_persistent_number_blk = %d\n", fInfo.flash_persistent_number_blk);
    printk("fInfo.flash_persistent_length = 0x%x\n", fInfo.flash_persistent_length);
    printk("fInfo.flash_persistent_blk_offset = 0x%x\n", (unsigned int)fInfo.flash_persistent_blk_offset);
#endif
}



/***********************************************************************
 * Function Name: kerSysFlashAddrInfoGet
 * Description  : Fills in a structure with information about the NVRAM
 *                and persistent storage sections of flash memory.  
 *                Fro physmap.c to mount the fs vol.
 * Returns      : None.
 ***********************************************************************/
void kerSysFlashAddrInfoGet(PFLASH_ADDR_INFO pflash_addr_info)
{
    pflash_addr_info->flash_nvram_blk_offset = fInfo.flash_nvram_blk_offset;
    pflash_addr_info->flash_nvram_length = fInfo.flash_nvram_length;
    pflash_addr_info->flash_nvram_number_blk = fInfo.flash_nvram_number_blk;
    pflash_addr_info->flash_nvram_start_blk = fInfo.flash_nvram_start_blk;
    pflash_addr_info->flash_persistent_blk_offset = fInfo.flash_persistent_blk_offset;
    pflash_addr_info->flash_persistent_length = fInfo.flash_persistent_length;
    pflash_addr_info->flash_persistent_number_blk = fInfo.flash_persistent_number_blk;
    pflash_addr_info->flash_persistent_start_blk = fInfo.flash_persistent_start_blk;
}


// get shared blks into *** pTempBuf *** which has to be released bye the caller!
// return: if pTempBuf != NULL, poits to the data with the dataSize of the buffer
// !NULL -- ok
// NULL  -- fail
static char *getSharedBlks(int start_blk, int end_blk)
{
    int i = 0;
    int usedBlkSize = 0;
    int sect_size = 0;
    char *pTempBuf = NULL;
    char *pBuf = NULL;

    for (i = start_blk; i < end_blk; i++)
        usedBlkSize += flash_get_sector_size( i);

#if defined(DEBUG_FLASH)
    printk("usedBlkSize = %d\n", usedBlkSize);
#endif

    if ((pTempBuf = (char *) retriedKmalloc(usedBlkSize)) == NULL)
    {
        printk("failed to allocate memory with size: %d\n", usedBlkSize);
        return pTempBuf;
    }
    
    pBuf = pTempBuf;
    for (i = start_blk; i < end_blk; i++)
    {
        sect_size = flash_get_sector_size( i);

#if defined(DEBUG_FLASH)
        printk("i = %d, sect_size = %d, end_blk = %d\n", i, sect_size, end_blk);
#endif
        flash_read_buf(i, 0, pBuf, sect_size);
        pBuf += sect_size;
    }
    return pTempBuf;
}
    
/*Michael: for fix crash when push reset button */
/* We add this because it happen only at 2Mflash 8M SDRAM. */
static char *getSharedBlksRESET(int start_blk, int end_blk)
{
    int i = 0;
    int usedBlkSize = 0;
    int sect_size = 0;
    char *pTempBuf = NULL;
    char *pBuf = NULL;

    for (i = start_blk; i < end_blk; i++)
        usedBlkSize += flash_get_sector_size( i);

#if defined(DEBUG_FLASH)
    printk("usedBlkSize = %d\n", usedBlkSize);
#endif
    
    if ((pTempBuf = (char *) retriedKmallocRESET(usedBlkSize)) == NULL)
    {
        printk("failed to allocate memory with size: %d\n", usedBlkSize);
    return pTempBuf;
}

    pBuf = pTempBuf;
    for (i = start_blk; i < end_blk; i++)
    {
        sect_size = flash_get_sector_size( i);
#if defined(DEBUG_FLASH)
        printk("i = %d, sect_size = %d, end_blk = %d\n", i, sect_size, end_blk);
#endif
        flash_read_buf(i, 0, pBuf, sect_size);
        pBuf += sect_size;
    }
    return pTempBuf;
}


// Set the pTempBuf to flash from start_blk to end_blk
// return:
// 0 -- ok
// -1 -- fail
static int setSharedBlks(int start_blk, int end_blk, char *pTempBuf)
{
    int i = 0;
    int sect_size = 0;
    int sts = 0;
    char *pBuf = pTempBuf;
//printk("yes sSB..no cli\n");
//cli();
    for (i = start_blk; i < end_blk; i++){
        sect_size = flash_get_sector_size( i);
        flash_sector_erase_int(i);
        if (flash_write_buf(i, 0, pBuf, sect_size) != sect_size){
            printk("Error writing flash sector %d.", i);
            sts = -1;
            break;
        }

        pBuf += sect_size;
    }
//sti();
if (sts == -1) printk("Error writing flash sector %d.", i);
    return sts;
}



/*******************************************************************************
 * NVRAM functions
 *******************************************************************************/

// get nvram data
// return:
//  0 - ok
//  -1 - fail
int kerSysNvRamGet(char *string, int strLen, int offset)
{
    char *pBuf = NULL;

    if (!flashInitialized)
        kerSysFlashInit();

    if (strLen > FLASH45_LENGTH_NVRAM)
        return -1;

    if ((pBuf = getSharedBlks(fInfo.flash_nvram_start_blk,
        (fInfo.flash_nvram_start_blk + fInfo.flash_nvram_number_blk))) == NULL)
        return -1;

    // get string off the memory buffer
    memcpy(string, (pBuf + fInfo.flash_nvram_blk_offset + offset), strLen);

    retriedKfree(pBuf);

    return 0;
}

#if defined(CONFIG_BCM96348) 
int kerSysErasePS1()
{
	int i,j;
	i=j=0;
	
         do {
	 i+=flash_get_sector_size(j);
	 j++;
	} while (i<64*1024);
	
	flash_sector_erase_int(j);

	
}
int kerSysBootFlagSet(char bootflag)
{
    int i = 0;
    int sect_size = 0;
    int sts = 0;
    char *pTempBuf = NULL;
    
       printk("set bootflag %d\n",bootflag);
        sect_size = flash_get_sector_size(0);	/* denny 09/13/04' modified */
        if ((pTempBuf = (char *) retriedKmalloc(sect_size)) == NULL) {
            printk("failed to allocate memory with size: %d\n", sect_size);
            return -1;
        
        }        
        flash_read_buf(0, 0, pTempBuf, sect_size);	
        flash_sector_erase_int(0);
        *((char *) pTempBuf) = bootflag;
        if (flash_write_buf(0, 0, pTempBuf, sect_size) != sect_size)
        {
            printk("Error writing flash sector %d.\n", i);
            sts = -1;
        }
        retriedKfree(pTempBuf);	
        return sts;
}
#endif

#if defined(CONFIG_BCM96338) 
int kerSysErasePS1()
{
	int j;
	
    if (flash_get_total_size() == 0x400000) {
        j=flash_get_blk(0xbfe00000);
	flash_sector_erase_int(j); /*erase the first block of bank 2*/
	j=flash_get_blk(0xbffe0000);
	flash_sector_erase_int(j); /*erase the bootflag of bank 2*/
    }
	
}

#ifdef NNN_POT

#define ON_USE	0

unsigned int kerPOTSet(unsigned int set)
{
    unsigned int rcnt=0,offset=0;
    unsigned short record;    
    unsigned short *memptr=(unsigned short*)flash_get_memptr(fInfo.flash_persistent_start_blk-1);

if ( set==2 ) {
  
    flash_sector_erase_int(fInfo.flash_persistent_start_blk-1);
    
} else {    
    
   while (*memptr == ON_USE && memptr<flash_get_memptr(fInfo.flash_persistent_start_blk)) {
           rcnt++;memptr++;offset+=2 ;
   }
   
   if (set && (rcnt <= 60*24*3) ) {    
   	  
   	  if (*memptr == 0xffff) {
   	  	
         flash_command(FLASH_UB, 0, 0, 0);
         flash_command(FLASH_PROG, fInfo.flash_persistent_start_blk-1, offset, 0);
         if (flash_wait(fInfo.flash_persistent_start_blk-1, offset, 0) != STATUS_READY) 
              printk("not ready\n");
          flash_command(FLASH_UBRESET, 0, 0, 0);
       }
       else printk("Unable to set POT\n");
    }    
    //printk("memptr %08x rcnt %d\n",memptr,rcnt);
}    
    return rcnt;
        	
}
#endif

/* bootflag will indicate which bank will be set */
int kerSysBootFlagSet(char bootflag)
{
    int i = 0;
    int sect_size = 0;
    int sts = 0;
    char *pTempBuf = NULL;
    unsigned char bootnum;
    int sector;
    
sector =bootflag?flash_get_blk(0xbffe0000):flash_get_blk(0xbfdf0000);
bootnum=bootflag?*(unsigned char*) (0xbfe00000-1):*(unsigned char*) (0xbfff0000-1);  /*flash bank1 get bootnum from bank2*/  

if ((bootnum >> 2)==0 && bootnum) {
    bootnum=(bootnum+1);
    if (bootnum>>2) bootnum>>=2;
}
else bootnum = 1;
   
        sect_size = flash_get_sector_size(sector);	/* denny 09/13/04' modified */
        if ((pTempBuf = (char *) retriedKmalloc(sect_size)) == NULL) {
            printk("failed to allocate memory with size: %d\n", sect_size);
            return -1;
        
        }        
        flash_read_buf(sector, 0, pTempBuf, sect_size);	
        flash_sector_erase_int(sector);
        *((char *) (pTempBuf+sect_size-1)) = bootnum;
        if (flash_write_buf(sector, 0, pTempBuf, sect_size) != sect_size)
        {
            printk("Error writing flash sector %d.\n", i);
            sts = -1;
        }
        retriedKfree(pTempBuf);	
        printk("Set next boot from bank %d/flag set to %d\n",bootflag,bootnum);
        return sts;
}
#endif
// set nvram 
// return:
//  0 - ok
//  -1 - fail
int kerSysNvRamSet(char *string, int strLen, int offset)
{
    int sts = 0;
    char *pBuf = NULL;

    if (strLen > FLASH45_LENGTH_NVRAM)
        return -1;
printk("nvram set %s, %d, %d\n",string,strLen,offset);
    if ((pBuf = getSharedBlks(fInfo.flash_nvram_start_blk,
        (fInfo.flash_nvram_start_blk + fInfo.flash_nvram_number_blk))) == NULL)
        return -1;

    // set string to the memory buffer
    memcpy((pBuf + fInfo.flash_nvram_blk_offset + offset), string, strLen);

    if (setSharedBlks(fInfo.flash_nvram_start_blk, 
        (fInfo.flash_nvram_number_blk + fInfo.flash_nvram_start_blk), pBuf) != 0)
        sts = -1;
    
    retriedKfree(pBuf);

    return sts;
}


/***********************************************************************
 * Function Name: kerSysEraseNvRam
 * Description  : Erase the NVRAM storage section of flash memory.
 * Returns      : 1 -- ok, 0 -- fail
 ***********************************************************************/
int kerSysEraseNvRam(void)
{
    int sts = 1;
    char *tempStorage = retriedKmalloc(FLASH45_LENGTH_NVRAM);
    
    // just write the whole buf with '0xff' to the flash
    if (!tempStorage)
        sts = 0;
    else{
        memset(tempStorage, 0xff, FLASH45_LENGTH_NVRAM);
        if (kerSysNvRamSet(tempStorage, FLASH45_LENGTH_NVRAM, 0) != 0)
            sts = 0;
        retriedKfree(tempStorage);
    }

    return sts;
}

/*==== add by Andrew (2004/09/14) ====*/
#if defined(CFG_XFER_TO_FACDEFLT)
int kerSysFactDefltGet(char *string, int strLen, int offset)
{
    char *pBuf = NULL;

    if (strLen > fInfo.flash_factdeflt_length)
        return -1;

    if ((pBuf = getSharedBlks(fInfo.flash_factdeflt_start_blk,
        (fInfo.flash_factdeflt_start_blk + fInfo.flash_factdeflt_number_blk))) == NULL)
        return -1;

    // get string off the memory buffer
    memcpy(string, (pBuf + fInfo.flash_factdeflt_blk_offset + offset), strLen);

    retriedKfree(pBuf);

    return 0;
}


// set factdeflt
// return:
//  0 - ok
//  -1 - fail
int kerSysFactDefltSet(char *string, int strLen, int offset)
{
    int sts = 0;
    char *pBuf = NULL;

    if (strLen > fInfo.flash_factdeflt_length)
        return -1;

    if ((pBuf = getSharedBlks(fInfo.flash_factdeflt_start_blk,
        (fInfo.flash_factdeflt_start_blk + fInfo.flash_factdeflt_number_blk))) == NULL)
        return -1;

    // set string to the memory buffer
    memcpy((pBuf + fInfo.flash_factdeflt_blk_offset + offset), string, strLen);
    if (setSharedBlks(fInfo.flash_factdeflt_start_blk, 
        (fInfo.flash_factdeflt_number_blk + fInfo.flash_factdeflt_start_blk), pBuf) != 0)
        sts = -1;
    retriedKfree(pBuf);

    return sts;
}
#endif
/*==== end ====*/
/*******************************************************************************
 * PSI functions
 *******************************************************************************/
// get psi data
// return:
//  0 - ok
//  -1 - fail
int kerSysPersistentGet(char *string, int strLen, int offset)
{
    char *pBuf = NULL;

    if (strLen > fInfo.flash_persistent_length)
        return -1;

    if ((pBuf = getSharedBlks(fInfo.flash_persistent_start_blk,
        (fInfo.flash_persistent_start_blk + fInfo.flash_persistent_number_blk))) == NULL)
        return -1;

    // get string off the memory buffer
    memcpy(string, (pBuf + fInfo.flash_persistent_blk_offset + offset), strLen);

    retriedKfree(pBuf);

    return 0;
}


// set psi 
// return:
//  0 - ok
//  -1 - fail
int kerSysPersistentSet(char *string, int strLen, int offset)
{
    int sts = 0;
    char *pBuf = NULL;

    if (strLen > fInfo.flash_persistent_length)
        return -1;

    if ((pBuf = getSharedBlks(fInfo.flash_persistent_start_blk,
        (fInfo.flash_persistent_start_blk + fInfo.flash_persistent_number_blk))) == NULL)
        return -1;

    // set string to the memory buffer
    memcpy((pBuf + fInfo.flash_persistent_blk_offset + offset), string, strLen);
    if (setSharedBlks(fInfo.flash_persistent_start_blk, 
        (fInfo.flash_persistent_number_blk + fInfo.flash_persistent_start_blk), pBuf) != 0)
        sts = -1;
    retriedKfree(pBuf);

    return sts;
}

/*Michael: for fix crash when push reset button */
/* We add this because it happen only at 2Mflash 8M SDRAM. */
int kerSysPersistentSetRESET()
{
    int sts = 0;
    char *pBuf = NULL;

//    if (strLen > fInfo.flash_persistent_length)
//        return -1;
    if ((pBuf = getSharedBlksRESET(fInfo.flash_persistent_start_blk,
        (fInfo.flash_persistent_start_blk + fInfo.flash_persistent_number_blk))) == NULL)
        return -1;
    // set string to the memory buffer
    memset((pBuf + fInfo.flash_persistent_blk_offset), 0xff, fInfo.flash_persistent_length);
    // memcpy((pBuf + fInfo.flash_persistent_blk_offset + offset), string, strLen);
    if (setSharedBlks(fInfo.flash_persistent_start_blk,
        (fInfo.flash_persistent_number_blk + fInfo.flash_persistent_start_blk), pBuf) != 0)
        sts = -1;
    retriedKfree(pBuf);
    return sts;
}

// flash bcm image 
// return: 
// 0 - ok
// !0 - the sector number fail to be flashed (should not be 0)
int kerSysBcmImageSet( int flash_start_addr, char *string, int size)
{
    int sts;
    int sect_size;
    int blk_start;
    int i;
    char *pTempBuf = NULL;
    int whole_image = 0;

    blk_start = flash_get_blk(flash_start_addr);
    if( blk_start < 0 )
        return( -1 );

    if (flash_start_addr == FLASH_BASE && size > FLASH45_LENGTH_BOOT_ROM)
        whole_image = 1;

   /* write image to flash memory */
    do 
    {
        sect_size = flash_get_sector_size(blk_start);
// NOTE: for memory problem in multiple PVC configuration, temporary get rid of kmalloc this 64K for now.
//        if ((pTempBuf = (char *)retriedKmalloc(sect_size)) == NULL)
//        {
//            printk("Failed to allocate memory with size: %d.  Reset the router...\n", sect_size);
//            kerSysMipsSoftReset();     // reset the board right away.
//        }
        // for whole image, no check on psi
        if (!whole_image && blk_start == fInfo.flash_persistent_start_blk)  // share the blk with psi
        {
            if (size > (sect_size - fInfo.flash_persistent_length))
            {
                printk("Image is too big\n");
                break;          // image is too big. Can not overwrite to nvram
            }
            if ((pTempBuf = (char *)retriedKmalloc(sect_size)) == NULL)
            {
               printk("Failed to allocate memory with size: %d.  Reset the router...\n", sect_size);
               kerSysMipsSoftReset();     // reset the board right away.
            }
            flash_read_buf(blk_start, 0, pTempBuf, sect_size);
            if (copy_from_user((void *)pTempBuf,(void *)string, size) != 0)
                break;  // failed ?
            flash_sector_erase_int(blk_start);     // erase blk before flash
            if (flash_write_buf(blk_start, 0, pTempBuf, sect_size) == sect_size) 
                size = 0;   // break out and say all is ok
            retriedKfree(pTempBuf);
            break;
        }
        
        flash_sector_erase_int(blk_start);     // erase blk before flash

        if (sect_size > size) 
        {
            if (size & 1) 
                size++;
            sect_size = size;
        }
        
        if ((i = flash_write_buf(blk_start, 0, string, sect_size)) != sect_size) {
            break;
        }
        blk_start++;
        string += sect_size;
        size -= sect_size; 
    } while (size > 0);

    if (whole_image)  
    {
        // If flashing a whole image, erase to end of flash.
        int total_blks = flash_get_numsectors();
        while( blk_start < total_blks )
        {
            flash_sector_erase_int(blk_start);
            blk_start++;
        }
    }
    if (pTempBuf)
        retriedKfree(pTempBuf);

    if( size == 0 ) 
        sts = 0;  // ok
    else  
        sts = blk_start;    // failed to flash this sector


    return sts;
}
/***********************************************************************
 * Function Name: kerSysErasePersistent
 * Description  : Erase the Psi storage section of flash memory.
 * Returns      : 1 -- ok, 0 -- fail
 ***********************************************************************/

/* This funtion ONLY called by user when reset to default button is pressed */


int kerSysErasePersistent(void)
{
    int sts = 1;
    char *tempStorage = NULL;
    int i, size=0, unit = fInfo.flash_persistent_length/8;
    
#if 0    

    printk("psi 0x%08x #blks 0x%08x len 0x%08x offset 0x%08x \n",fInfo.flash_persistent_start_blk,fInfo.flash_persistent_number_blk,fInfo.flash_persistent_length,fInfo.flash_persistent_blk_offset);
    printk("pad 0x%08x #blks 0x%08x len 0x%08x offset 0x%08x \n",fInfo.flash_scratch_pad_start_blk,fInfo.flash_scratch_pad_number_blk,fInfo.flash_scratch_pad_length,fInfo.flash_scratch_pad_blk_offset);
    printk("nvram 0x%08x #blks 0x%08x len 0x%08x offset 0x%08x \n",fInfo.flash_nvram_start_blk,fInfo.flash_nvram_number_blk,fInfo.flash_nvram_length,fInfo.flash_nvram_blk_offset);

    printk("\nErasing PSI storage section of flash memory....\n");
    for(i=0;i<7;i++) {
        tempStorage = retriedKmallocRESET(unit);
       // just write the whole buf with '0xff' to the flash
       if (!tempStorage) {
          sts = 0;
       } else {
          memset(tempStorage, 0xff, unit);
          if(kerSysPersistentSetRESET(tempStorage, unit, size) != 0)
              sts = 0;
          retriedKfree(tempStorage);
       }
       size += unit;
       if(!sts) {printk("Cannot alloc %d\n",i); return sts;}
       else printk("success in %d\n",i);
    }

    unit = fInfo.flash_persistent_length - size;
    printk("try to alloc %d\n",unit);
    tempStorage = retriedKmalloc(unit);
    if (!tempStorage) {
       sts = 0;
    } else {
    	printk("success in last\n");
       memset(tempStorage, 0xff, unit);
       if(kerSysPersistentSet(tempStorage, unit, size) != 0)
           sts = 0;
       retriedKfree(tempStorage);
    }
#else
   printk("\nErasing PSI....\n");
   if (kerSysPersistentSetRESET() != 0)
              sts = 0;
#endif
    printk("\nRebooting ADSL router.........");
    return sts;
}



/*******************************************************************************
 * SP functions
 *******************************************************************************/
// get sp data.  NOTE: memcpy work here -- not using copy_from/to_user
// return:
//  0 - ok
//  -1 - fail
int kerSysScratchPadGet(char *tokenId, char *tokBuf, int bufLen)
{
    PSP_HEADER pHead = NULL;
    PSP_TOKEN pToken = NULL;
    char *pBuf = NULL;
    char *pShareBuf = NULL;
    char *startPtr = NULL;
    char *endPtr = NULL;
    char *spEndPtr = NULL;
    int sts = -1;

    if (fInfo.flash_scratch_pad_length == 0)
        return sts;

    if (bufLen >= (fInfo.flash_scratch_pad_length - sizeof(SP_HEADER) - sizeof(SP_TOKEN))) 
    {
        printk("Exceed scratch pad space by %d\n", bufLen  - fInfo.flash_scratch_pad_length \
            - sizeof(SP_HEADER) - sizeof(SP_TOKEN));
        return sts;
    }

    if ((pShareBuf = getSharedBlks(fInfo.flash_scratch_pad_start_blk,
        (fInfo.flash_scratch_pad_start_blk + fInfo.flash_scratch_pad_number_blk))) == NULL)
        return sts;

    // pBuf points to SP buf
    pBuf = pShareBuf + fInfo.flash_scratch_pad_blk_offset;  

    pHead = (PSP_HEADER) pBuf;
    if (memcmp(pHead->SPMagicNum, MAGIC_NUMBER, MAGIC_NUM_LEN) != 0) 
    {
        printk("Scrap pad is not initialized.\n");
        return sts;
    }

    // search up to SPUsedLen for the token
    startPtr = pBuf + sizeof(SP_HEADER);
    endPtr = pBuf + pHead->SPUsedLen;
    spEndPtr = pBuf + SP_MAX_LEN;
    while (startPtr < endPtr && startPtr < spEndPtr)
    {
        pToken = (PSP_TOKEN) startPtr;
        if (strncmp(pToken->tokenName, tokenId, TOKEN_NAME_LEN) == 0)
        {
            memcpy(tokBuf, startPtr + sizeof(SP_TOKEN), bufLen);
            sts = 0;
            break;
        }
        // get next token
        startPtr += sizeof(SP_TOKEN) + pToken->tokenLen;
    }

    retriedKfree(pShareBuf);

    return sts;
}


// set sp.  NOTE: memcpy work here -- not using copy_from/to_user
// return:
//  0 - ok
//  -1 - fail
int kerSysScratchPadSet(char *tokenId, char *tokBuf, int bufLen)
{
    PSP_TOKEN pToken = NULL;
    PSP_HEADER pHead = NULL;
    char *pShareBuf = NULL;
    char *pBuf = NULL;
    SP_HEADER SPHead;
    SP_TOKEN SPToken;
    char *curPtr;
    int sts = -1;

    if (fInfo.flash_scratch_pad_length == 0)
        return sts;

    if (bufLen >= (fInfo.flash_scratch_pad_length - sizeof(SP_HEADER) - sizeof(SP_TOKEN))) 
    {
        printk("Exceed scratch pad space by %d\n", bufLen  - fInfo.flash_scratch_pad_length \
            - sizeof(SP_HEADER) - sizeof(SP_TOKEN));
        return sts;
    }

    if ((pShareBuf = getSharedBlks(fInfo.flash_scratch_pad_start_blk,
        (fInfo.flash_scratch_pad_start_blk + fInfo.flash_scratch_pad_number_blk))) == NULL)
        return sts;

    // pBuf points to SP buf
    pBuf = pShareBuf + fInfo.flash_scratch_pad_blk_offset;  
    pHead = (PSP_HEADER) pBuf;

    // form header info.  SPUsedLen later on...
    memset((char *)&SPHead, 0, sizeof(SP_HEADER));
    memcpy(SPHead.SPMagicNum, MAGIC_NUMBER, MAGIC_NUM_LEN);
    SPHead.SPVersion = SP_VERSION;

    // form token info.
    memset((char*)&SPToken, 0, sizeof(SP_TOKEN));
    strncpy(SPToken.tokenName, tokenId, TOKEN_NAME_LEN - 1);
    SPToken.tokenLen = bufLen;
    if (memcmp(pHead->SPMagicNum, MAGIC_NUMBER, MAGIC_NUM_LEN) != 0) 
    {
        // new sp, so just flash the token
        printk("No Scrap pad found.  Initialize scratch pad...\n");
        SPHead.SPUsedLen = sizeof(SP_HEADER) + sizeof(SP_TOKEN) + bufLen;
        memcpy(pBuf, (char *)&SPHead, sizeof(SP_HEADER));
        curPtr = pBuf + sizeof(SP_HEADER);
        memcpy(curPtr, (char *)&SPToken, sizeof(SP_TOKEN));
        curPtr += sizeof(SP_TOKEN);
        memcpy(curPtr, tokBuf, bufLen);
    }
    else  
    {
        // need search for the token, if exist with same size overwrite it. if sizes differ, 
        // move over the later token data over and put the new one at the end
        char *endPtr = pBuf + pHead->SPUsedLen;
        char *spEndPtr = pBuf + SP_MAX_LEN;
        curPtr = pBuf + sizeof(SP_HEADER);
        while (curPtr < endPtr && curPtr < spEndPtr)
        {
            pToken = (PSP_TOKEN) curPtr;
            if (strncmp(pToken->tokenName, tokenId, TOKEN_NAME_LEN) == 0)
            {
                if (pToken->tokenLen == bufLen) // overwirte it
                {
                    memcpy((curPtr+sizeof(SP_TOKEN)), tokBuf, bufLen);
                    break;
                }
                else // move later data over and put the new token at the end
                {
                    memcpy((curPtr+sizeof(SP_TOKEN)), tokBuf, bufLen);  // ~~~
                    break;
                }
            }
            else // not same token ~~~
            {
            }
            // get next token
            curPtr += sizeof(SP_TOKEN) + pToken->tokenLen;
        } // end while
        SPHead.SPUsedLen = sizeof(SP_HEADER) + sizeof(SP_TOKEN) + bufLen; // ~~~
        if (SPHead.SPUsedLen > SP_MAX_LEN)
        {
            printk("No more Scratch pad space left! Over limit by %d bytes\n", SPHead.SPUsedLen - SP_MAX_LEN);
            return sts;
        }

    } // else if not new sp

    sts = setSharedBlks(fInfo.flash_scratch_pad_start_blk, 
        (fInfo.flash_scratch_pad_number_blk + fInfo.flash_scratch_pad_start_blk), pShareBuf);
    
    retriedKfree(pShareBuf);

    return sts;

    
}

/*This function is copied from V2.10.1 to fix the bug SDRAM change from 16M to 8M 
  during upgrade include CFE */
/***********************************************************************
 * Function Name: kerSysMemoryTypeSet
 * Description  : set the memory type 4 bytes
 * Returns      : 1 -- ok, 0 -- fail
 ***********************************************************************/
int kerSysMemoryTypeSet(int flash_start_addr, char *string, int size)
{
    int sect_size;
    int blk_start;
    char *pTempBuf;

    blk_start = flash_get_blk(flash_start_addr);
    sect_size = flash_get_sector_size(blk_start);
    if ((pTempBuf = (char *) retriedKmalloc(sect_size)) == NULL)
    {
        printk("Failed to allocate memory with size: %d\n", sect_size);
        return 0;
    }
    flash_read_buf(blk_start, 0, pTempBuf, sect_size);
    memcpy(pTempBuf+SDRAM_TYPE_ADDRESS_OFFSET, string, size);
    flash_sector_erase_int(blk_start);     // erase blk before flash
//    flash_ub(0);
    if (flash_write_buf(blk_start, 0, pTempBuf, sect_size) != sect_size)
        printk("Failed to flash the memory type\n");
//    flash_reset_ub();
    return 1;
}



int kerSysFlashSizeGet(void)
{
   return flash_get_total_size();
}

