/*
<:copyright-gpl 
 Copyright 2002 Broadcom Corp. All Rights Reserved. 
 
 This program is free software; you can distribute it and/or modify it 
 under the terms of the GNU General Public License (Version 2) as 
 published by the Free Software Foundation. 
 
 This program is distributed in the hope it will be useful, but WITHOUT 
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
 FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
 for more details. 
 
 You should have received a copy of the GNU General Public License along 
 with this program; if not, write to the Free Software Foundation, Inc., 
 59 Temple Place - Suite 330, Boston MA 02111-1307, USA. 
:>
*/
/***************************************************************************
 * File Name  : board.c
 *
 * Description: This file contains Linux character device driver entry 
 *              for the board related ioctl calls: flash, get free kernel
 *              page and dump kernel memory, etc.
 *
 * Created on :  2/20/2002  seanl:  use cfiflash.c, cfliflash.h (AMD specific)
 *
 ***************************************************************************/


/* Includes. */
#include <linux/version.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/capability.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/pagemap.h>
#include <asm/uaccess.h>
#include <linux/wait.h>
#include <linux/poll.h>
#include <linux/sched.h>
#include <linux/list.h>
#include <linux/if.h>

#include <bcm_map_part.h>
#include <board.h>
#include <bcmTag.h>
#include "boardparms.h"
#include "cfiflash.h"
#include "bcm_intr.h"
#include "board.h"
#include "bcm_map_part.h"

#ifdef NNN_POT
extern unsigned int kerPOTSet(unsigned int);
static unsigned int pot=0;
static int tcnt=0;
#endif

/* Typedefs. */
#if defined (NON_CONSECUTIVE_MAC)
// used to be the last octet. Now changed to the first 5 bits of the the forth octet
// to reduced the duplicated MAC addresses.
#define CHANGED_OCTET   3
#define SHIFT_BITS      3
#else
#define CHANGED_OCTET   1
#define SHIFT_BITS      0
#endif

#if defined (WIRELESS)
#define SES_BTN_PRESSED 0x00000001
#define SES_EVENTS      SES_BTN_PRESSED /*OR all values if any*/
#define SES_LED_OFF     0
#define SES_LED_ON      1
#define SES_LED_BLINK   2
#endif

#define FLASHLAYOUT_W_DUALIMAGE	4
typedef struct
{
    unsigned long ulId;
    char chInUse;
    char chReserved[3];
} MAC_ADDR_INFO, *PMAC_ADDR_INFO;

typedef struct
{
    unsigned long ulSdramSize;
    unsigned long ulPsiSize;
    unsigned long ulNumMacAddrs;
    unsigned long ucaBaseMacAddr[NVRAM_MAC_ADDRESS_LEN];
    MAC_ADDR_INFO MacAddrs[1];
} NVRAM_INFO, *PNVRAM_INFO;

typedef struct
{
    unsigned long eventmask;    
} BOARD_IOC, *PBOARD_IOC;


/*Dyinggasp callback*/
typedef void (*cb_dgasp_t)(void *arg);
typedef struct _CB_DGASP__LIST
{
    struct list_head list;
    char name[IFNAMSIZ];
    cb_dgasp_t cb_dgasp_fn;
    void *context;
}CB_DGASP_LIST , *PCB_DGASP_LIST;


static LED_MAP_PAIR LedMapping[] =
{   // led name     Initial state       physical pin (ledMask)
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0},
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0},
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0},
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0},
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0},
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0}, 
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0}, 
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0}, 
    {kLedEnd,       kLedStateOff,       0, 0, 0, 0} // NOTE: kLedEnd has to be at the end.
};

/* Externs. */
extern struct file fastcall *fget_light(unsigned int fd, int *fput_needed);
extern unsigned int nr_free_pages (void);
extern const char *get_system_type(void);
extern void kerSysFlashInit(void);
extern unsigned long get_nvram_start_addr(void);
extern unsigned long get_scratch_pad_start_addr(void);
extern unsigned long getMemorySize(void);
extern void __init boardLedInit(PLED_MAP_PAIR);
extern void boardLedCtrl(BOARD_LED_NAME, BOARD_LED_STATE);
extern void kerSysLedRegisterHandler( BOARD_LED_NAME ledName,
    HANDLE_LED_FUNC ledHwFunc, int ledFailType );

/* Prototypes. */
void __init InitNvramInfo( void );
static int board_open( struct inode *inode, struct file *filp );
static int board_ioctl( struct inode *inode, struct file *flip, unsigned int command, unsigned long arg );
static ssize_t board_read(struct file *filp,  char __user *buffer, size_t count, loff_t *ppos); 
static unsigned int board_poll(struct file *filp, struct poll_table_struct *wait);
static int board_release(struct inode *inode, struct file *filp);                        

static BOARD_IOC* borad_ioc_alloc(void);
static void borad_ioc_free(BOARD_IOC* board_ioc);

/* DyingGasp function prototype */
static void __init kerSysDyingGaspMapIntr(void);
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static irqreturn_t kerSysDyingGaspIsr(int irq, void * dev_id, struct pt_regs * regs);
#else
static unsigned int kerSysDyingGaspIsr(void);
#endif
static void __init kerSysInitDyingGaspHandler( void );
static void __exit kerSysDeinitDyingGaspHandler( void );
/* -DyingGasp function prototype - */


#if defined (WIRELESS)
static irqreturn_t sesBtn_isr(int irq, void *dev_id, struct pt_regs *ptregs);
static void __init sesBtn_mapGpio(void);
static void __init sesBtn_mapIntr(int context);
static unsigned int sesBtn_poll(struct file *file, struct poll_table_struct *wait);
static ssize_t sesBtn_read(struct file *file,  char __user *buffer, size_t count, loff_t *ppos);
static void __init sesLed_mapGpio(void);
static void sesLed_ctrl(int action);
static void __init ses_board_init(void);
static void __exit ses_board_deinit(void);
#endif

static PNVRAM_INFO g_pNvramInfo = NULL;
static char g_pSystemBoardId[NVRAM_BOARD_ID_STRING_LEN];
static int g_ledInitialized = 0;
static wait_queue_head_t g_board_wait_queue;
static CB_DGASP_LIST *g_cb_dgasp_list_head = NULL;

static int g_wakeup_monitor = 0;
static struct file *g_monitor_file = NULL;
static struct task_struct *g_monitor_task = NULL;
static unsigned int (*g_orig_fop_poll)
    (struct file *, struct poll_table_struct *) = NULL;

static struct file_operations board_fops =
{
  open:       board_open,
  ioctl:      board_ioctl,
  poll:       board_poll,
  read:       board_read,
  release:    board_release,
};

uint32 board_major = 0;

#if defined (WIRELESS)
static unsigned short sesBtn_irq = BP_NOT_DEFINED;
static unsigned short sesBtn_gpio = BP_NOT_DEFINED;
static unsigned short sesLed_gpio = BP_NOT_DEFINED;
#endif

#define NVRAM_SPROM_TABLE_OFFSET 740
#define NVRAM_SPROM_TABLE_SIZE 128

//swda add,02/04/2005
#ifdef SERIAL_NUMBER
#define SN_OFFSET	680
#define SN_SIZE	31
static char g_Serial_No[SN_SIZE];
#endif

#ifdef TTT_SERIAL_NUMBER
#define ST716_SN_OFFSET	725
#define ST716_SN_SIZE	11
static char g_Serial_No[ST716_SN_SIZE];
#endif
//swda add end

#ifdef	DEFAULT_WLAN_WEP128
#define	WEP128KEY_OFFSET	712
#define	WEP128KEY_SIZE		13
static char g_WEP128bit_key[WEP128KEY_SIZE];
#endif

#ifdef WLAN_ENABLE_CTRL_BUTTON
#define GET_WLAN_ENABLE_GPIO_STATUS	0
#define GET_BOOT_COMPLETE_FLAG	1
#define GPIO_WLAN_ENABLE_STATE	0x40000		//GPIO 18	
#endif
int is_update_flash = 0;
int IsMultiConfig = 0;
//swda add end
#if defined(MODULE)
int init_module(void)
{
    return( brcm_board_init() );              
}

void cleanup_module(void)
{
    if (MOD_IN_USE)
        printk("brcm flash: cleanup_module failed because module is in use\n");
    else
        brcm_board_cleanup();
}
#endif //MODULE 

/*Michael: Add following code for hold reset button */

#define K100MS              	((100 * HZ) / 1000)     // ~100 ms
#define DURATION		600                    //about 3 seconds
// global variables:
struct timer_list gMonitorTimer;
int gMonitorTimerOn = FALSE;
void MonitorTimerExpire(void);
#if VOXXXLOAD
struct timer_list gMonitorTimerPSTN;
int gMonitorTimerOnPSTN = FALSE;
void MonitorTimerExpirePSTN(void);
struct timer_list gMonitorTimerOFFHOOK;
int gMonitorTimerOnOFFHOOK = FALSE;
void MonitorTimerExpireOFFHOOK(void);
int PSTNComingCall=0;
int PSTNphone2OFFHOOK=0;
int pstn_spin_lock;
int offhook_spin_lock;
unsigned long NoActionTimePSTN=0;
unsigned long NoActionTimeOFFHOOK=0;
#endif

#if ODM_AUTO_PROVISION_LAN
int autoprovisionlan=0;
#endif

//swda add,09/12/2005
int boot_complete_flag = 0;
//swda add end

unsigned long PressTime = DURATION;

void MonitorTimerStart(void)
{
#if defined(CONFIG_BCM96345)||defined(CONFIG_BCM96348)||defined(CONFIG_BCM96338)
    if (gMonitorTimerOn)
        return;

    init_timer(&gMonitorTimer);
    gMonitorTimer.function = (void*)MonitorTimerExpire;
    gMonitorTimer.expires = jiffies + K100MS;        // timer expires in ~100ms
    add_timer(&gMonitorTimer);
    gMonitorTimerOn = TRUE;
#endif
}

void MonitorTimerExpire(void)
{
#if defined(CONFIG_BCM96345)
  #define GPIO_PRESS_AND_HOLD_RESET   0x2000
    //press the GPIO_PRESS_AND_HOLD_RESET button
    if (!(GPIO->GPIOio & (unsigned short)GPIO_PRESS_AND_HOLD_RESET)) {
#elif defined(CONFIG_BCM96348)
  #define GPIO_PRESS_AND_HOLD_RESET   0x80000000	//GPIO 31
    //press the GPIO_PRESS_AND_HOLD_RESET button
    if (!(GPIO->GPIOio & (unsigned int)GPIO_PRESS_AND_HOLD_RESET)) {
#else	//CONFIG_BCM96338
#ifdef WWWGPIO
	//printk("PERF->ExtIrqCfg = %X\n",PERF->ExtIrqCfg);
	if ( !(PERF->ExtIrqCfg & 0x00000020) ) {
#else
  #define GPIO_PRESS_AND_HOLD_RESET   0x80	//GPIO 7
    //press the GPIO_PRESS_AND_HOLD_RESET button
    if (!(GPIO->GPIOio & (unsigned int)GPIO_PRESS_AND_HOLD_RESET)) {
#endif
#endif
        if(!PressTime) PressTime = jiffies;
        printk("\rHold Reset button for %u seconds", (jiffies - PressTime)/200);
        if(jiffies - PressTime > DURATION)  { //reset to factory default setting
#if ODM_AUTO_PROVISION_LAN
			if ( boot_complete_flag == 1 ) {
				printk("\nReset to factory default setting...\n");
				kerSysErasePersistent();
				kerSysMipsSoftReset();
			} else {
            	printk("\nRun auto provision on lan mode...\n");
            	kerSysLedCtrl( kLedDiag, kLedStateOff );//turn off the Diag LED(red)
            	kerSysLedCtrl( kLedPower, kLedStateFastBlinkContinues );//Power LED blinking(green)
            	autoprovisionlan=1;
        	}
#else
            printk("\nReset to factory default setting...\n");
            kerSysErasePersistent();
            //flash_reset();
            //setDefaultBootline();
            kerSysMipsSoftReset();
#endif
            gMonitorTimerOn = TRUE;
            return;
        }
    } else {
        PressTime = 0;
    }
#ifdef NNN_POT    
    //if (tcnt%10 == 0) printk(".");
    if (pot<=60*24*3)
     if (tcnt++ == 600) {
           pot=kerPOTSet(1);tcnt=0;
    }
#endif    
    
    gMonitorTimerOn = FALSE;
    MonitorTimerStart();
}

#if VOXXXLOAD
void MonitorTimerStartPSTN(void)
{
#if defined(CONFIG_BCM96345)||defined(CONFIG_BCM96348)
    if (gMonitorTimerOnPSTN)
        return;

    init_timer(&gMonitorTimerPSTN);
    gMonitorTimerPSTN.function = (void*)MonitorTimerExpirePSTN;
    gMonitorTimerPSTN.expires = jiffies + K100MS;        // timer expires in ~100ms
    add_timer(&gMonitorTimerPSTN);
    gMonitorTimerOnPSTN = TRUE;
#endif
}

void MonitorTimerExpirePSTN(void)
{
#if defined(CONFIG_BCM96345)
  #define GPIO_PSTN   0x0200
    if (GPIO->GPIOio & (unsigned short)GPIO_PSTN){
#else
  #define GPIO_PSTN   0x800000
    if (GPIO->GPIOio & (unsigned int)GPIO_PSTN){
#endif
       if(!PSTNComingCall) {
           spin_lock(&pstn_spin_lock);
           //printk("**PSTN incoming call\n");
           PSTNComingCall=1;
           spin_unlock(&pstn_spin_lock);
       }
       NoActionTimePSTN=0;
       //printk("                                  **PSTN incoming call\n");
    } else {
       NoActionTimePSTN++;
       if(NoActionTimePSTN>50) {
             NoActionTimePSTN=0;
             if(PSTNComingCall) {
               spin_lock(&pstn_spin_lock);
               //printk("**No PSTN incoming call\n");
               PSTNComingCall=0;
               spin_unlock(&pstn_spin_lock);
             }
        }
        //printk("                                            *NO PSTN incoming call signal\n");
     }
    gMonitorTimerOnPSTN = FALSE;
    MonitorTimerStartPSTN();
}

void MonitorTimerStartOFFHOOK(void)
{
    if (gMonitorTimerOnOFFHOOK)
        return;

    init_timer(&gMonitorTimerOFFHOOK);
    gMonitorTimerOFFHOOK.function = (void*)MonitorTimerExpireOFFHOOK;
    gMonitorTimerOFFHOOK.expires = jiffies + K100MS;        // timer expires in ~100ms
    add_timer(&gMonitorTimerOFFHOOK);
    gMonitorTimerOnOFFHOOK = TRUE;
}

void MonitorTimerExpireOFFHOOK(void)
{
#if defined(CONFIG_BCM96345)
  #define GPIO_OFFHOOK   0x0002
    if (!(GPIO->GPIOio & (unsigned short)GPIO_OFFHOOK)){
#else
  #define GPIO_OFFHOOK   0x400000
    if (!(GPIO->GPIOio & (unsigned int)GPIO_OFFHOOK)){
#endif 
       if((!PSTNphone2OFFHOOK)&&(!PSTNComingCall)) {
           spin_lock(&offhook_spin_lock);
          // printk("**PSTN OFFHOOK\n");
           PSTNphone2OFFHOOK=1;
           spin_unlock(&offhook_spin_lock);
       }
       NoActionTimeOFFHOOK=0;
    //printk("**PSTN OFFHOOK\n");
    } else {
       NoActionTimeOFFHOOK++;
       if(NoActionTimeOFFHOOK>20) {
             NoActionTimeOFFHOOK=0;
             if(PSTNphone2OFFHOOK) {
               spin_lock(&offhook_spin_lock);
               //printk("**No PSTN OFFHOOK\n");
               PSTNphone2OFFHOOK=0;
               spin_unlock(&offhook_spin_lock);
             }
        }
     //printk("              **PSTN ONHOOK\n");
     }
    gMonitorTimerOnOFFHOOK = FALSE;
    MonitorTimerStartOFFHOOK();
}


#endif
/*End of adding*/

/* swda add begin */ 
#if defined(CONFIG_BCM96348)
struct timer_list gLanSwitchTimer;
void LanSwitchTimerExpire(void)
{
	GPIO->GPIODir |= 0x0020; //turn on the direction bit in case was turned off by some one
	//pull GPIO5 high
	GPIO->GPIOio |= 0x0020;
}
void LanSwitchReset(void)
{ 
	GPIO->GPIODir |= 0x0020; //turn on the direction bit in case was turned off by some one
	//pull GPIO5 low first
	GPIO->GPIOio &= 0xffdf;
	//start timer
    init_timer(&gLanSwitchTimer);
    gLanSwitchTimer.function = (void*)LanSwitchTimerExpire;
    gLanSwitchTimer.expires = jiffies + 50*HZ/1000;        // timer expires in ~50ms
    add_timer(&gLanSwitchTimer);	
}
#endif
/* swda add end   */


//swda add,11/02/2005
#ifdef	AAA_TEST_LED
struct timer_list gAAATestLedTimer;
static int gAAATestLedTimerRunning = 0;

void AAATestLedTimerExpireFunc(void)
{
	kerSysLedCtrl( kLedPPP, kLedStateOff );
	gAAATestLedTimerRunning = 0;	
}
void AAATestLedTimerStart(void)
{
    init_timer(&gAAATestLedTimer);
    gAAATestLedTimer.function = (void*)AAATestLedTimerExpireFunc;
    gAAATestLedTimer.expires = jiffies + 30*HZ;        // timer expires in ~30s
    add_timer(&gAAATestLedTimer);
    gAAATestLedTimerRunning = 1;	
}
void AAATestLedTimerDelete(void)
{
	if (gAAATestLedTimerRunning ) {
		del_timer(&gAAATestLedTimer);
		gAAATestLedTimerRunning = 0;
	}
}
#endif
//swda add end

static int __init brcm_board_init( void )
{
    typedef int (*BP_LED_FUNC) (unsigned short *);
    static struct BpLedInformation
    {
        BOARD_LED_NAME ledName;
        BP_LED_FUNC bpFunc;
        BP_LED_FUNC bpFuncFail;
    } bpLedInfo[] =
    {{kLedAdsl, BpGetAdslLedGpio, BpGetAdslFailLedGpio},
     {kLedWireless, BpGetWirelessLedGpio, NULL},
     {kLedUsb, BpGetUsbLedGpio, NULL},
     {kLedHpna, BpGetHpnaLedGpio, NULL},
     {kLedWanData, BpGetWanDataLedGpio, NULL},
     {kLedPPP, BpGetPppLedGpio, BpGetPppFailLedGpio},
     {kLedVoip, BpGetVoipLedGpio, NULL},
     {kLedSes, BpGetWirelessSesLedGpio, NULL},     
     {kLedDiag, BpGetDiagLedGpio, NULL},
     {kPSTN, BpGetPSTNGpio, NULL},
     {kLedPower, BpGetBootloaderPowerOnLedGpio, NULL},
     {kLedEnd, NULL, NULL}
    };

    int ret;
        
    ret = register_chrdev(BOARD_DRV_MAJOR, "bcrmboard", &board_fops );
    if (ret < 0)
        printk( "brcm_board_init(major %d): fail to register device.\n",BOARD_DRV_MAJOR);
    else {
        PLED_MAP_PAIR pLedMap = LedMapping;
        unsigned short gpio;
        struct BpLedInformation *pInfo;

        printk("brcmboard: brcm_board_init entry\n");
        board_major = BOARD_DRV_MAJOR;
        InitNvramInfo();

        for( pInfo = bpLedInfo; pInfo->ledName != kLedEnd; pInfo++ )
        {
            if( pInfo->bpFunc && (*pInfo->bpFunc) (&gpio) == BP_SUCCESS )
            {
                pLedMap->ledName = pInfo->ledName;
                pLedMap->ledMask = GPIO_NUM_TO_MASK(gpio);
                pLedMap->ledActiveLow = (gpio & BP_ACTIVE_LOW) ? 1 : 0;
                
                //swda add begin
                if (pInfo->ledName == kLedPower)
                	pLedMap->ledInitState = kLedStateOff;
                if (pInfo->ledName == kLedDiag)
                	pLedMap->ledInitState = kLedStateOn;
                //swda add end
            }
            if( pInfo->bpFuncFail && (*pInfo->bpFuncFail) (&gpio) == BP_SUCCESS )
            {
                pLedMap->ledName = pInfo->ledName;
                pLedMap->ledMaskFail = GPIO_NUM_TO_MASK(gpio);
                pLedMap->ledActiveLowFail = (gpio & BP_ACTIVE_LOW) ? 1 : 0;
            }
            if( pLedMap->ledName != kLedEnd )
                pLedMap++;
        }

        init_waitqueue_head(&g_board_wait_queue);
#if defined (WIRELESS)
        ses_board_init();
#endif        
        kerSysInitDyingGaspHandler();
        kerSysDyingGaspMapIntr();

        boardLedInit(LedMapping);
        g_ledInitialized = 1;
    }
    //Start the timer to detect the GPIO_PRESS_AND_HOLD_RESET
#ifdef CONFIG_BCM96338	//swda add
	GPIO->GPIODir &= 0xFFFFFF7F;//GPIO 7 used as reset to default pin, set GPIO 7 as input
	 
	GPIO->GPIODir |= 0x0004;	//set GPIO2 as output pin,so that we can control GPIO4 value 
	GPIO->GPIOio |=	0x0004;		//pull GPIO2 high, so that we can control ppp led
#endif

//swda add,02/01/2004
#ifdef CONFIG_BCM96348
	GPIO->GPIODir |= 0x0010;	//set GPIO4 as output pin,so that we can control GPIO4 value 
	GPIO->GPIOio |=	0x0010;		//pull GPIO4 high, so that we can control ppp led via GPIO3
#endif
//swda add end
#ifdef NNN_POT
    pot=kerPOTSet(0);
#endif

    MonitorTimerStart();
#if VOXXXLOAD
    MonitorTimerStartPSTN();
    MonitorTimerStartOFFHOOK();
#endif
	
//swda add,11/01/2005
#ifdef WLAN_ENABLE_CTRL_BUTTON
	GPIO->GPIODir &= (unsigned int)(~GPIO_WLAN_ENABLE_STATE);//GPIO 18 used to inform wlan to turn off, so set GPIO 18 as input pin
#endif
//swda add end

//swda add begin: Lan Switch Reset(pull GPIO5 low for at least 50ms)	
#if defined(CONFIG_BCM96348)
	LanSwitchReset();
#endif
//swda add end

    return ret;
} 

void __init InitNvramInfo( void )
{
    PNVRAM_DATA pNvramData = (PNVRAM_DATA) get_nvram_start_addr();


/************************************/
//Michael: BRCM orig    unsigned long ulNumMacAddrs = pNvramData->ulNumMacAddrs;

   unsigned long ulNumMacAddrs = 22;  /*We skip what the user set up.  change it max 12, each PVC use one but we change first byte*/

    //read the BoardId string from NVRam
    memset(g_pSystemBoardId, 0, NVRAM_BOARD_ID_STRING_LEN);
    memcpy(g_pSystemBoardId, pNvramData->szBoardId, NVRAM_BOARD_ID_STRING_LEN);

    if( ulNumMacAddrs > 0 && ulNumMacAddrs <= NVRAM_MAC_COUNT_MAX )
    {
        unsigned long ulNvramInfoSize =
            sizeof(NVRAM_INFO) + ((sizeof(MAC_ADDR_INFO) - 1) * ulNumMacAddrs);

        g_pNvramInfo = (PNVRAM_INFO) kmalloc( ulNvramInfoSize, GFP_KERNEL );

        if( g_pNvramInfo )
        {
            unsigned long ulPsiSize;
            if( BpGetPsiSize( &ulPsiSize ) != BP_SUCCESS )
                ulPsiSize = NVRAM_PSI_DEFAULT;
            memset( g_pNvramInfo, 0x00, ulNvramInfoSize );
            g_pNvramInfo->ulPsiSize = ulPsiSize * 1024;
            //Michael: brcm orig   g_pNvramInfo->ulNumMacAddrs = pNvramData->ulNumMacAddrs;
            g_pNvramInfo->ulNumMacAddrs = 22;
            memcpy( g_pNvramInfo->ucaBaseMacAddr, pNvramData->ucaBaseMacAddr,
                NVRAM_MAC_ADDRESS_LEN );
            g_pNvramInfo->ulSdramSize = getMemorySize();
        }
        else
            printk("ERROR - Could not allocate memory for NVRAM data\n");
    }
    else
        printk("ERROR - Invalid number of MAC addresses (%ld) is configured.\n",
            ulNumMacAddrs);

//swda add,04/08/2005
#ifdef SERIAL_NUMBER
	kerSysNvRamGet(g_Serial_No, SN_SIZE, SN_OFFSET);
	//check if valid serial number
	if ( (g_Serial_No[0]<0x21 || g_Serial_No[0]>0x7e) && g_Serial_No[0]!=0 ) {
		memset(g_Serial_No, 0, SN_SIZE);
	}
	printk("Serial number:%s\n",g_Serial_No);
#endif
#ifdef	DEFAULT_WLAN_WEP128
	kerSysNvRamGet(g_WEP128bit_key,WEP128KEY_SIZE,WEP128KEY_OFFSET);
	{	//check if valid ASCII string
		int key_index,valid_key;
		
		valid_key = 1;
		for (key_index=0;key_index<WEP128KEY_SIZE;key_index++) {
			//printable ASCII character 0x21~0x7E
			if ( g_WEP128bit_key[key_index]<0x21 || g_WEP128bit_key[key_index]>0x7e ) {
				valid_key = 0;
				break;
			}	
		}
		if (!valid_key)	
			strcpy(g_WEP128bit_key,"0000000000000");	
	}
#endif
//swda add end
}

void __exit brcm_board_cleanup( void )
{
    printk("brcm_board_cleanup()\n");

    if (board_major != -1) 
    {
#if defined (WIRELESS)    	
    	ses_board_deinit();
#endif    	
        kerSysDeinitDyingGaspHandler();
        unregister_chrdev(board_major, "board_ioctl");
    }
} 

static BOARD_IOC* borad_ioc_alloc(void)
{
    BOARD_IOC *board_ioc =NULL;
    board_ioc = (BOARD_IOC*) kmalloc( sizeof(BOARD_IOC) , GFP_KERNEL );
    if(board_ioc)
    {
        memset(board_ioc, 0, sizeof(BOARD_IOC));
    }
    return board_ioc;
}

static void borad_ioc_free(BOARD_IOC* board_ioc)
{
    if(board_ioc)
    {
        kfree(board_ioc);
    }	
}


static int board_open( struct inode *inode, struct file *filp )
{
    filp->private_data = borad_ioc_alloc();

    if (filp->private_data == NULL)
        return -ENOMEM;
            
    return( 0 );
} 

static int board_release(struct inode *inode, struct file *filp)
{
    BOARD_IOC *board_ioc = filp->private_data;
    
    wait_event_interruptible(g_board_wait_queue, 1);    
    borad_ioc_free(board_ioc);

    return( 0 );
} 


static unsigned int board_poll(struct file *filp, struct poll_table_struct *wait)
{
    unsigned int mask = 0;
#if defined (WIRELESS)        	
    BOARD_IOC *board_ioc = filp->private_data;    	
#endif
    	
    poll_wait(filp, &g_board_wait_queue, wait);
#if defined (WIRELESS)        	
    if(board_ioc->eventmask & SES_EVENTS){
        mask |= sesBtn_poll(filp, wait);
    }			
#endif    

    return mask;
}


static ssize_t board_read(struct file *filp,  char __user *buffer, size_t count, loff_t *ppos)
{
#if defined (WIRELESS)    
    BOARD_IOC *board_ioc = filp->private_data;
    if(board_ioc->eventmask & SES_EVENTS){
    	return sesBtn_read(filp, buffer, count, ppos);
    }
#endif    
    return 0;
}

//**************************************************************************************
// Utitlities for dump memory, free kernel pages, mips soft reset, etc.
//**************************************************************************************

/***********************************************************************
 * Function Name: dumpaddr
 * Description  : Display a hex dump of the specified address.
 ***********************************************************************/
void dumpaddr( unsigned char *pAddr, int nLen )
{
    static char szHexChars[] = "0123456789abcdef";
    char szLine[80];
    char *p = szLine;
    unsigned char ch, *q;
    int i, j;
    unsigned long ul;

    while( nLen > 0 )
    {
        sprintf( szLine, "%8.8lx: ", (unsigned long) pAddr );
        p = szLine + strlen(szLine);

        for(i = 0; i < 16 && nLen > 0; i += sizeof(long), nLen -= sizeof(long))
        {
            ul = *(unsigned long *) &pAddr[i];
            q = (unsigned char *) &ul;
            for( j = 0; j < sizeof(long); j++ )
            {
                *p++ = szHexChars[q[j] >> 4];
                *p++ = szHexChars[q[j] & 0x0f];
                *p++ = ' ';
            }
        }

        for( j = 0; j < 16 - i; j++ )
            *p++ = ' ', *p++ = ' ', *p++ = ' ';

        *p++ = ' ', *p++ = ' ', *p++ = ' ';

        for( j = 0; j < i; j++ )
        {
            ch = pAddr[j];
            *p++ = (ch > ' ' && ch < '~') ? ch : '.';
        }

        *p++ = '\0';
        printk( "%s\r\n", szLine );

        pAddr += i;
    }
    printk( "\r\n" );
} /* dumpaddr */


void kerSysMipsSoftReset(void)
{
#if defined(CONFIG_BCM96348)
    if (PERF->RevID == 0x634800A1) {
        typedef void (*FNPTR) (void);
        FNPTR bootaddr = (FNPTR) FLASH_BASE;
        int i;

        /* Disable interrupts. */
        cli();

        /* Reset all blocks. */
        PERF->BlockSoftReset &= ~BSR_ALL_BLOCKS;
        for( i = 0; i < 1000000; i++ )
            ;
        PERF->BlockSoftReset |= BSR_ALL_BLOCKS;
        /* Jump to the power on address. */
        (*bootaddr) ();
    }
    else
        PERF->pll_control |= SOFT_RESET;    // soft reset mips
#else
    PERF->pll_control |= SOFT_RESET;    // soft reset mips
#endif
}


int kerSysGetMacAddress( unsigned char *pucaMacAddr, unsigned long ulId )
{
    int nRet = 0;
    PMAC_ADDR_INFO pMai = NULL;
    PMAC_ADDR_INFO pMaiFreeNoId = NULL;
    PMAC_ADDR_INFO pMaiFreeId = NULL;
    unsigned long i = 0, ulIdxNoId = 0, ulIdxId = 0, shiftedIdx = 0;

    for( i = 0, pMai = g_pNvramInfo->MacAddrs; i < g_pNvramInfo->ulNumMacAddrs;
        i++, pMai++ )
    {
        if( ulId == pMai->ulId || ulId == MAC_ADDRESS_ANY )
        {
            /* This MAC address has been used by the caller in the past. */
            memcpy( pucaMacAddr, g_pNvramInfo->ucaBaseMacAddr,
                NVRAM_MAC_ADDRESS_LEN );
            shiftedIdx = i;
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - CHANGED_OCTET] += (shiftedIdx << SHIFT_BITS);
            pMai->chInUse = 1;
            pMaiFreeNoId = pMaiFreeId = NULL;
            break;
        }
        else
            if( pMai->chInUse == 0 )
            {
                if( pMai->ulId == 0 && pMaiFreeNoId == NULL )
                {
                    /* This is an available MAC address that has never been
                     * used.
                     */
                    pMaiFreeNoId = pMai;
                    ulIdxNoId = i;
                }
                else
                    if( pMai->ulId != 0 && pMaiFreeId == NULL )
                    {
                        /* This is an available MAC address that has been used
                         * before.  Use addresses that have never been used
                         * first, before using this one.
                         */
                        pMaiFreeId = pMai;
                        ulIdxId = i;
                    }
            }
    }

    if( pMaiFreeNoId || pMaiFreeId )
    {
        /* An available MAC address was found. */
        memcpy(pucaMacAddr, g_pNvramInfo->ucaBaseMacAddr,NVRAM_MAC_ADDRESS_LEN);
        if( pMaiFreeNoId )
        {
           //Brcm orig           shiftedIdx = ulIdxNoId;
           //Brcm orig           pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - CHANGED_OCTET] += (shiftedIdx << SHIFT_BITS);
#ifdef BRCM_DRIVER_USB
        if(ulIdxNoId>2)
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 6] += (ulIdxNoId-3)*8+2;
	else {
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1] += ulIdxNoId;

/*Michael: 03/01/2004   eth: 0000ff-->usb is 000100 and 000101
                        eth: 0000fe-->usb is 0000ff and 000100
*/
            if (ulIdxNoId == 1){//USB client
                 if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]&0xff) == 0x00){
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2] += 1;
                      if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2]&0xff) == 0x00)
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 3] += 1;
                }
            } 
            if (ulIdxNoId == 2){//USB Host 
                 if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]&0xff) == 0x00){
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2] += 1;
                      if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2]&0xff) == 0x00)
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 3] += 1;
                }
                 if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]&0xff) == 0x01){
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2] += 1;
                      if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2]&0xff) == 0x00)
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 3] += 1;
                }
            }
        } 
#else

        if(ulIdxNoId>0)
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 6] += (ulIdxNoId-1)*8+2;
        else
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1] += ulIdxNoId;

#endif
            pMaiFreeNoId->ulId = ulId;
            pMaiFreeNoId->chInUse = 1;
        }
        else
        {
           //Brcm orig           shiftedIdx = ulIdxId;
          //Brcm orig            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - CHANGED_OCTET] += (shiftedIdx << SHIFT_BITS);
#ifdef BRCM_DRIVER_USB
        if(ulIdxNoId>2)
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 6] += (ulIdxNoId-3)*8+2;
        else {
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1] += ulIdxNoId;

/*Michael: 03/01/2004   eth: 0000ff-->usb is 000100 and 000101
                        eth: 0000fe-->usb is 0000ff and 000100
*/
            if (ulIdxNoId == 1){//USB client
                 if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]&0xff) == 0x00){
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2] += 1;
                      if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2]&0xff) == 0x00)
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 3] += 1;
                }
            }
            if (ulIdxNoId == 2){//USB Host
                 if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]&0xff) == 0x00){
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2] += 1;
                      if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2]&0xff) == 0x00)
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 3] += 1;
                }
                 if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]&0xff) == 0x01){
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2] += 1;
                      if( (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 2]&0xff) == 0x00)
                      pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 3] += 1;
                }
            }
        }
#else
        if(ulIdxNoId>0)
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 6] += (ulIdxNoId-1)*8+2;
        else
            pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - 1] += ulIdxNoId;
#endif 
            pMaiFreeId->ulId = ulId;
            pMaiFreeId->chInUse = 1;
        }
    }
    else
        if( i == g_pNvramInfo->ulNumMacAddrs )
            nRet = -EADDRNOTAVAIL;

    return( nRet );
} /* kerSysGetMacAddr */

int kerSysReleaseMacAddress( unsigned char *pucaMacAddr )
{
    int nRet = -EINVAL;
    unsigned long ulIdx = 0;
    int idx = (pucaMacAddr[NVRAM_MAC_ADDRESS_LEN - CHANGED_OCTET] -
        g_pNvramInfo->ucaBaseMacAddr[NVRAM_MAC_ADDRESS_LEN - CHANGED_OCTET]);

    // if overflow 255 (negitive), add 256 to have the correct index
    if (idx < 0)
        idx += 256;
    ulIdx = (unsigned long) (idx >> SHIFT_BITS);

    if( ulIdx < g_pNvramInfo->ulNumMacAddrs )
    {
        PMAC_ADDR_INFO pMai = &g_pNvramInfo->MacAddrs[ulIdx];
        if( pMai->chInUse == 1 )
        {
            pMai->chInUse = 0;
            nRet = 0;
        }
    }

    return( nRet );
} /* kerSysReleaseMacAddr */

int kerSysGetSdramSize( void )
{
    return( (int) g_pNvramInfo->ulSdramSize );
} /* kerSysGetSdramSize */


void kerSysLedCtrl(BOARD_LED_NAME ledName, BOARD_LED_STATE ledState)
{
    if (g_ledInitialized)
      boardLedCtrl(ledName, ledState);
}

unsigned int kerSysMonitorPollHook( struct file *f, struct poll_table_struct *t)
{
    int mask = (*g_orig_fop_poll) (f, t);

    if( g_wakeup_monitor == 1 && g_monitor_file == f )
    {
        /* If g_wakeup_monitor is non-0, the user mode application needs to
         * return from a blocking select function.  Return POLLPRI which will
         * cause the select to return with the exception descriptor set.
         */
        mask |= POLLPRI;
        g_wakeup_monitor = 0;
    }

    return( mask );
}

/* Put the user mode application that monitors link state on a run queue. */
void kerSysWakeupMonitorTask( void )
{
    g_wakeup_monitor = 1;
    if( g_monitor_task )
        wake_up_process( g_monitor_task );
}


//swda.030211
char *kerSysGetProductName(void)
{
    return g_pSystemBoardId;
}

//swda
int kerSysGetPsiSize(void)
{
    return (g_pNvramInfo->ulPsiSize);
}

//swda add
void kerSysGetEthWlanMacAddress(unsigned char *pMacAddr, int MacType )
{
	 PNVRAM_DATA pNvramData = (PNVRAM_DATA) get_nvram_start_addr();
	 if (MacType == 2)//WLAN MAC
	 	memcpy( pMacAddr, pNvramData->ucaWlanMacAddr,6 );
	 else	//Eth MAC
	 	memcpy( pMacAddr, pNvramData->ucaBaseMacAddr,6 );
}

//swda add
void kerSysGetWlan4318MacAddress(unsigned char *pMacAddr)
{
        PMAC_ADDR_INFO pMai = NULL;
	PNVRAM_DATA pNvramData = (PNVRAM_DATA) get_nvram_start_addr();
	unsigned int temp_mac;
	memcpy( pMacAddr, pNvramData->ucaBaseMacAddr,6 );
        pMai = pNvramData->ucaBaseMacAddr;
#ifdef BRCM_DRIVER_USB
//use fourth mac address
	temp_mac = pMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]+3;
	pMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]+=3;
	if (temp_mac > 0xff)
	{
		pMacAddr[NVRAM_MAC_ADDRESS_LEN - 2] += 1;
		if( (pMacAddr[NVRAM_MAC_ADDRESS_LEN - 2]&0xff) == 0x00)
			pMacAddr[NVRAM_MAC_ADDRESS_LEN - 3] += 1;
	}
        pMai=pMai+3;
        pMai->chInUse =1;
#else
//use second mac address
        pMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]+=1;
	if( (pMacAddr[NVRAM_MAC_ADDRESS_LEN - 1]&0xff) == 0x00){
		pMacAddr[NVRAM_MAC_ADDRESS_LEN - 2] += 1;
		if( (pMacAddr[NVRAM_MAC_ADDRESS_LEN - 2]&0xff) == 0x00)
			pMacAddr[NVRAM_MAC_ADDRESS_LEN - 3] += 1;
	}
        pMai=pMai+1;
        pMai->chInUse =1;
#endif
}

//swda add
void kerSysGetSpromVer(unsigned char *pspromver)
{
	 PNVRAM_DATA pNvramData = (PNVRAM_DATA) get_nvram_start_addr();
	 *pspromver = pNvramData->wlan_sprom_ver;
}
void kerSysGetSpromSavedFlag(unsigned char *pspromsavedflag)
{
	 PNVRAM_DATA pNvramData = (PNVRAM_DATA) get_nvram_start_addr();
	 *pspromsavedflag = pNvramData->sprom_saved_flag;
}
void kerSysSetSpromSavedFlag(unsigned char spromsavedflag)
{
	 PNVRAM_DATA pNvramData = (PNVRAM_DATA) get_nvram_start_addr();
	 NVRAM_DATA SaveNvramData;
	 UINT32 crc = CRC32_INIT_VALUE;
    
	 // save NVRAM data into a local structure
     memcpy( &SaveNvramData, pNvramData, sizeof(NVRAM_DATA) );
     SaveNvramData.sprom_saved_flag = spromsavedflag;
     
     //write the nvramData struct to nvram after CRC is calculated
     SaveNvramData.ulCheckSum = 0;
     crc = getCrc32((char *)&SaveNvramData, (UINT32) sizeof(NVRAM_DATA), crc);      
     SaveNvramData.ulCheckSum = crc;
     kerSysNvRamSet((char *) &SaveNvramData, sizeof(SaveNvramData), 0);
}

void kerSysGetSpromTable(char *pspromdata,int size,int offset)
{
     kerSysNvRamGet(pspromdata, size, offset);
}
void kerSysSetSpromTable(char *pspromdata,int size,int offset)
{
     kerSysNvRamSet(pspromdata, size, offset);
}

//swda add
int kerSysGetChipId( void )
{
#if BCM6348A
return (0x6348A);
#elif BCM6348B
return (0x6348B);
#elif BCM6338A
return (0x6338A);
#elif BCM6338B
return (0x6338B);
#elif BCM6345A
return (0x6345A);
#elif BCM6345A2
return (0x6345A2);
#elif BCM6345B
return (0x6345B);
#else
return (0x6348);
#endif
}

//********************************************************************************************
// misc. ioctl calls come to here. (flash, led, reset, kernel memory access, etc.)
//********************************************************************************************
static int board_ioctl( struct inode *inode, struct file *flip,
                        unsigned int command, unsigned long arg )
{
    int ret = 0;
    int totalSize = flash_get_total_size();
    unsigned char nbootflag; /*vic add for bootflag check*/
    BOARD_IOCTL_PARMS ctrlParms;
    unsigned char ucaMacAddr[NVRAM_MAC_ADDRESS_LEN];
    int allowedSize;
    static unsigned char HasWriteBinfs=0;
    
    switch (command) 
    {
#if VOXXXLOAD
        case BOARD_IOCTL_PSTNCALL:
           spin_lock(&pstn_spin_lock);
           ctrlParms.result = (int) PSTNComingCall;
           spin_unlock(&pstn_spin_lock);
           __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
           ret=0;
	   break;
        case BOARD_IOCTL_OFFHOOK:
           spin_lock(&offhook_spin_lock);
            ctrlParms.result = (int) PSTNphone2OFFHOOK;
           spin_unlock(&offhook_spin_lock);
            __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
            ret = 0;
            break;
#endif
#if ODM_AUTO_PROVISION_LAN
        case BOARD_IOCTL_AUTO_PROVISION_LAN:
            ctrlParms.result = (int) autoprovisionlan;
            __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
            ret = 0;
            break;
#endif
        case BOARD_IOCTL_FLASH_INIT:
            // not used for now.  kerSysBcmImageInit();
            break;


        case BOARD_IOCTL_FLASH_WRITE:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0)
            {
            	char SaveNvramData_sprom[NVRAM_SPROM_TABLE_SIZE];//swda add
                NVRAM_DATA SaveNvramData;
                PNVRAM_DATA pNvramData = (PNVRAM_DATA) get_nvram_start_addr();
//swda add,02/04/2005
#ifdef SERIAL_NUMBER
				char Serial_No[SN_SIZE];
#endif
//swda add end
//swda add,04/08/2005
#ifdef	DEFAULT_WLAN_WEP128
				char WEP128bit_key[WEP128KEY_SIZE];
#endif
//swda add end
                //kerSysLedCtrl(kLedDiag, kLedStateBlinkOnce);
                switch (ctrlParms.action)
                {
    /*==== add by Andrew (2004/09/14)====*/
#if defined(CFG_XFER_TO_FACDEFLT)
                    case FACTDEFLT:
                        ret = kerSysFactDefltSet(ctrlParms.string, ctrlParms.strLen, ctrlParms.offset);
                        break;
#endif
    /*==== end ====*/

                    case SCRATCH_PAD:
                        ret = kerSysScratchPadSet(ctrlParms.string, ctrlParms.buf, ctrlParms.offset);
                        break;

                    case PERSISTENT:
#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led flash red
                        kerSysLedCtrl(kLedPower, kLedStateOff);//green off
                        kerSysLedCtrl(kLedDiag, kLedStateOn);//red on
                        kerSysSetUpgradeState(2); //upgrade configuration start
#endif                                                               
                        ret = kerSysPersistentSet(ctrlParms.string, ctrlParms.strLen, ctrlParms.offset);
#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led solid green
                        kerSysSetUpgradeState(0);//upgrade configuration end
                        if ( IsMultiConfig || (!boot_complete_flag) ) {	
                        	kerSysLedCtrl(kLedDiag, kLedStateOn);//red on
                        } else {
                        	kerSysLedCtrl(kLedDiag, kLedStateOff);//red
                        	kerSysLedCtrl(kLedPower, kLedStateOn);//green
                        }
#endif
                        break;
                
                    case NVRAM:
                        if (ctrlParms.offset == 1024 + 10) {
                            unsigned long memType=*(unsigned long *)ctrlParms.string;                             
                            kerSysMemoryTypeSet(FLASH_BASE,(char *) &memType, sizeof(int));
                        }
                        else                        	
                            ret = kerSysNvRamSet(ctrlParms.string, ctrlParms.strLen, ctrlParms.offset);
                        break;
                    case BCM_IMAGE_BINFS:
 //printk("will flash bin fs offset at 0%x\n",ctrlParms.offset);
                        if ( flash_get_total_size() < 8*1024*1024 ) {
                            printk(" Bin file system only is allowed for 8MB not for %d MB Flash\n",flash_get_total_size()/0x100000);
                            ret = -1;
                            break;
                        }   
                     
                            
                        if( ctrlParms.strLen <= 0 || ctrlParms.strLen > 4*1024*1024 - 64*1024)
                        {
                            printk("Illegal root file system size [%d]. Size allowed: [%d]\n",
                                ctrlParms.strLen,  allowedSize);
                            ret = -1;
                            break;
                        }
#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led flash red
                        kerSysLedCtrl(kLedPower, kLedStateOff);//green
                        kerSysLedCtrl(kLedDiag, kLedStateOn);//red 
                        kerSysSetUpgradeState(1); //upgrade start
#endif                                                                                                                                                                         
                        ret = kerSysBcmImageSet(ctrlParms.offset, ctrlParms.string, ctrlParms.strLen);
#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led solid green
                        kerSysLedCtrl(kLedDiag, kLedStateOff);//red
                        kerSysLedCtrl(kLedPower, kLedStateOn);//green
                        kerSysSetUpgradeState(0);//upgrade end
#endif
                        HasWriteBinfs = 1;
                        break;    
                    case BCM_IMAGE_CFE:
                        if( ctrlParms.strLen <= 0 || ctrlParms.strLen > FLASH45_LENGTH_BOOT_ROM )
                        {
                            printk("Illegal CFE size [%d]. Size allowed: [%d]\n",
                                ctrlParms.strLen, FLASH45_LENGTH_BOOT_ROM);
                            ret = -1;
                            break;
                        }
#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led flash red
                        kerSysLedCtrl(kLedPower, kLedStateOff);//green
                        kerSysLedCtrl(kLedDiag, kLedStateOn);//red
                        kerSysSetUpgradeState(1); //upgrade start
#endif
                        // save NVRAM data into a local structure
                        memcpy( &SaveNvramData, pNvramData, sizeof(NVRAM_DATA) );

                        //swda : save NVRAM sprom data into a local structure
                        kerSysNvRamGet(SaveNvramData_sprom, NVRAM_SPROM_TABLE_SIZE, NVRAM_SPROM_TABLE_OFFSET);
//swda add,02/04/2005        
#ifdef SERIAL_NUMBER		//save Serial Number into a local structure
						kerSysNvRamGet(Serial_No, SN_SIZE, SN_OFFSET);
#endif
//swda add end
//swda add,04/08/2005
#ifdef	DEFAULT_WLAN_WEP128	//save WEP 128bit key into a local structure
						kerSysNvRamGet(WEP128bit_key,WEP128KEY_SIZE,WEP128KEY_OFFSET);
#endif
//swda add end
                        // set memory type field
                        BpGetSdramSize( (unsigned long *) &ctrlParms.string[SDRAM_TYPE_ADDRESS_OFFSET] );

                        ret = kerSysBcmImageSet(ctrlParms.offset, ctrlParms.string, ctrlParms.strLen);

                        // if nvram is not valid, restore the current nvram settings
                        if( BpSetBoardId( pNvramData->szBoardId ) != BP_SUCCESS &&
                            *(unsigned long *) pNvramData == NVRAM_DATA_ID )
                        {
                            kerSysNvRamSet((char *) &SaveNvramData, sizeof(SaveNvramData), 0);
                        }
                        //swda : restore nvram sprom data
                        kerSysNvRamSet(SaveNvramData_sprom, NVRAM_SPROM_TABLE_SIZE, NVRAM_SPROM_TABLE_OFFSET);

//swda add,02/04/2005        
#ifdef SERIAL_NUMBER		//restore Serial Number
						kerSysNvRamSet(Serial_No, SN_SIZE, SN_OFFSET);
#endif
//swda add end                        
//swda add,04/08/2005
#ifdef	DEFAULT_WLAN_WEP128 //restore WEP 128bit key
						kerSysNvRamSet(WEP128bit_key,WEP128KEY_SIZE,WEP128KEY_OFFSET);
#endif
//swda add end                      
#if defined(CONFIG_BCM96345)

                        /*Michael: fix when use web to upgrade image include CFE cause SDRAM size change from 16M to 8M */

                        if((strncmp(pNvramData->szBoardId,"RTA230",6)==0) || (strncmp(pNvramData->szBoardId,"RTA230B",7)==0)){
                        printk("According to your board ID(%s)Set your memory size to 8M\n",pNvramData->szBoardId);
                        //default is 8M so don't need kerSysMemoryTypeSet
                         } else {
                          unsigned long memType=1;  //  0==>8M SDRAM  1==>16M SDRAM
			  #define ODM_FLASH_BASE                   0xBFC00000
		          #define ODM_FLASH63XX_ADDR_BOOT_ROM     ((char *) ODM_FLASH_BASE)
 	        	   kerSysMemoryTypeSet((int) ODM_FLASH63XX_ADDR_BOOT_ROM, (char *) &memType, sizeof(int));
                          printk("According to your board ID(%s)Set your memory size to 16M\n",pNvramData->szBoardId);
			}
#endif
#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led solid green
                        kerSysLedCtrl(kLedDiag, kLedStateOff);//red
                        kerSysLedCtrl(kLedPower, kLedStateOn);//green
                        kerSysSetUpgradeState(0); //upgrade end
#endif
                        break;
                        
                    case BCM_IMAGE_FS:
                        allowedSize = (int) flash_get_total_size() - \
                            FLASH_RESERVED_AT_END - TAG_LEN - FLASH45_LENGTH_BOOT_ROM;
                        if( ctrlParms.strLen <= 0 || ctrlParms.strLen > allowedSize)
                        {
                            printk("Illegal root file system size [%d]. Size allowed: [%d]\n",
                                ctrlParms.strLen,  allowedSize);
                            ret = -1;
                            break;
                        }
#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led flash red
                        kerSysLedCtrl(kLedPower, kLedStateOff);//green
                        kerSysLedCtrl(kLedDiag, kLedStateOn);//red
                        kerSysSetUpgradeState(1); //upgrade start
#endif
                        //printk("\nflash fs offset at 0%x\n",ctrlParms.offset);
              
#if defined(CONFIG_BCM96348)                        
                        {
                           unsigned char* bootflag;
			   PFILE_TAG pTag;
                           bootflag=flash_get_memptr((byte) 0);
                           int tagVer, curVer;
                           
                           pTag = (PFILE_TAG) ctrlParms.string;
                           //printk("got %s cur %s\n", pTag->FlashLayoutVersion,BCM_FLASH_LAYOUT_VER);
                           tagVer = (u_int32_t) simple_strtoul(pTag->FlashLayoutVersion, NULL, 10);
    			   curVer = (u_int32_t) simple_strtoul(BCM_FLASH_LAYOUT_VER, NULL, 10);
                           
                           if (totalSize == 0x800000) 
                              if (curVer > 4 && tagVer < 5) { /* 5 start from 3.28u */
                   	          printk("Not allow to downgrade to flash layout version less than 5\n"); 
                              ret = -1;
                              break;
                           }  
                           
                           
                           
                           
                           if (totalSize == 0x800000) {nbootflag=*bootflag;printk("\nget bootflag %d\n",*bootflag);}
                           else nbootflag=1; 
                           
                                                   
                           if (((curVer >= 4) && ( tagVer <4 )) || HasWriteBinfs ) { /* flash layout 3 -> 4 will knows dualimage@8M */
                                 printk("flash layout curVer %x tagVer %x\n", curVer,tagVer);
                           	/* ?? erase bootflag to avoid choose an broken partition to start */
                                 if (!HasWriteBinfs) { printk("Not "); kerSysErasePS1();}
                                 printk("Has Writtten Binfs\n");   
                           	 nbootflag=1; /* boot from bf800000 will flash bfc00000 */
                           	/* will erase bf800000 after image(bfc00000) upgrade successfully */
                           }
                           
                           /*	nbootflag=1;  FAKE!!!! always flash bfc00000 */

                           /* bootflag is 0 means boot from bfc00000 will flash bf800000 */
                           if (nbootflag==0) {
                           	
                           	u_int32_t rootfs_addr,kernel_addr,tagCrc;
                           	                           	
                           	//printk("rootfs addr %s kernelAddress %s\n", pTag->rootfsAddress,pTag->kernelAddress);
                           	rootfs_addr = (u_int32_t) simple_strtoul(pTag->rootfsAddress, NULL, 10);
                                kernel_addr = (u_int32_t) simple_strtoul(pTag->kernelAddress, NULL, 10);
                                //printk("rootfs addr %x kernelAddress %x\n", rootfs_addr,kernel_addr);
                                rootfs_addr-=0x400000;
                                sprintf(pTag->rootfsAddress,"%lu",(unsigned long) rootfs_addr);
                                kernel_addr-=0x400000;
                                sprintf(pTag->kernelAddress,"%lu",(unsigned long) kernel_addr);
                                // get tag crc
                                //printk("rootfs addr %x kernelAddress %x\n", rootfs_addr,kernel_addr);
    				tagCrc = CRC32_INIT_VALUE;
    				tagCrc = getCrc32((byte*)pTag, TAG_LEN-TOKEN_LEN, tagCrc);
    				memcpy(pTag->tagValidationToken, (byte*)&tagCrc, CRC_LEN);
                           	ctrlParms.offset-=0x400000;
                           	nbootflag=1;
                           }
                           else  nbootflag=0;  /* will flash bfc00000 if bootflag is 1(bf800000 is valid) or 2(binfs) or else */
                         
                        }
#endif

#if defined(CONFIG_BCM96338)
	{
		PFILE_TAG pTag;		
    		unsigned char bootf1,bootf2;
    		int tagVer, curVer;
                           
                 pTag = (PFILE_TAG) ctrlParms.string;
                 printk("\ngot len %d/%d \n", ctrlParms.strLen,(0x200000 - TAG_LEN - FLASH45_LENGTH_BOOT_ROM -1));
                 tagVer = (u_int32_t) simple_strtoul(pTag->FlashLayoutVersion, NULL, 10);
    	         curVer = (u_int32_t) simple_strtoul(BCM_FLASH_LAYOUT_VER, NULL, 10);
    	         
    	         
    	         if (totalSize == 0x400000) 
                   if (curVer > 4 && tagVer < 5) { /* 5 start from 3.28u */
                   	   printk("Not allow to downgrade to flash layout version less than 5\n"); 
                       ret = -1;
                       break;
                   }  

		    if ((totalSize == 0x400000) && (ctrlParms.strLen <= (0x200000 - TAG_LEN - FLASH45_LENGTH_BOOT_ROM -1) ) ) { /* get bootflag to compare where to boot from */ 
    		   bootf1 = *(unsigned char*) (0xbfe00000 -1); /*see design note 4MB flash layout*/
    		   bootf2 = *(unsigned char*) (0xbfff0000 -1);
    		   if ((bootf1 >> 2) == 0 && bootf1 ) {
             	    if ((bootf2 >> 2) == 0 && bootf2 ) { /* both are valid */
          	      if ((bootf1+1) == bootf2 || ((bootf1+1) >>2 == bootf2) ) nbootflag=1; /*bank2*/
          	      if ((bootf2+1) == bootf1 || ((bootf2+1) >>2 == bootf1) ) nbootflag=0; /*bank1*/ 
                    } else nbootflag = 0; /* bank2 invalid bank1 valid */
               } else  if ((bootf2 >> 2) == 0 && bootf2) nbootflag =1; /*bank 1 invalid,bank 2 valid*/
                   else  nbootflag=0; /* bank1 bank2 both invalid */   
                   printk("\nbootf1 %x bootf2 %x,boot from bank %d\n",bootf1,bootf2, nbootflag);
            } else { nbootflag=2; kerSysErasePS1(); }
                
                if (totalSize == 0x400000)
                if ((curVer >= 4) && ( tagVer <4 )) { /* flash layout 3 -> 4 will knows dualimage@8M */
                    printk("flash layout curVer %x tagVer %x\n", curVer,tagVer);
                    /* ?? erase bootflag to avoid choose an broken partition to start */
                    kerSysErasePS1();
                    nbootflag=2; /* skip set bootflag for bank 2 */
                }                
                
                if (nbootflag==0) { /*boot from bank0 will flash bank1*/

                        u_int32_t rootfs_addr,kernel_addr,tagCrc;
                           	                           	
                        //printk("rootfs addr %s kernelAddress %s\n", pTag->rootfsAddress,pTag->kernelAddress);
                        rootfs_addr = (u_int32_t) simple_strtoul(pTag->rootfsAddress, NULL, 10);
                        kernel_addr = (u_int32_t) simple_strtoul(pTag->kernelAddress, NULL, 10);
                        //printk("rootfs addr %x kernelAddress %x\n", rootfs_addr,kernel_addr);
                        rootfs_addr+=0x1f0000;
                        sprintf(pTag->rootfsAddress,"%lu",(unsigned long) rootfs_addr);
                        kernel_addr+=0x1f0000;
                        sprintf(pTag->kernelAddress,"%lu",(unsigned long) kernel_addr);
                        // get tag crc
                        //printk("rootfs addr %x kernelAddress %x\n", rootfs_addr,kernel_addr);
    			tagCrc = CRC32_INIT_VALUE;
    			tagCrc = getCrc32((byte*)pTag, TAG_LEN-TOKEN_LEN, tagCrc);
    			memcpy(pTag->tagValidationToken, (byte*)&tagCrc, CRC_LEN);
                        ctrlParms.offset+=0x1f0000;
                        nbootflag=1;                	                	                	
        	} else if ( nbootflag ==1 ) nbootflag=0; 				
	}
#endif                       
                        //printk("will flash fs offset at 0%x\n",ctrlParms.offset);                                                                                                                                                
                        ret = kerSysBcmImageSet(ctrlParms.offset, ctrlParms.string, ctrlParms.strLen);
#if defined(CONFIG_BCM96348)                         
                        if (totalSize == 0x800000)
                         if(HasWriteBinfs)  kerSysBootFlagSet(0x02);
                         else  kerSysBootFlagSet(nbootflag);

#endif
#if defined(CONFIG_BCM96338)
                       if (totalSize == 0x400000 && (nbootflag<2) ) 
                           kerSysBootFlagSet(nbootflag);

#endif
                        
#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led solid green
                        kerSysLedCtrl(kLedDiag, kLedStateOff);//red
                        kerSysLedCtrl(kLedPower, kLedStateOn);//green
                        kerSysSetUpgradeState(0); //upgrade end
#endif
                        kerSysMipsSoftReset();
                        break;

                    case BCM_IMAGE_KERNEL:  // not used for now.
                        break;
                    case BCM_IMAGE_WHOLE:
                        if(ctrlParms.strLen <= 0)
                        {
                            printk("Illegal flash image size [%d].\n", ctrlParms.strLen);
                            ret = -1;
                            break;
                        }

#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led flash red
                        kerSysLedCtrl(kLedPower, kLedStateOff);//green
                        kerSysLedCtrl(kLedDiag, kLedStateOn);//red
                        kerSysSetUpgradeState(1); //upgrade start
#endif
                        // save NVRAM data into a local structure
                        memcpy( &SaveNvramData, pNvramData, sizeof(NVRAM_DATA) );

                        //swda : save NVRAM sprom data into a local structure
                        kerSysNvRamGet(SaveNvramData_sprom, NVRAM_SPROM_TABLE_SIZE, NVRAM_SPROM_TABLE_OFFSET);
                        
//swda add,02/04/2005        
#ifdef SERIAL_NUMBER		//save Serial Number into a local structure
						kerSysNvRamGet(Serial_No, SN_SIZE, SN_OFFSET);
#endif
//swda add end                        
//swda add,04/08/2005
#ifdef	DEFAULT_WLAN_WEP128	//save WEP 128bit key into a local structure
						kerSysNvRamGet(WEP128bit_key,WEP128KEY_SIZE,WEP128KEY_OFFSET);
#endif
//swda add end
                        
                        ret = kerSysBcmImageSet(ctrlParms.offset, ctrlParms.string, ctrlParms.strLen);

                        // if nvram is not valid, restore the current nvram settings
                        if( BpSetBoardId( pNvramData->szBoardId ) != BP_SUCCESS &&
                            *(unsigned long *) pNvramData == NVRAM_DATA_ID )
                        {
                            kerSysNvRamSet((char *) &SaveNvramData, sizeof(SaveNvramData), 0);
                        }

						//swda : restore nvram sprom data
                        kerSysNvRamSet(SaveNvramData_sprom, NVRAM_SPROM_TABLE_SIZE, NVRAM_SPROM_TABLE_OFFSET);

//swda add,02/04/2005        
#ifdef SERIAL_NUMBER		//restore Serial Number
						kerSysNvRamSet(Serial_No, SN_SIZE, SN_OFFSET);
#endif
//swda add end 
//swda add,04/08/2005
#ifdef	DEFAULT_WLAN_WEP128	//restore WEP 128bit key
						kerSysNvRamSet(WEP128bit_key,WEP128KEY_SIZE,WEP128KEY_OFFSET);
#endif
//swda add end        

#if defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
                        //swda: Power Led solid green
                        kerSysLedCtrl(kLedDiag, kLedStateOff);//red
                        kerSysLedCtrl(kLedPower, kLedStateOn);//green
                        kerSysSetUpgradeState(0); //upgrade end
#endif						

                        kerSysMipsSoftReset();
                        break;

                    default:
                        ret = -EINVAL;
                        printk("flash_ioctl_command: invalid command %d\n", ctrlParms.action);
                        break;
                }
                ctrlParms.result = ret;
                __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
                //kerSysLedCtrl(kLedDiag, kLedStateOff);
            }
            else
                ret = -EFAULT;
            break;

        case BOARD_IOCTL_FLASH_READ:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) 
            {
                switch (ctrlParms.action)
                {
    /*==== add by Andrew (2004/09/14)====*/
#if defined(CFG_XFER_TO_FACDEFLT)
                    case FACTDEFLT:
                        ret = kerSysFactDefltGet(ctrlParms.string, ctrlParms.strLen, ctrlParms.offset);
                        break;
#endif
    /*==== end ====*/

                    case SCRATCH_PAD:
                        ret = kerSysScratchPadGet(ctrlParms.string, ctrlParms.buf, ctrlParms.offset);
                        break;

                    case PERSISTENT:
                        ret = kerSysPersistentGet(ctrlParms.string, ctrlParms.strLen, ctrlParms.offset);
                        break;

                    case NVRAM:
                        ret = kerSysNvRamGet(ctrlParms.string, ctrlParms.strLen, ctrlParms.offset);
                        break;

                    case FLASH_SIZE:
                        ret = kerSysFlashSizeGet();
                        break;

                    default:
                        ret = -EINVAL;
                        printk("Not supported.  invalid command %d\n", ctrlParms.action);
                        break;
                }
                ctrlParms.result = ret;
                __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
            }
            else
                ret = -EFAULT;
            break;

        case BOARD_IOCTL_GET_NR_PAGES:
            ctrlParms.result = nr_free_pages() + get_page_cache_size();
            __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
            ret = 0;
            break;

        case BOARD_IOCTL_DUMP_ADDR:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) 
            {
                dumpaddr( (unsigned char *) ctrlParms.string, ctrlParms.strLen );
                ctrlParms.result = 0;
                __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
                ret = 0;
            }
            else
                ret = -EFAULT;
            break;

        case BOARD_IOCTL_SET_MEMORY:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) 
            {
                unsigned long  *pul = (unsigned long *)  ctrlParms.string;
                unsigned short *pus = (unsigned short *) ctrlParms.string;
                unsigned char  *puc = (unsigned char *)  ctrlParms.string;
                switch( ctrlParms.strLen )
                {
                    case 4:
                        *pul = (unsigned long) ctrlParms.offset;
                        break;
                    case 2:
                        *pus = (unsigned short) ctrlParms.offset;
                        break;
                    case 1:
                        *puc = (unsigned char) ctrlParms.offset;
                        break;
                }
                dumpaddr( (unsigned char *) ctrlParms.string, sizeof(long) );
                ctrlParms.result = 0;
                __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
                ret = 0;
            }
            else
                ret = -EFAULT;
            break;
      
        case BOARD_IOCTL_MIPS_SOFT_RESET:
            kerSysMipsSoftReset();
            break;

        case BOARD_IOCTL_LED_CTRL:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) 
            {
	            kerSysLedCtrl((BOARD_LED_NAME)ctrlParms.strLen, (BOARD_LED_STATE)ctrlParms.offset);
	            ret = 0;
	        }
            break;

        case BOARD_IOCTL_GET_ID:
            if (copy_from_user((void*)&ctrlParms, (void*)arg,
                sizeof(ctrlParms)) == 0) 
            {
                if( ctrlParms.string )
                {
                    char *p = (char *) get_system_type();
                    if( strlen(p) + 1 < ctrlParms.strLen )
                        ctrlParms.strLen = strlen(p) + 1;
                    __copy_to_user(ctrlParms.string, p, ctrlParms.strLen);
                }

                ctrlParms.result = 0;
                __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms,
                    sizeof(BOARD_IOCTL_PARMS));
            }
            break;
#ifdef NNN_POT            
        case BOARD_IOCTL_GET_POT:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
        	
               if ( ctrlParms.offset == -1) {
            	   kerPOTSet(2);
            	   pot = 0;
               }
               ctrlParms.result = (int) pot;
               __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));    
               ret = 0;
            }
            else
                ret = -EFAULT;
            break;    
#endif
        case BOARD_IOCTL_GET_MAC_ADDRESS:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) 
            {
                ctrlParms.result = kerSysGetMacAddress( ucaMacAddr,
                    ctrlParms.offset );

                if( ctrlParms.result == 0 )
                {
                    __copy_to_user(ctrlParms.string, ucaMacAddr,
                        sizeof(ucaMacAddr));
                }

                __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms,
                    sizeof(BOARD_IOCTL_PARMS));
                ret = 0;
            }
            else
                ret = -EFAULT;
            break;

        case BOARD_IOCTL_RELEASE_MAC_ADDRESS:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) 
            {
                if (copy_from_user((void*)ucaMacAddr, (void*)ctrlParms.string, \
                     NVRAM_MAC_ADDRESS_LEN) == 0) 
                {
                    ctrlParms.result = kerSysReleaseMacAddress( ucaMacAddr );
                }
                else
                {
                    ctrlParms.result = -EACCES;
                }

                __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms,
                    sizeof(BOARD_IOCTL_PARMS));
                ret = 0;
            }
            else
                ret = -EFAULT;
            break;

        case BOARD_IOCTL_GET_PSI_SIZE:
            ctrlParms.result = (int) g_pNvramInfo->ulPsiSize;
            __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
            ret = 0;
            break;

        case BOARD_IOCTL_GET_SDRAM_SIZE:
            ctrlParms.result = (int) g_pNvramInfo->ulSdramSize;
            __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
            ret = 0;
            break;

        case BOARD_IOCTL_GET_BASE_MAC_ADDRESS:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) 
            {
                __copy_to_user(ctrlParms.string, g_pNvramInfo->ucaBaseMacAddr, NVRAM_MAC_ADDRESS_LEN);
                ctrlParms.result = 0;

                __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms,
                    sizeof(BOARD_IOCTL_PARMS));
                ret = 0;
            }
            else
                ret = -EFAULT;
            break;

        case BOARD_IOCTL_GET_CHIP_ID:
            ctrlParms.result = (int) (PERF->RevID & 0xFFFF0000) >> 16;
            __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
            ret = 0;
            break;
        
        case BOARD_IOCTL_GET_CHIP_REV:
            ctrlParms.result = (int) (PERF->RevID & 0x000000FF);
            __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
            ret = 0;
            break;    

        case BOARD_IOCTL_GET_NUM_ENET: {
            ETHERNET_MAC_INFO EnetInfos[BP_MAX_ENET_MACS];
            int i, numeth = 0;
            if (BpGetEthernetMacInfo(EnetInfos, BP_MAX_ENET_MACS) == BP_SUCCESS) {
            for( i = 0; i < BP_MAX_ENET_MACS; i++) {
                if (EnetInfos[i].ucPhyType != BP_ENET_NO_PHY) {
                numeth++;
                }
            }
            ctrlParms.result = numeth;
            __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms,	 sizeof(BOARD_IOCTL_PARMS));   
            ret = 0;
            }
	        else {
	            ret = -EFAULT;
	        }
	        break;
            }

        case BOARD_IOCTL_GET_CFE_VER:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
                char *vertag =  (char *)(FLASH_BASE + CFE_VERSION_OFFSET);
                if (ctrlParms.strLen < CFE_VERSION_SIZE) {
                    ctrlParms.result = 0;
                    __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
                    ret = -EFAULT;
                }
                else if (strncmp(vertag, "cfe-v", 5)) { // no tag info in flash
                    ctrlParms.result = 0;
                    __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
                    ret = 0;
                }
                else {
                    ctrlParms.result = 1;
                    __copy_to_user(ctrlParms.string, vertag+CFE_VERSION_MARK_SIZE, CFE_VERSION_SIZE);
                    __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
                    ret = 0;
                }
            }
            else {
                ret = -EFAULT;
            }
            break;

        case BOARD_IOCTL_GET_ENET_CFG:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
                ETHERNET_MAC_INFO EnetInfos[BP_MAX_ENET_MACS];
                if (BpGetEthernetMacInfo(EnetInfos, BP_MAX_ENET_MACS) == BP_SUCCESS) {
                    if (ctrlParms.strLen == sizeof(EnetInfos)) {
                        __copy_to_user(ctrlParms.string, EnetInfos, sizeof(EnetInfos));
                        ctrlParms.result = 0;
                        __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms,	 sizeof(BOARD_IOCTL_PARMS));   
                        ret = 0;
                    } else
	                    ret = -EFAULT;
                }
                else {
                    ret = -EFAULT;
                }
                break;
            }
            else {
                ret = -EFAULT;
            }
            break;            

#if defined (WIRELESS)
        case BOARD_IOCTL_GET_WLAN_ANT_INUSE:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
                unsigned short antInUse = 0;
                if (BpGetWirelessAntInUse(&antInUse) == BP_SUCCESS) {
                    if (ctrlParms.strLen == sizeof(antInUse)) {
                        __copy_to_user(ctrlParms.string, &antInUse, sizeof(antInUse));
                        ctrlParms.result = 0;
                        __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));   
                        ret = 0;
                    } else
	                    ret = -EFAULT;
                }
                else {
                    ret = -EFAULT;
                }
                break;
            }
            else {
                ret = -EFAULT;
            }
            break;            
#endif            
        case BOARD_IOCTL_SET_TRIGGER_EVENT:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {            	
            	BOARD_IOC *board_ioc = (BOARD_IOC *)flip->private_data;            	
                ctrlParms.result = -EFAULT;
                ret = -EFAULT;
                if (ctrlParms.strLen == sizeof(unsigned long)) {                 	                    
                    board_ioc->eventmask |= *((int*)ctrlParms.string);                    
#if defined (WIRELESS)                    
                    if((board_ioc->eventmask & SES_EVENTS)) {
                        if(sesBtn_irq != BP_NOT_DEFINED) {
                            BcmHalInterruptEnable(sesBtn_irq);
                            ctrlParms.result = 0;
                            ret = 0;
                        }                                                
                    } 
#endif                                                
                    __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));                        
                }
	        break;
            }
            else {
                ret = -EFAULT;
            }
            break;                        

        case BOARD_IOCTL_GET_TRIGGER_EVENT:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
            	BOARD_IOC *board_ioc = (BOARD_IOC *)flip->private_data;
                if (ctrlParms.strLen == sizeof(unsigned long)) {
                    __copy_to_user(ctrlParms.string, &board_ioc->eventmask, sizeof(unsigned long));
                    ctrlParms.result = 0;
                    __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));   
                    ret = 0;
                } else
	            ret = -EFAULT;

	        break;
            }
            else {
                ret = -EFAULT;
            }
            break;                
            
        case BOARD_IOCTL_UNSET_TRIGGER_EVENT:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
                if (ctrlParms.strLen == sizeof(unsigned long)) {
                    BOARD_IOC *board_ioc = (BOARD_IOC *)flip->private_data;                	
                    board_ioc->eventmask &= (~(*((int*)ctrlParms.string)));                  
                    ctrlParms.result = 0;
                    __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));   
                    ret = 0;
                } else
	            ret = -EFAULT;

	        break;
            } 
            else {
                ret = -EFAULT;
            }
            break;            
#if defined (WIRELESS)
        case BOARD_IOCTL_SET_SES_LED:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
                if (ctrlParms.strLen == sizeof(int)) {
                    sesLed_ctrl(*(int*)ctrlParms.string);
                    ctrlParms.result = 0;
                    __copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));   
                    ret = 0;
                } else
	            ret = -EFAULT;

	        break;
            }
            else {
                ret = -EFAULT;
            }
            break;            
#endif                                                            

        case BOARD_IOCTL_SET_MONITOR_FD:
            if (copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
                int fput_needed = 0;

                g_monitor_file = fget_light( ctrlParms.offset, &fput_needed );
                if( g_monitor_file ) {
                    /* Hook this file descriptor's poll function in order to set
                     * the exception descriptor when there is a change in link
                     * state.
                     */
                    g_monitor_task = current;
                    g_orig_fop_poll = g_monitor_file->f_op->poll;
                    g_monitor_file->f_op->poll = kerSysMonitorPollHook;
                }
            }
            break;

        case BOARD_IOCTL_WAKEUP_MONITOR_TASK:
            kerSysWakeupMonitorTask();
            break;

//swda add,02/04/2005
#ifdef	SERIAL_NUMBER
		case BOARD_IOCTL_GET_SERIAL_NUMBER:
			if(copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
				//for cfesetup may need to re-read again
				kerSysNvRamGet(g_Serial_No, SN_SIZE, SN_OFFSET);
				//check if valid serial number
				if ( (g_Serial_No[0]<0x21 || g_Serial_No[0]>0x7e) && g_Serial_No[0]!=0 ) {
					printk("Invalid Serial number Reset to 0\n",g_Serial_No);
					memset(g_Serial_No, 0, SN_SIZE);
				}				
				strncpy(ctrlParms.string, g_Serial_No, SN_SIZE);
				ctrlParms.strLen = SN_SIZE;
				ctrlParms.string[ctrlParms.strLen] = '\0';
				ctrlParms.result = 0;
				__copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
				ret = 0;			
			} else {
				ret = -EFAULT;
			}
			break;	
#endif	
//swda add end

//swda add,04/08/2005
#ifdef	DEFAULT_WLAN_WEP128
		case BOARD_IOCTL_GET_WEP128_KEY:
			if(copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
				kerSysNvRamGet(g_WEP128bit_key,WEP128KEY_SIZE,WEP128KEY_OFFSET);
				{	//check if valid ASCII string
					int key_index,valid_key;
		
					valid_key = 1;
					for (key_index=0;key_index<WEP128KEY_SIZE;key_index++) {
					//printable ASCII character 0x21~0x7E
					  if ( g_WEP128bit_key[key_index]<0x21 || g_WEP128bit_key[key_index]>0x7e ) {
						valid_key = 0;
						break;
					  }	
					}
					if (!valid_key) {
						printk("invalid WEP KEY Reset ..\n");	
						strcpy(g_WEP128bit_key,"0000000000000");
					}	
				}		
				strncpy(ctrlParms.string, g_WEP128bit_key,WEP128KEY_SIZE);
				ctrlParms.strLen = WEP128KEY_SIZE;
				ctrlParms.string[ctrlParms.strLen] = '\0';
				ctrlParms.result = 0;
				__copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
				ret = 0;			
			} else {
				ret = -EFAULT;
			}
			break;
#endif
//swda add end
//swda add,09/12/2005
		case BOARD_IOCTL_INFO_BOOT_COMPLETE:
			printk("board_ioctl: boot complete!\n");
			boot_complete_flag = 1;
			break;
//swda add end
//swda add,11/02/2005
#ifdef	AAA_TEST_LED
		case BOARD_IOCTL_SET_AAA_TEST_LED:
			if(copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
				if (ctrlParms.offset==0) {
					AAATestLedTimerDelete();//if timer already running, delete it
					kerSysLedCtrl( kLedPPP, kLedStateOff );
				} else if (ctrlParms.offset==1) {
					AAATestLedTimerDelete();//if timer already running, delete it
					kerSysLedCtrl( kLedPPP, kLedStateOn );
					AAATestLedTimerStart();//turn off AAA test led after 30 seconds
				} else if (ctrlParms.offset==2) {
					AAATestLedTimerDelete();//if timer already running, delete it
					kerSysLedCtrl( kLedPPP, kLedStateFastBlinkContinues );
					AAATestLedTimerStart();//turn off AAA test led after 30 seconds
				}
				ctrlParms.result = 0;
				ret = 0;			
			} else {
				ret = -EFAULT;
			}
			break;
#endif
//swda add end
//swda add,11/03/2005
#ifdef WLAN_ENABLE_CTRL_BUTTON
		case BOARD_IOCTL_WLAN_ENABLE_CTRL_BUTTON:
			if(copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
				if ( ctrlParms.strLen == GET_BOOT_COMPLETE_FLAG ) {
					ctrlParms.result = boot_complete_flag;
				} else if ( ctrlParms.strLen == GET_WLAN_ENABLE_GPIO_STATUS ) {
					ctrlParms.result = (GPIO->GPIOio & (unsigned int)GPIO_WLAN_ENABLE_STATE) ? 1 : 0;
				}
				
				__copy_to_user((BOARD_IOCTL_PARMS*)arg, &ctrlParms, sizeof(BOARD_IOCTL_PARMS));
				ret = 0;			
			} else {
				ret = -EFAULT;
			}
			break;
#endif
//swda add end
//swda add,11/29/2006
		case BOARD_IOCTL_SET_VAR:
			if(copy_from_user((void*)&ctrlParms, (void*)arg, sizeof(ctrlParms)) == 0) {
				if ( strcmp(ctrlParms.string,"MultiConfig")==0 ) {
					IsMultiConfig = ctrlParms.strLen;
				}
				ctrlParms.result = 0;
				ret = 0;			
			} else {
				ret = -EFAULT;
			}
			break;
//swda add end
        default:
            ret = -EINVAL;
            ctrlParms.result = 0;
            printk("board_ioctl: invalid command %x, cmd %d .\n",command,_IOC_NR(command));
            break;

  } /* switch */

  return (ret);

} /* board_ioctl */

/***************************************************************************
 * SES Button ISR/GPIO/LED functions.
 ***************************************************************************/
#if defined (WIRELESS) 
static irqreturn_t sesBtn_isr(int irq, void *dev_id, struct pt_regs *ptregs)
{   
#if defined(_BCM96338_) || defined(CONFIG_BCM96338)
    unsigned long gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned long *gpio_reg = &GPIO->GPIOio;
#endif
#if defined(_BCM96345_) || defined(CONFIG_BCM96345)
    unsigned short gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned short *gpio_reg = &GPIO->GPIOio;
#endif
#if defined(_BCM96348_) || defined (CONFIG_BCM96348)
    unsigned long gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned long *gpio_reg = &GPIO->GPIOio;

    if( (sesBtn_gpio & ~BP_ACTIVE_MASK) >= 32 )
    {
        gpio_mask = GPIO_NUM_TO_MASK_HIGH(sesBtn_gpio);
        gpio_reg = &GPIO->GPIOio_high;
    }
#endif 
    		
    if (!(*gpio_reg & gpio_mask)){
        wake_up_interruptible(&g_board_wait_queue);
        return IRQ_RETVAL(1);
    } else {
        return IRQ_RETVAL(0);    	
    }
}

static void __init sesBtn_mapGpio()
{	
    if( BpGetWirelessSesBtnGpio(&sesBtn_gpio) == BP_SUCCESS )
    {
        printk("SES: Button GPIO 0x%x is enabled\n", sesBtn_gpio);    
    }
}

static void __init sesBtn_mapIntr(int context)
{    	
    if( BpGetWirelessSesExtIntr(&sesBtn_irq) == BP_SUCCESS )
    {
    	printk("SES: Button Interrupt 0x%x is enabled\n", sesBtn_irq);
    }
    else
    	return;
    	    
    sesBtn_irq += INTERRUPT_ID_EXTERNAL_0;	
    		
    if (BcmHalMapInterrupt((FN_HANDLER)sesBtn_isr, context, sesBtn_irq)) {
    	printk("SES: Interrupt mapping failed\n");
    }    
    BcmHalInterruptEnable(sesBtn_irq);
}


static unsigned int sesBtn_poll(struct file *file, struct poll_table_struct *wait)
{
#if defined(_BCM96338_) || defined(CONFIG_BCM96338)
    unsigned long gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned long *gpio_reg = &GPIO->GPIOio;
#endif
#if defined(_BCM96345_) || defined(CONFIG_BCM96345)
    unsigned short gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned short *gpio_reg = &GPIO->GPIOio;
#endif
#if defined(_BCM96348_) || defined (CONFIG_BCM96348)
    unsigned long gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned long *gpio_reg = &GPIO->GPIOio;

    if( (sesBtn_gpio & ~BP_ACTIVE_MASK) >= 32 )
    {
        gpio_mask = GPIO_NUM_TO_MASK_HIGH(sesBtn_gpio);
        gpio_reg = &GPIO->GPIOio_high;
    }
#endif 
    		
    if (!(*gpio_reg & gpio_mask)){
	return POLLIN;
    }	
    return 0;
}

static ssize_t sesBtn_read(struct file *file,  char __user *buffer, size_t count, loff_t *ppos)
{
    volatile unsigned int event=0;
    ssize_t ret=0;	

#if defined(_BCM96338_) || defined (CONFIG_BCM96338)
    unsigned long gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned long *gpio_reg = &GPIO->GPIOio;
#endif
#if defined(_BCM96345_) || defined (CONFIG_BCM96345)
    unsigned short gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned short *gpio_reg = &GPIO->GPIOio;
#endif
#if defined(_BCM96348_) || defined (CONFIG_BCM96348)
    unsigned long gpio_mask = GPIO_NUM_TO_MASK(sesBtn_gpio);
    volatile unsigned long *gpio_reg = &GPIO->GPIOio;
    
    if( (sesBtn_gpio & ~BP_ACTIVE_MASK) >= 32 )
    {
        gpio_mask = GPIO_NUM_TO_MASK_HIGH(sesBtn_gpio);
        gpio_reg = &GPIO->GPIOio_high;
    }
#endif 

    if(*gpio_reg & gpio_mask){
	BcmHalInterruptEnable(sesBtn_irq);		
	return ret;
    }	
    event = SES_EVENTS;
    __copy_to_user((char*)buffer, (char*)&event, sizeof(event));	
    BcmHalInterruptEnable(sesBtn_irq);	
    count -= sizeof(event);
    buffer += sizeof(event);
    ret += sizeof(event);	
    return ret;	
}

static void __init sesLed_mapGpio()
{	
    if( BpGetWirelessSesBtnGpio(&sesLed_gpio) == BP_SUCCESS )
    {
        printk("SES: LED GPIO 0x%x is enabled\n", sesBtn_gpio);    
    }
}

static void sesLed_ctrl(int action)
{

    //char status = ((action >> 8) & 0xff); /* extract status */
    //char event = ((action >> 16) & 0xff); /* extract event */        
    //char blinktype = ((action >> 24) & 0xff); /* extract blink type for SES_LED_BLINK  */
    
    BOARD_LED_STATE led;
    
    if(sesLed_gpio == BP_NOT_DEFINED)
        return;
    	
    action &= 0xff; /* extract led */

    //printk("blinktype=%d, event=%d, status=%d\n",(int)blinktype, (int)event, (int)status);
            	
    switch (action) 
    {
        case SES_LED_ON:
            //printk("SES: led on\n");
            led = kLedStateOn;                                          
            break;
        case SES_LED_BLINK:
            //printk("SES: led blink\n");
            led = kLedStateSlowBlinkContinues;           		
            break;
        case SES_LED_OFF:
            default:
            //printk("SES: led off\n");
            led = kLedStateOff;  						
    }	
    
    kerSysLedCtrl(kLedSes, led);
}

static void __init ses_board_init()
{
    sesBtn_mapGpio();
    sesBtn_mapIntr(0);
    sesLed_mapGpio();
}
static void __exit ses_board_deinit()
{
    if(sesBtn_irq)
        BcmHalInterruptDisable(sesBtn_irq);
}
#endif

/***************************************************************************
 * Dying gasp ISR and functions.
 ***************************************************************************/
#define KERSYS_DBG	printk

#if defined(CONFIG_BCM96345)
#define	CYCLE_PER_US	70
#elif defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)
/* The BCM6348 cycles per microsecond is really variable since the BCM6348
 * MIPS speed can vary depending on the PLL settings.  However, an appoximate
 * value of 120 will still work OK for the test being done.
 */
#define	CYCLE_PER_US	120
#endif
#define	DG_GLITCH_TO	(100*CYCLE_PER_US)
 
static void __init kerSysDyingGaspMapIntr()
{
    unsigned long ulIntr;
    	
#if defined(CONFIG_BCM96348) || defined(_BCM96348_) || defined(CONFIG_BCM96338) || defined(_BCM96338_)
    //if( BpGetAdslDyingGaspExtIntr( &ulIntr ) == BP_SUCCESS ) {
		BcmHalMapInterrupt((FN_HANDLER)kerSysDyingGaspIsr, 0, INTERRUPT_ID_DG);
		BcmHalInterruptEnable( INTERRUPT_ID_DG );
    //}
#elif defined(CONFIG_BCM96345) || defined(_BCM96345_)
    if( BpGetAdslDyingGaspExtIntr( &ulIntr ) == BP_SUCCESS ) {
        ulIntr += INTERRUPT_ID_EXTERNAL_0;
        BcmHalMapInterrupt((FN_HANDLER)kerSysDyingGaspIsr, 0, ulIntr);
        BcmHalInterruptEnable( ulIntr );
    }
#endif

} 

void kerSysSetWdTimer(ulong timeUs)
{
	TIMER->WatchDogDefCount = timeUs * (FPERIPH/1000000);
	TIMER->WatchDogCtl = 0xFF00;
	TIMER->WatchDogCtl = 0x00FF;
}

ulong kerSysGetCycleCount(void)
{
    ulong cnt; 
#ifdef _WIN32_WCE
    cnt = 0;
#else
    __asm volatile("mfc0 %0, $9":"=d"(cnt));
#endif
    return(cnt); 
}

static Bool kerSysDyingGaspCheckPowerLoss(void)
{
    ulong clk0;
    ulong ulIntr;

    ulIntr = 0;
    clk0 = kerSysGetCycleCount();

    UART->Data = 'D';
    UART->Data = '%';
    UART->Data = 'G';

#if defined(CONFIG_BCM96345)
    BpGetAdslDyingGaspExtIntr( &ulIntr );

    do {
        ulong clk1;
        
        clk1 = kerSysGetCycleCount();		/* time cleared */
	/* wait a little to get new reading */
        while ((kerSysGetCycleCount()-clk1) < CYCLE_PER_US*2)
            ;
    } while ((0 == (PERF->ExtIrqCfg & (1 << (ulIntr + EI_STATUS_SHFT)))) && ((kerSysGetCycleCount() - clk0) < DG_GLITCH_TO));

    if (PERF->ExtIrqCfg & (1 << (ulIntr + EI_STATUS_SHFT))) {	/* power glitch */
        BcmHalInterruptEnable( ulIntr + INTERRUPT_ID_EXTERNAL_0);
        KERSYS_DBG(" - Power glitch detected. Duration: %ld us\n", (kerSysGetCycleCount() - clk0)/CYCLE_PER_US);
        return 0;
    }
#elif (defined(CONFIG_BCM96348) || defined(CONFIG_BCM96338)) && !defined(VXWORKS)
    do {
        ulong clk1;
        
        clk1 = kerSysGetCycleCount();		/* time cleared */
	/* wait a little to get new reading */
        while ((kerSysGetCycleCount()-clk1) < CYCLE_PER_US*2)
            ;
     } while ((PERF->IrqStatus & (1 << (INTERRUPT_ID_DG - INTERNAL_ISR_TABLE_OFFSET))) && ((kerSysGetCycleCount() - clk0) < DG_GLITCH_TO));

    if (!(PERF->IrqStatus & (1 << (INTERRUPT_ID_DG - INTERNAL_ISR_TABLE_OFFSET)))) {
        BcmHalInterruptEnable( INTERRUPT_ID_DG );
        KERSYS_DBG(" - Power glitch detected. Duration: %ld us\n", (kerSysGetCycleCount() - clk0)/CYCLE_PER_US);
        return 0;
    }
#endif
    return 1;
}


static void kerSysDyingGaspShutdown( void )
{
//swda marked,09/30/2005
#if POWER_SAVING_DG

    kerSysSetWdTimer(1000000);
#if defined(CONFIG_BCM96345)
    PERF->blkEnables &= ~(EMAC_CLK_EN | USB_CLK_EN | CPU_CLK_EN);
#elif defined(CONFIG_BCM96348)
    PERF->blkEnables &= ~(EMAC_CLK_EN | USBS_CLK_EN | USBH_CLK_EN | SAR_CLK_EN);
#endif

#endif	//end of if 0
//swda marked end
}


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
static irqreturn_t kerSysDyingGaspIsr(int irq, void * dev_id, struct pt_regs * regs)
#else
static unsigned int kerSysDyingGaspIsr(void)
#endif
{	
    struct list_head *pos;
    CB_DGASP_LIST *tmp, *dsl = NULL;	

    if (kerSysDyingGaspCheckPowerLoss()) {        
#ifdef POWER_SAVING_DG
		GPIO->GPIODir_high |= GPIO_NUM_TO_MASK_HIGH(35);//set GPIO 35 as output
		GPIO->GPIOio_high &= (~GPIO_NUM_TO_MASK_HIGH(35));//pull GPIO 35 low to reset 5325 ethernet switch
		GPIO->GPIODir_high |= GPIO_NUM_TO_MASK_HIGH(36);//set GPIO 36 as output
		GPIO->GPIOio_high &= (~GPIO_NUM_TO_MASK_HIGH(36));//pull GPIO 36 low to reset dsp 6341
		//printk("GPIO->GPIODir_high=0x%X\n",GPIO->GPIODir_high);
		//printk("GPIO->GPIOio_high=0x%X\n",GPIO->GPIOio_high);
#endif
        /* first to turn off everything other than dsl */        
        list_for_each(pos, &g_cb_dgasp_list_head->list) {    	
            tmp = list_entry(pos, CB_DGASP_LIST, list);
    	    if(strncmp(tmp->name, "dsl", 3)) {
    	        (tmp->cb_dgasp_fn)(tmp->context); 
    	    }else {
    		dsl = tmp;    		    	
    	    }       
        }  
        
        /* now send dgasp */
        if(dsl)
            (dsl->cb_dgasp_fn)(dsl->context); 

        /* reset and shutdown system */
        kerSysDyingGaspShutdown();
    }
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
return( IRQ_HANDLED );
#else
    return( 1 );
#endif
}

static void __init kerSysInitDyingGaspHandler( void )
{
    CB_DGASP_LIST *new_node;

    if( g_cb_dgasp_list_head != NULL) {
        printk("Error: kerSysInitDyingGaspHandler: list head is not null\n");
        return;	
    }
    new_node= (CB_DGASP_LIST *)kmalloc(sizeof(CB_DGASP_LIST), GFP_KERNEL);
    memset(new_node, 0x00, sizeof(CB_DGASP_LIST));
    INIT_LIST_HEAD(&new_node->list);    
    g_cb_dgasp_list_head = new_node; 
		
} /* kerSysInitDyingGaspHandler */

static void __exit kerSysDeinitDyingGaspHandler( void )
{
    struct list_head *pos;
    CB_DGASP_LIST *tmp; 
     	
    if(g_cb_dgasp_list_head == NULL)
        return;
        
    list_for_each(pos, &g_cb_dgasp_list_head->list) {    	
    	tmp = list_entry(pos, CB_DGASP_LIST, list);
        list_del(pos);
	kfree(tmp);
    }       

    kfree(g_cb_dgasp_list_head);	
    g_cb_dgasp_list_head = NULL;
    
} /* kerSysDeinitDyingGaspHandler */

void kerSysRegisterDyingGaspHandler(char *devname, void *cbfn, void *context)
{
    CB_DGASP_LIST *new_node;

    if( g_cb_dgasp_list_head == NULL) {
        printk("Error: kerSysRegisterDyingGaspHandler: list head is null\n");	
        return;    
    }
    
    if( devname == NULL || cbfn == NULL ) {
        printk("Error: kerSysRegisterDyingGaspHandler: register info not enough (%s,%x,%x)\n", devname, (unsigned int)cbfn, (unsigned int)context);	    	
        return;
    }
       
    new_node= (CB_DGASP_LIST *)kmalloc(sizeof(CB_DGASP_LIST), GFP_KERNEL);
    memset(new_node, 0x00, sizeof(CB_DGASP_LIST));    
    INIT_LIST_HEAD(&new_node->list);
    strncpy(new_node->name, devname, IFNAMSIZ);
    new_node->cb_dgasp_fn = (cb_dgasp_t)cbfn;
    new_node->context = context;
    list_add(&new_node->list, &g_cb_dgasp_list_head->list);
    
    //printk("dgasp: kerSysRegisterDyingGaspHandler: %s registered \n", devname);
        	
} /* kerSysRegisterDyingGaspHandler */

void kerSysDeregisterDyingGaspHandler(char *devname)
{
    struct list_head *pos;
    CB_DGASP_LIST *tmp;    
    
    if(g_cb_dgasp_list_head == NULL) {
        printk("Error: kerSysDeregisterDyingGaspHandler: list head is null\n");
        return;	
    }

    if(devname == NULL) {
        printk("Error: kerSysDeregisterDyingGaspHandler: devname is null\n");
        return;	
    }
    
    printk("kerSysDeregisterDyingGaspHandler: %s is deregistering\n", devname);

    list_for_each(pos, &g_cb_dgasp_list_head->list) {    	
    	tmp = list_entry(pos, CB_DGASP_LIST, list);
    	if(!strcmp(tmp->name, devname)) {
            list_del(pos);
	    kfree(tmp);
	    printk("kerSysDeregisterDyingGaspHandler: %s is deregistered\n", devname);
	    return;
	}
    }	
    printk("kerSysDeregisterDyingGaspHandler: %s not (de)registered\n", devname);
	
} /* kerSysDeregisterDyingGaspHandler */

//swda add
int kerSysSetUpgradeState(int state)
{
	is_update_flash = state;
}
int kerSysGetUpgradeState(void)
{
	return is_update_flash;
}
//swda add end

/***************************************************************************
 * MACRO to call driver initialization and cleanup functions.
 ***************************************************************************/
module_init( brcm_board_init );
module_exit( brcm_board_cleanup );

EXPORT_SYMBOL(kerSysNvRamGet);
EXPORT_SYMBOL(dumpaddr);
EXPORT_SYMBOL(kerSysGetMacAddress);
EXPORT_SYMBOL(kerSysReleaseMacAddress);
EXPORT_SYMBOL(kerSysGetSdramSize);
EXPORT_SYMBOL(kerSysLedCtrl);
EXPORT_SYMBOL(kerSysLedRegisterHwHandler);
EXPORT_SYMBOL(kerSysGetProductName);//swda add
EXPORT_SYMBOL(kerSysGetPsiSize);//swda add
EXPORT_SYMBOL(kerSysGetEthWlanMacAddress);//swda add
EXPORT_SYMBOL(kerSysGetWlan4318MacAddress);//swda add
EXPORT_SYMBOL(kerSysGetSpromVer);//swda add
EXPORT_SYMBOL(kerSysGetSpromSavedFlag);//swda add
EXPORT_SYMBOL(kerSysSetSpromSavedFlag);//swda add
EXPORT_SYMBOL(kerSysGetSpromTable);//swda add
EXPORT_SYMBOL(kerSysSetSpromTable);//swda add
EXPORT_SYMBOL(kerSysGetChipId);//swda add
EXPORT_SYMBOL(BpGetBoardIds);
EXPORT_SYMBOL(BpGetSdramSize);
EXPORT_SYMBOL(BpGetPsiSize);
EXPORT_SYMBOL(BpGetEthernetMacInfo);
EXPORT_SYMBOL(BpGetRj11InnerOuterPairGpios);
EXPORT_SYMBOL(BpGetPressAndHoldResetGpio);
EXPORT_SYMBOL(BpGetVoipResetGpio);
EXPORT_SYMBOL(BpGetVoipIntrGpio);
EXPORT_SYMBOL(BpGetPcmciaResetGpio);
EXPORT_SYMBOL(BpGetRtsCtsUartGpios);
EXPORT_SYMBOL(BpGetAdslLedGpio);
EXPORT_SYMBOL(BpGetDiagLedGpio);
EXPORT_SYMBOL(BpGetPSTNGpio);
EXPORT_SYMBOL(BpGetAdslFailLedGpio);
EXPORT_SYMBOL(BpGetWirelessLedGpio);
EXPORT_SYMBOL(BpGetUsbLedGpio);
EXPORT_SYMBOL(BpGetHpnaLedGpio);
EXPORT_SYMBOL(BpGetWanDataLedGpio);
EXPORT_SYMBOL(BpGetPppLedGpio);
EXPORT_SYMBOL(BpGetPppFailLedGpio);
EXPORT_SYMBOL(BpGetVoipLedGpio);
EXPORT_SYMBOL(BpGetWirelessExtIntr);
EXPORT_SYMBOL(BpGetAdslDyingGaspExtIntr);
EXPORT_SYMBOL(BpGetVoipExtIntr);
EXPORT_SYMBOL(BpGetHpnaExtIntr);
EXPORT_SYMBOL(BpGetHpnaChipSelect);
EXPORT_SYMBOL(BpGetVoipChipSelect);
EXPORT_SYMBOL(BpGetWirelessSesBtnGpio);
EXPORT_SYMBOL(BpGetWirelessSesExtIntr);
EXPORT_SYMBOL(BpGetWirelessSesLedGpio);
EXPORT_SYMBOL(kerSysRegisterDyingGaspHandler);
EXPORT_SYMBOL(kerSysDeregisterDyingGaspHandler);
EXPORT_SYMBOL(kerSysGetCycleCount);
EXPORT_SYMBOL(kerSysSetWdTimer);
EXPORT_SYMBOL(kerSysWakeupMonitorTask);
EXPORT_SYMBOL(kerSysGetUpgradeState);//swda add,06/27/2006
