/************************************************************************/
/*                                                                      */
/*  AMD CFI Enabled Flash Memory Drivers                                */
/*  File name: CFIFLASH.C                                               */
/*  Revision:  1.0  5/07/98                                             */
/*                                                                      */
/* Copyright (c) 1998 ADVANCED MICRO DEVICES, INC. All Rights Reserved. */
/* This software is unpublished and contains the trade secrets and      */
/* confidential proprietary information of AMD. Unless otherwise        */
/* provided in the Software Agreement associated herewith, it is        */
/* licensed in confidence "AS IS" and is not to be reproduced in whole  */
/* or part by any means except for backup. Use, duplication, or         */
/* disclosure by the Government is subject to the restrictions in       */
/* paragraph (b) (3) (B) of the Rights in Technical Data and Computer   */
/* Software clause in DFAR 52.227-7013 (a) (Oct 1988).                  */
/* Software owned by                                                    */
/* Advanced Micro Devices, Inc.,                                        */
/* One AMD Place,                                                       */
/* P.O. Box 3453                                                        */
/* Sunnyvale, CA 94088-3453.                                            */
/************************************************************************/
/*  This software constitutes a basic shell of source code for          */
/*  programming all AMD Flash components. AMD                           */
/*  will not be responsible for misuse or illegal use of this           */
/*  software for devices not supported herein. AMD is providing         */
/*  this source code "AS IS" and will not be responsible for            */
/*  issues arising from incorrect user implementation of the            */
/*  source code herein. It is the user's responsibility to              */
/*  properly design-in this source code.                                */
/*                                                                      */ 
/************************************************************************/                        
#ifdef _CFE_                                                
#include "lib_types.h"
#include "lib_printf.h"
#include "lib_string.h"
#include "cfe_timer.h"
#define printk  printf
#else       // linux
#include <linux/param.h>
#include <linux/sched.h>
#include <linux/timer.h>
#endif

//#define DEBUG_FLASH 

#include "cfiflash.h"


static UINT16 flash_get_device_id(void);
static int flash_get_cfi(struct cfi_query *query, UINT16 *cfi_struct, int flashFamily);
static int flash_write(WORD sector, int offset, byte *buf, int nbytes);
//void flash_command(int command, WORD sector, int offset, UINT16 data);

//swda add,06/27/2006
static int DiagLedBlinking(void);
#define T150ms              ((150 * HZ) / 1000)   // ~150 ms
#define T100ms				(HZ / 10)	//~100ms
#define T50ms              ((50 * HZ) / 1000)   // ~50 ms
#define T25ms              ((25 * HZ) / 1000)   // ~25 ms
//swda add end

/*********************************************************************/
/* 'meminfo' should be a pointer, but most C compilers will not      */
/* allocate static storage for a pointer without calling             */
/* non-portable functions such as 'new'.  We also want to avoid      */
/* the overhead of passing this pointer for every driver call.       */
/* Systems with limited heap space will need to do this.             */
/*********************************************************************/
struct flashinfo meminfo; /* Flash information structure */
static int flashFamily = FLASH_UNDEFINED;
static int totalSize = 0;
static struct cfi_query query;
UINT16 device_id;
UINT16 man_id;
static UINT16 cfi_data_struct_29W160[] = {
    0x0020, 0x0049, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
    0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
    0x0051, 0x0052, 0x0059, 0x0002, 0x0000, 0x0040, 0x0000, 0x0000,
    0x0000, 0x0000, 0x0000, 0x0027, 0x0036, 0x0000, 0x0000, 0x0004,
    0x0000, 0x000a, 0x0000, 0x0004, 0x0000, 0x0003, 0x0000, 0x0015,
    0x0002, 0x0000, 0x0000, 0x0000, 0x0004, 0x0000, 0x0000, 0x0040,
    0x0000, 0x0001, 0x0000, 0x0020, 0x0000, 0x0000, 0x0000, 0x0080,
    0x0000, 0x001e, 0x0000, 0x0000, 0x0001, 0xffff, 0xffff, 0xffff,
    0x0050, 0x0052, 0x0049, 0x0031, 0x0030, 0x0000, 0x0002, 0x0001,
    0x0001, 0x0004, 0x0000, 0x0000, 0x0000, 0xffff, 0xffff, 0x0002,
    0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
    0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
    0xffff, 0x0888, 0x252b, 0x8c84, 0x7dbc, 0xffff, 0xffff, 0xffff,
    0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
    0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff,
    0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff
};

#if defined(_BCM96338_) || defined(CONFIG_BCM96338)
unsigned long get_flash_base(unsigned long size)
{
	static unsigned long flash_base = FLASH_BASE;
	if (size > 0x10000000) {
		flash_base = size;
	}
	else if (size) {
		unsigned long fsize = 8 * 1024;
		unsigned long fbase = 0x20000000;
		unsigned long frmap;

		for (frmap = 0; frmap < 17; frmap++) {
			if (size == fsize) {
				fbase -= fsize;
				if (fbase > 0x1fc00000) fbase = 0x1fc00000;
				*(volatile unsigned long *)0xfffe3160 = fbase | frmap;
				flash_base = (unsigned long)(fbase|0xa0000000);
				break;
			}
			fsize <<= 1;
		}
	}

	return flash_base;
}	
#else
/* denny 08/17/04' add, reconfigure and obtin flash base address */
unsigned long get_flash_base(unsigned long size)
{
	static unsigned long flash_base = FLASH_BASE;
	
	if (size > 0x10000000) {
		flash_base = size;
	}
	else if (size) {
		unsigned long fsize = 8 * 1024;
		unsigned long fbase = 0x20000000;
		unsigned long frmap;

		for (frmap = 0; frmap < 17; frmap++) {
			if (size == fsize) {
				fbase -= fsize;
				if (fbase > 0x1fc00000) fbase = 0x1fc00000;
				*(volatile unsigned long *)0xfffe2000 = fbase | frmap;
				flash_base = (unsigned long)(fbase|0xa0000000);
				break;
			}
			fsize <<= 1;
		}
	}

	return flash_base;
}
#endif

#ifdef CONFIG_CS1 
unsigned long get_flash2_base(unsigned long size)
{
	static unsigned long flash2_base = FLASH_BASE;

	if (size == -1) {
		*(volatile unsigned long *)0xfffe2008 = 0;
		*(volatile unsigned long *)0xfffe200c = 0;
		flash2_base = FLASH_BASE;
	}    
	else if (size > 0xa0000000) {
		flash2_base = size;
	}
	else if (size) {
		unsigned long fsize = 8 * 1024;
		unsigned long fbase = *(volatile unsigned long *)0xfffe2000 & ~0xf;
		unsigned long frmap;

		for (frmap = 0; frmap < 17; frmap++) {
			if (size == fsize) {
				fbase -= fsize;
				*(volatile unsigned long *)0xfffe200c = *(volatile unsigned long *)0xfffe2004;
				*(volatile unsigned long *)0xfffe2008 = fbase | frmap;
				flash2_base = (unsigned long)(fbase|0xa0000000);
				break;
			}
			fsize <<= 1;
		}
	}

	return flash2_base;
}
#endif
/* end */

/*********************************************************************/
/* Init_flash is used to build a sector table from the information   */
/* provided through the CFI query.  This information is translated   */
/* from erase_block information to base:offset information for each  */
/* individual sector. This information is then stored in the meminfo */
/* structure, and used throughout the driver to access sector        */
/* information.                                                      */
/*                                                                   */
/* This is more efficient than deriving the sector base:offset       */
/* information every time the memory map switches (since on the      */
/* development platform can only map 64k at a time).  If the entire  */
/* flash memory array can be mapped in, then the addition static     */
/* allocation for the meminfo structure can be eliminated, but the   */
/* drivers will have to be re-written.                               */
/*                                                                   */
/* The meminfo struct occupies 653 bytes of heap space, depending    */
/* on the value of the define MAXSECTORS.  Adjust to suit            */
/* application                                                       */ 
/*********************************************************************/
byte flash_init(void)
{
    int i=0, j=0, count=0;
    int basecount=0L;
    //UINT16 device_id;
    int flipCFIGeometry = FALSE;

    /* First, assume
    * a single 8k sector for sector 0.  This is to allow
    * the system to perform memory mapping to the device,
    * even though the actual physical layout is unknown.
    * Once mapped in, the CFI query will produce all
    * relevant information.
    */
    meminfo.addr = 0L;
    meminfo.areg = 0;
    meminfo.nsect = 1;
    meminfo.bank1start = 0;
    meminfo.bank2start = 0;
    
    meminfo.sec[0].size = 8192;
    meminfo.sec[0].base = 0x00000;
    meminfo.sec[0].bank = 1;
        
    flash_command(FLASH_RESET, 0, 0, 0);

    device_id = flash_get_device_id();

    switch (device_id) {
        case ID_I28F160C3B:
        case ID_I28F320C3B:
        case ID_I28F160C3T:
        case ID_I28F320C3T:
            flashFamily = FLASH_INTEL;
            break;
        case ID_AM29DL800B:
        case ID_AM29LV800B:
//        case ID_AM29LV400B:   
        case ID_AM29LV160B:
        case ID_AM29LV320B:
        case ID_MX29LV320AB:
        case ID_AM29LV320MB:
        case ID_AM29DL800T:
        case ID_AM29LV800T:
        case ID_AM29LV160T:
        case ID_AM29LV320T:
        case ID_MX29LV320AT:
        case ID_AM29LV320MT:
        case ID_W19L320ST:
        case ID_W19L320SB:
        case ID_K8D6316UB:
        case ID_K8D3216UB:
        case ID_K8D1716UB:
        case ID_K8D1716UT:
        case ID_K8D3216UT:
        case 0x43:
        case ID_STM29W320EB:
        case ID_STM29W640FB:
            flashFamily = FLASH_AMD;
            break;
        case ID_SST39VF1601:
		case ID_SST39VF3201:
            flashFamily = FLASH_SST;
            break;
        default:
            printk("Flash memory not supported!  Device id = %x\n", device_id);
            return -1;           
    }
if (device_id == 0x43){
  	 query.num_erase_blocks = 4;
  	 query.erase_block[0].num_sectors = 1;
  	 query.erase_block[0].sector_size = 16384;
  	 query.erase_block[1].num_sectors = 2;
  	 query.erase_block[1].sector_size = 8192;
  	 query.erase_block[2].num_sectors = 1;
  	 query.erase_block[2].sector_size = 32768;
  	 query.erase_block[3].num_sectors = 31;
  	 query.erase_block[3].sector_size = 65536;
}	
else {
    if (flash_get_cfi(&query, 0, flashFamily) == -1) {
        switch(device_id) {
        case ID_AM29LV160T:
        case ID_AM29LV160B:
            flash_get_cfi(&query, cfi_data_struct_29W160, flashFamily);
            break;
        default:
            printk("CFI data structure not found. Device id = %x\n", device_id);
            return -1;           
        }
    }
}

#if defined(_BCM96338_) || defined(CONFIG_BCM96338)
	/* reconfigure CS0 and flash base */
	get_flash_base(query.device_size);
	printk("man/device id %02x/%02x\n",(man_id&0x00ff),device_id);
#else

/* denny 08/17/04' add, reconfigure the CS0 and CS1 for flash */
    {
    	
    	volatile unsigned int *cs = (volatile unsigned int *)0xfffe2000;

	/* reconfigure CS0 and flash base */
	get_flash_base(query.device_size);
		
#ifdef CONFIG_CS1
         struct cfi_query query2;
	/* try to detection CS1 for flash base */
	get_flash_base(get_flash2_base(0x100000));
	if (flash_get_cfi(&query2, 0,flashFamily) == -1) {
		get_flash2_base(-1);
		get_flash_base(query.device_size);
	}
	else {
		/* reconfigure CS0 and flash base */
		get_flash_base(query.device_size);
		/* set flash base value */
	    	get_flash_base(get_flash2_base(query2.device_size));
	    	#if DEBUG_FLASH 
		printk("query.num_erase_blocks = %x\n", query.num_erase_blocks);
		printk("query.device_size = %x\n", query.device_size);
		printk("query2.num_erase_blocks = %x\n", query2.num_erase_blocks);
		printk("query2.device_size = %x\n", query2.device_size);
		printk("flash_base = %x\n", get_flash_base(0));
		#endif
		for (i=0; i<query.num_erase_blocks; i++)
			memcpy(&query.erase_block[i+query2.num_erase_blocks], &query.erase_block[i], sizeof(query.erase_block[0]));
		for (i=0; i<query2.num_erase_blocks; i++)
			memcpy(&query.erase_block[i], &query2.erase_block[i], sizeof(query.erase_block[0]));
		query.num_erase_blocks += query2.num_erase_blocks;
		query.device_size += query2.device_size;
	}
#endif

        printk("%02x/%02x System PLL( MPI clock 0x%x)\n",(man_id&0x00ff),device_id,*(volatile unsigned int*)(0xfffe0008) & 0x00000018);
	if (device_id == ID_AM29LV320MB || device_id == ID_AM29LV320MT)
	 if ((*(volatile unsigned int*)(0xfffe0008) & 0x00000018) == 0) /* 50Mhz sysclock */
            *(volatile unsigned int*)(0xfffe2004) |= 0x0000000e; /* if spansion flash CS0 wait time -> Three */
         else
            *(volatile unsigned int*)(0xfffe2004) |= 0x0000000e; /* 33Mhz sysclock */
	#if DEBUG_FLASH  
	printk("query.num_erase_blocks = %x\n", query.num_erase_blocks);
	printk("query.device_size = %x\n", query.device_size);
	printk("flash_base = %x\n", get_flash_base(0));
	#endif
	printk("Flash Config: ");
	printk("CS0(%x,%x),", cs[0], cs[1]);
	if (cs[2]) printk("CS1(%x,%x) ", cs[2], cs[3]);
	printk("Base(%x),", (unsigned int)get_flash_base(0));
	printk("Size(%dMB)\n", (0x20000000-(get_flash_base(0)-0xa0000000))>>20);
    }
/* end */

    count=0;basecount=0L;

    for (i=0; i<query.num_erase_blocks; i++) {
        count += query.erase_block[i].num_sectors;
    }
    
    meminfo.nsect = count;
    count=0;
#endif
    // need to determine if it top or bottom boot here
    switch (device_id)
    {
        case ID_AM29DL800B:
        case ID_AM29LV800B:
//        case ID_AM29LV400B:   
        case ID_AM29LV160B:
        case ID_AM29LV320B:
        case ID_MX29LV320AB:
        case ID_AM29LV320MB:
        case ID_I28F160C3B:
        case ID_I28F320C3B:
        case ID_W19L320SB:
        case ID_I28F160C3T:
        case ID_I28F320C3T:
        case ID_K8D1716UB:
        case ID_K8D3216UB:
        case ID_K8D6316UB:
		case ID_SST39VF1601:
		case ID_SST39VF3201:
        case 0x43: // for Toshiba 58FVB160FT-85
        case ID_STM29W320EB:
        case ID_STM29W640FB:
            flipCFIGeometry = FALSE;
            break;
        case ID_AM29DL800T:
        case ID_AM29LV800T:
        case ID_AM29LV160T:
        case ID_AM29LV320T:
        case ID_MX29LV320AT:
        case ID_AM29LV320MT:
        case ID_K8D1716UT:
        case ID_K8D3216UT:
        case ID_W19L320ST:
            flipCFIGeometry = TRUE;
            break;
        default:
            printk("Flash memory not supported!  Device id = %x\n", device_id);
            return -1;           
    }

    count=0;basecount=0L;

    if (!flipCFIGeometry)
    {
       for (i=0; i<query.num_erase_blocks; i++) {
            for(j=0; j<query.erase_block[i].num_sectors; j++) {
                meminfo.sec[count].size = (int) query.erase_block[i].sector_size;
                meminfo.sec[count].base = (int) basecount;
                basecount += (int) query.erase_block[i].sector_size;
                count++;
            }
        }
    }
    else
    {
        for (i = (query.num_erase_blocks - 1); i >= 0; i--) {
            for(j=0; j<query.erase_block[i].num_sectors; j++) {
                meminfo.sec[count].size = (int) query.erase_block[i].sector_size;
                meminfo.sec[count].base = (int) basecount;
                basecount += (int) query.erase_block[i].sector_size;
				count++;
            }
        }
    }

    meminfo.nsect = count;
    totalSize = meminfo.sec[count-1].base + meminfo.sec[count-1].size;
    return (0);
}

/*********************************************************************/
/* Flash_sector_erase_int() is identical to flash_sector_erase(),    */
/* except it will wait until the erase is completed before returning */
/* control to the calling function.  This can be used in cases which */
/* require the program to hold until a sector is erased, without     */
/* adding the wait check external to this function.                  */
/*********************************************************************/
byte flash_sector_erase_int(WORD sector)
{
    int i;

    for( i = 0; i < 3; i++ ) {
        flash_command(FLASH_SERASE, sector, 0, 0);
        if (flash_wait(sector, 0, 0xffff) == STATUS_READY)
            break;
    }

    return(1);
}

/*********************************************************************/
/* flash_read_buf() reads buffer of data from the specified          */
/* offset from the sector parameter.                                 */
/*********************************************************************/
int flash_read_buf(WORD sector, int offset,
                        byte *buffer, int numbytes)
{
    byte *fwp;

    fwp = (byte *)flash_get_memptr(sector);

	while (numbytes) {
		*buffer++ = *(fwp + offset);
		numbytes--;
		fwp++;
    }

    return (1);
}

/*********************************************************************/
/* flash_write_buf() utilizes                                        */
/* the unlock bypass mode of the flash device.  This can remove      */
/* significant overhead from the bulk programming operation, and     */
/* when programming bulk data a sizeable performance increase can be */
/* observed.                                                         */
/*********************************************************************/
int flash_write_buf(WORD sector, int offset, byte *buffer, int numbytes)
{
    int ret = -1;
    int i;
    unsigned char *p = flash_get_memptr(sector) + offset;

    /* After writing the flash block, compare the contents to the source
     * buffer.  Try to write the sector successfully up to three times.
     */
    for( i = 0; i < 3; i++ ) {
        ret = flash_write(sector, offset, buffer, numbytes);
        if( !memcmp( p, buffer, numbytes ) )
            break;
        /* Erase and try again */
        flash_sector_erase_int(sector);
        ret = -1;
    }

    if( ret == -1 )
        printk( "Flash write error.  Verify failed\n" );

    return( ret );
}

/*********************************************************************/
/* Usefull funtion to return the number of sectors in the device.    */
/* Can be used for functions which need to loop among all the        */
/* sectors, or wish to know the number of the last sector.           */
/*********************************************************************/
int flash_get_numsectors(void)
{
    return meminfo.nsect;
}

/*********************************************************************/
/* flash_get_sector_size() is provided for cases in which the size   */
/* of a sector is required by a host application.  The sector size   */
/* (in bytes) is returned in the data location pointed to by the     */
/* 'size' parameter.                                                 */
/*********************************************************************/
int flash_get_sector_size(WORD sector)
{
    return meminfo.sec[sector].size;
}

/*********************************************************************/
/* The purpose of flash_get_memptr() is to return a memory pointer   */
/* which points to the beginning of memory space allocated for the   */
/* flash.  All function pointers are then referenced from this       */
/* pointer. 							     */
/*                                                                   */
/* Different systems will implement this in different ways:          */
/* possibilities include:                                            */
/*  - A direct memory pointer                                        */
/*  - A pointer to a memory map                                      */
/*  - A pointer to a hardware port from which the linear             */
/*    address is translated                                          */
/*  - Output of an MMU function / service                            */
/*                                                                   */
/* Also note that this function expects the pointer to a specific    */
/* sector of the device.  This can be provided by dereferencing      */
/* the pointer from a translated offset of the sector from a         */
/* global base pointer (e.g. flashptr = base_pointer + sector_offset)*/
/*                                                                   */
/* Important: Many AMD flash devices need both bank and or sector    */
/* address bits to be correctly set (bank address bits are A18-A16,  */
/* and sector address bits are A18-A12, or A12-A15).  Flash parts    */
/* which do not need these bits will ignore them, so it is safe to   */
/* assume that every part will require these bits to be set.         */
/*********************************************************************/
unsigned char *flash_get_memptr(WORD sector)
{
	unsigned char *memptr = (unsigned char*)(FLASH_BASE_ADDR_REG + meminfo.sec[sector].base);

	return (memptr);
}

/*********************************************************************/
/* The purpose of flash_get_blk() is to return a the block number    */
/* for a given memory address.                                       */
/*********************************************************************/
int flash_get_blk(int addr)
{
    int blk_start, i;
    int last_blk = flash_get_numsectors();
    int relative_addr = addr - (int) FLASH_BASE_ADDR_REG;

    for(blk_start=0, i=0; i < relative_addr && blk_start < last_blk; blk_start++)
        i += flash_get_sector_size(blk_start);

    if( i > relative_addr )
    {
        blk_start--;        // last blk, dec by 1
    }
    else
        if( blk_start == last_blk )
        {
            printk("Address is too big.\n");
            blk_start = -1;
        }

    return( blk_start );
}

/************************************************************************/
/* The purpose of flash_get_total_size() is to return the total size of */
/* the flash                                                            */
/************************************************************************/
int flash_get_total_size()
{
    return totalSize;
}

/*********************************************************************/
/* Flash_command() is the main driver function.  It performs         */
/* every possible command available to AMD B revision                */
/* flash parts. Note that this command is not used directly, but     */
/* rather called through the API wrapper functions provided below.   */
/*********************************************************************/
void flash_command(int command, WORD sector, int offset, UINT16 data)
{
    volatile UINT16 *flashptr;
    volatile UINT16 *flashbase;

    flashptr = (UINT16 *) flash_get_memptr(sector);
    flashbase = (UINT16 *) flash_get_memptr(0);
    
    switch (flashFamily) {
    case FLASH_UNDEFINED:
        /* These commands should work for AMD, Intel and SST flashes */
        switch (command) {
        case FLASH_RESET:
            flashptr[0] = 0xF0;
            flashptr[0] = 0xFF;
            break;
        case FLASH_READ_ID:
			flashptr[0x5555] = 0xAA;       /* unlock 1 */
            flashptr[0x2AAA] = 0x55;       /* unlock 2 */
            flashptr[0x5555] = 0x90;
            break;
        default:
            break;
        }
        break;
    case FLASH_AMD:
        switch (command) {
        case FLASH_RESET:
            flashptr[0] = 0xF0;
            break;
        case FLASH_READ_ID:
            flashptr[0x555] = 0xAA;       /* unlock 1 */
            flashptr[0x2AA] = 0x55;       /* unlock 2 */
            flashptr[0x555] = 0x90;
            break;
        case FLASH_CFIQUERY:
            flashptr[0x55] = 0x98;
            break;
        case FLASH_UB:
            flashptr[0x555] = 0xAA;       /* unlock 1 */
            flashptr[0x2AA] = 0x55;       /* unlock 2 */
            flashptr[0x555] = 0x20;
            break;
        case FLASH_PROG:
            flashptr[0] = 0xA0;
            flashptr[offset/2] = data;
            break;
        case FLASH_UBRESET:
            flashptr[0] = 0x90;
            flashptr[0] = 0x00;
            break;
        case FLASH_SERASE:
            flashptr[0x555] = 0xAA;       /* unlock 1 */
            flashptr[0x2AA] = 0x55;       /* unlock 2 */
            flashptr[0x555] = 0x80;
            flashptr[0x555] = 0xAA;
            flashptr[0x2AA] = 0x55;
            flashptr[0] = 0x30;
            break;
        default:
            break;
        }
        break;
    case FLASH_INTEL:
        switch (command) {
        case FLASH_RESET:
            flashptr[0] = 0xFF;
            break;
        case FLASH_READ_ID:
            flashptr[0] = 0x90;
            break;
        case FLASH_CFIQUERY:
            flashptr[0] = 0x98;
            break;
        case FLASH_PROG:
            flashptr[0] = 0x40;
            flashptr[offset/2] = data;
            break;
        case FLASH_SERASE:
            flashptr[0] = 0x60;
            flashptr[0] = 0xD0;
            flashptr[0] = 0x20;
            flashptr[0] = 0xD0;
            break;
        default:
            break;
        }
        break;
    case FLASH_SST:
        switch (command) {
        case FLASH_RESET:
            flashbase[0x5555] = 0xAA;       /* unlock 1 */
            flashbase[0x2AAA] = 0x55;       /* unlock 2 */
            flashbase[0x5555] = 0xf0;
            break;
        case FLASH_READ_ID:
            flashbase[0x5555] = 0xAA;       /* unlock 1 */
            flashbase[0x2AAA] = 0x55;       /* unlock 2 */
            flashbase[0x5555] = 0x90;
            break;
        case FLASH_CFIQUERY:
            flashbase[0x5555] = 0xAA;       /* unlock 1 */
            flashbase[0x2AAA] = 0x55;       /* unlock 2 */
            flashbase[0x5555] = 0x98;
            break;
        case FLASH_UB:
            break;
        case FLASH_PROG:
            flashbase[0x5555] = 0xAA;       /* unlock 1 */
            flashbase[0x2AAA] = 0x55;       /* unlock 2 */
            flashbase[0x5555] = 0xa0;
            flashptr[offset/2] = data;
            break;
        case FLASH_UBRESET:
            break;
        case FLASH_SERASE:
            flashbase[0x5555] = 0xAA;       /* unlock 1 */
            flashbase[0x2AAA] = 0x55;       /* unlock 2 */
            flashbase[0x5555] = 0x80;
            flashbase[0x5555] = 0xAA;
            flashbase[0x2AAA] = 0x55;
            flashptr[0] = 0x30;
            break;
        default:
            break;
        }
        break;
    default:
        break;
    }
}

/*********************************************************************/
/* flash_write extends the functionality of flash_program() by       */
/* providing an faster way to program multiple data words, without   */
/* needing the function overhead of looping algorithms which         */
/* program word by word.  This function utilizes fast pointers       */
/* to quickly loop through bulk data.                                */
/*********************************************************************/
static int flash_write(WORD sector, int offset, byte *buf, int nbytes)
{
    UINT16 *src;
    src = (UINT16 *)buf;

    if ((nbytes | offset) & 1) {
        return -1;
    }
    if (device_id == 0x43) {
	volatile UINT16* flashptr;
	volatile UINT16* dst;
	int stat = STATUS_READY ;
	int retry = 0, retried = 0;
	
	flashptr = (UINT16 *)flash_get_memptr(sector);

	dst = &flashptr[offset/2];   /* (byte offset) */
   
again:

#if 0    
    	/* Check to see if we're in unlock bypass mode */
    	flashptr[0] = 0xF0;  /* reset device to read mode */
    
    	if ((stat=flash_wait(sector)) != STATUS_READY) {
    	  printk("1.error\n");
        	return (byte*)src - buf;
    	}
#else
	/* reset device to read mode */
	flash_command(FLASH_RESET, 0, 0, 0);
#endif
    
    	while (nbytes > 0) {
       
		//swda add,06/27/2006
		if ( (nbytes%1024) == 0 ) {
			DiagLedBlinking();
		}
		//swda add end    
		   
        flashptr[0x555] = 0xAA;      /* unlock 1 */
        flashptr[0x2AA] = 0x55;      /* unlock 2 */

        flashptr[0x555] = 0xA0;
        *dst = *src;

        if ((stat=flash_wait(sector,offset,*src)) != STATUS_READY) {
    	    printk("2.error\n");
            break;
        }
        dst++;
        src++;
        nbytes -= 2;
        offset += 2;
    	}

    	if (stat != STATUS_READY || nbytes != 0) {
        	if (retry-- > 0) {
            	++retried;
#if 0
            	--dst, --src;   /* back up */
#endif
            	goto again;     /* and retry the last word */
        	}
      
     	flash_command(FLASH_RESET,sector,0,0);
    	}
      

    } else if (device_id == ID_AM29LV320MB && ((man_id & 0x00ff) != 0xda)) { 
    	
	volatile UINT16* flashptr;
	volatile UINT16* dst;
	int stat = STATUS_READY ;
	UINT16 d1,d2;
	UINT16* retrydst,retrysrc;
	int retry = 3, retried = 0, i=0, k=0 ,retryoffset ;
	//printk("\nsector %d write %d bytes to offset %d\n",sector,nbytes,offset);
	//printk("...........................\n");
	//printk("..\n");

	flashptr = (UINT16 *)flash_get_memptr(sector);

	dst = &flashptr[offset/2];   /* (byte offset) */
   
aa:
	flash_command(FLASH_RESET, 0, 0, 0);
   
    	while (nbytes > 0) {

        retrydst = dst;
        retrysrc = src;
        retryoffset = offset;
        
		//swda add,06/27/2006
		if ( (nbytes%1024) == 0 ) {
			DiagLedBlinking();
		}
		//swda add end       
		      
        flashptr[0x555] = 0xAA;      /* unlock 1 */
        flashptr[0x2AA] = 0x55;      /* unlock 2 */
       
        flashptr[0] = 0x25;
        k = (nbytes>32) ? 15 : nbytes/2-1;

        flashptr[0] = k;        
        for ( i=0; i < k; i++) { *dst=*src; dst++;src++; offset+=2; } /*total k write*/
        *dst = *src; /*k+1 write, src,dst,offset point to current pos should be written */
        
        flashptr[0] = 0x29; /* program to flash confirm */
        //udelay(5);   
#if 1
 	do {
            d1 = flashptr[offset/2];    /* read data */
            if ((d1 & 0x80) == (*src & 0x80)) {   /* check DQ7 ok */
                stat = STATUS_READY;
                goto next;
            }
            i++;
        } while ( ((d1 & 0x20)!=0x20) && ((d1 & 0x02)!=0x02) );

        d1 = flashptr[offset/2];    /* read data */
        if ((d1 & 0x80) != (*src & 0x80))   /* check DQ7 ok */
           {
            printk("CHECK Time_Out\n");
            stat = STATUS_TIMEOUT;
            break;
           }
        else stat = STATUS_READY;
#else
        do {
            d1 = *flashptr;        /* read data */
            d2 = d1 ^ *flashptr;   /* read it again and see what toggled */
            if (d2 == 0) {          /* no toggles, nothing's happening */
                stat = STATUS_READY;
                goto next;
            }
        } while (!(d2 & 0x20));

        d1 = *flashptr;        /* read data */
        d2 = d1 ^ *flashptr;   /* read it again and see what toggled */

        if (d2 != 0) {
            stat=STATUS_TIMEOUT;
            break;
        }
#endif        
next:           	        
        dst++; /* next batch start to write */
        src++; /* next batch start to read  */
        nbytes -= 32;
        offset += 2 ;
    	}

    	if (stat != STATUS_READY || nbytes > 0) {
    	   UINT16 data;
    	   data = flashptr[offset/2];
    	   if ((data & 0x02) == 0x02) {	
             flashptr[0x555] = 0xaa;
             flashptr[0x2aa] = 0x55;
             flashptr[0x555] = 0xf0;
             printk("abort to write to buffer @ 0x%lx =0x%x\n", offset,data);
           }
           else {
     	      flash_command(FLASH_RESET,sector,0,0);
     	      printk("fail to write to buffer @ 0x%lx =0x%x\n", offset,data);
     	   }  
    	}
    	else { flash_command(FLASH_RESET,sector,0,0);}
      
    	     
    
    }
    else {

    flash_command(FLASH_UB, 0, 0, 0);
    while (nbytes > 0) {

		//swda add,06/27/2006
		if ( (nbytes%1024) == 0 ) {
			DiagLedBlinking();
		}
		//swda add end       

        flash_command(FLASH_PROG, sector, offset, *src);
        	if (flash_wait(sector, offset, *src) != STATUS_READY) {
        		printk("break not ready\n");
            break;
        	}
        offset +=2;
        nbytes -=2;
        src++;
    }
    flash_command(FLASH_UBRESET, 0, 0, 0);
    }    
    return (byte*)src - buf;
}

/*********************************************************************/
/* flash_wait utilizes the DQ6, DQ5, and DQ2 polling algorithms      */
/* described in the flash data book.  It can quickly ascertain the   */
/* operational status of the flash device, and return an             */
/* appropriate status code (defined in flash.h)                      */
/*********************************************************************/
int flash_wait(WORD sector, int offset, UINT16 data)
{
    volatile UINT16 *flashptr; /* flash window */
    UINT16 d1;
    int i1 = 0;//swda add,06/27/2006

    flashptr = (UINT16 *) flash_get_memptr(sector);

    if (flashFamily == FLASH_AMD || flashFamily == FLASH_SST) {
#if defined(_BCM96338_) || defined(CONFIG_BCM96338)
        do {
            d1 = flashptr[offset/2];
            if (d1 == data)
                return STATUS_READY;
			//swda add,06/27/2006
			i1++;
			if ( (i1%1024) == 0 ) {
				DiagLedBlinking();
				i1 = 0;
			}
			//swda add end  
        } while (!(d1 & 0x20));

        d1 = flashptr[offset/2];

        if (d1 != data) {
            flash_command(FLASH_RESET, 0, 0, 0);
            return STATUS_TIMEOUT;
        }
#else
        do {
            d1 = *flashptr;    /* read data */
            d1 ^= *flashptr;   /* read it again and see what toggled */
            if (d1 == 0)       /* no toggles, nothing's happening */
                return STATUS_READY;
			//swda add,06/27/2006
			i1++;
			if ( (i1%1024) == 0 ) {
				DiagLedBlinking();
				i1 = 0;
			}
			//swda add end  
        } while (!(d1 & 0x20));

        d1 = *flashptr;        /* read data */
        d1 ^= *flashptr;   /* read it again and see what toggled */

        if (d1 != 0) {
            flash_command(FLASH_RESET, 0, 0, 0);
            return STATUS_TIMEOUT;
        }
#endif
    } else if (flashFamily == FLASH_INTEL) {
        flashptr[0] = 0x70;
#if 0	//swda modify,06/27/2006
        /* Wait for completion */
        while(!(*flashptr & 0x80));
#else
		while(!(*flashptr & 0x80)) {
			i1++;
			if ( (i1%1024) == 0 ) {
				DiagLedBlinking();
				i1 = 0;
			}
		}
#endif	//swda modify end
        if (*flashptr & 0x30) {
            flashptr[0] = 0x50;
            flash_command(FLASH_RESET, 0, 0, 0);
            return STATUS_TIMEOUT;
        }
        flashptr[0] = 0x50;
        flash_command(FLASH_RESET, 0, 0, 0);
    }
    
    return STATUS_READY;
}

/*********************************************************************/
/* flash_get_device_id() will perform an autoselect sequence on the  */
/* flash device, and return the device id of the component.          */
/* This function automatically resets to read mode.                  */
/*********************************************************************/
static UINT16 flash_get_device_id()
{
    volatile UINT16 *fwp; /* flash window */
    UINT16 answer;
    
    fwp = (UINT16 *)flash_get_memptr(0);
    
    flash_command(FLASH_READ_ID, 0, 0, 0);
    man_id = *(fwp);
    answer = *(fwp + 1);
    if (answer == ID_AM29LV320M) {
        answer = *(fwp + 0xe);
        answer = *(fwp + 0xf);
    }
    flash_command(FLASH_RESET, 0, 0, 0);
    return( (UINT16) answer );
}

/*********************************************************************/
/* flash_get_cfi() is the main CFI workhorse function.  Due to it's  */
/* complexity and size it need only be called once upon              */
/* initializing the flash system.  Once it is called, all operations */
/* are performed by looking at the meminfo structure.                */
/* All possible care was made to make this algorithm as efficient as */
/* possible.  90% of all operations are memory reads, and all        */
/* calculations are done using bit-shifts when possible              */
/*********************************************************************/
static int flash_get_cfi(struct cfi_query *query, UINT16 *cfi_struct, int flashFamily)
{
    volatile UINT16 *fwp; /* flash window */
    int i=0;

    flash_command(FLASH_CFIQUERY, 0, 0, 0);
    
    if (cfi_struct == 0)
        fwp = (UINT16 *)flash_get_memptr(0);
    else
        fwp = cfi_struct;
    
    /* Initial house-cleaning */
    for(i=0; i < 8; i++) {
        query->erase_block[i].sector_size = 0;
        query->erase_block[i].num_sectors = 0;
    }
    
    /* If not 'QRY', then we dont have a CFI enabled device in the socket */
    if( fwp[0x10] != 'Q' &&
        fwp[0x11] != 'R' &&
        fwp[0x12] != 'Y') {
        flash_command(FLASH_RESET, 0, 0, 0);
        return(-1);
    }
    
    query->device_size = (int) (((int)1) << fwp[0x27]);		/* denny 09/13/04' add */
    
	query->num_erase_blocks = fwp[0x2C];
	if(flashFamily == FLASH_SST)
		query->num_erase_blocks = 1;
	
    for(i=0; i < query->num_erase_blocks; i++) {
			query->erase_block[i].num_sectors = fwp[(0x2D+(4*i))] + (fwp[0x2E + (4*i)] << 8);
			query->erase_block[i].num_sectors++;
			query->erase_block[i].sector_size = 256 * (256 * fwp[(0x30+(4*i))] + fwp[(0x2F+(4*i))]);
    }
    
    flash_command(FLASH_RESET, 0, 0, 0);
    return(1);
}

//swda add,06/27/2006
static int DiagLedBlinking(void)
{
	static int diag_led = 0;
	static int curr_time = 0;
	static int prev_time = 0;
	static int UpgradeState = 0;
	static int blink_interval = T50ms;
	
	UpgradeState = kerSysGetUpgradeState();
	if ( !UpgradeState ) 
		return 0;//not update image or not update configuration, so do nothing
	
	if ( UpgradeState == 2 ) {	//update configuration
		blink_interval = T25ms;
	} else {	//update image
		blink_interval = T50ms;
	} 
	curr_time = jiffies;	
	if ( (curr_time - prev_time) >= blink_interval ) {	//blinking time: 100ms
		prev_time = curr_time;
		if (diag_led) {
			kerSysLedCtrl(kLedDiag, kLedStateOff);
			diag_led = 0;
		} else {
			kerSysLedCtrl(kLedDiag, kLedStateOn);
			diag_led = 1;
		}
	}
	return 0;
}
//swda add end
