/*
 * SIP extension for UDP/TCP NAT alteration. 
 */
 
#include <linux/module.h>
#include <linux/netfilter_ipv4.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/ctype.h>
#include <net/tcp.h>
#include <net/udp.h>

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
#include <bcmtypes.h>
#include <bcmatmapi.h>
#endif

#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_nat_helper.h>
#include <linux/netfilter_ipv4/ip_nat_rule.h>
#include <linux/netfilter_ipv4/ip_conntrack_sip.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>

#if 0
  #define DEBUG_PRINTK
  #define DEBUGP printk
#else
  #define DEBUGP(format, args...)
#endif

#define MAX_PORTS 8
static int ports[MAX_PORTS];
static int ports_c = 0;

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
static unsigned int stunaddr=0;
static unsigned int stunmask=0;
static int stunaddr_first;
static int stunaddr_end;
static int stunmask_first;
static int stunmask_end;
static int vpi;
static int vci;
static int protocol;
static int encapmode;
module_param(stunaddr_first, int,0);
module_param(stunaddr_end, int,0);
module_param(stunmask_first, int,0);
module_param(stunmask_end, int,0);
module_param(vpi, int, 0);
module_param(vci, int, 0);
module_param(protocol, int, 0);//1:pppoe, 2:pppoa, 3:mer, 4:ipoa, 5:bridge
module_param(encapmode, int, 0);//0:LLC/SNAP, 1:VCMUX (PPPOA 0:VCMUX, 1:LLC/SNAP)
#else
#ifdef MODULE_PARM
MODULE_PARM(ports, "1-" __MODULE_STRING(MAX_PORTS) "i");
#endif
#endif

DECLARE_LOCK_EXTERN(ip_sip_lock);

#define RTCP_SUPPORT


/*dump_packet(struct sk_buff **pskb)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	printk("%s", data);
	printk("\n");
}*/

/* down(stream): caller -> callee
     up(stream): caller <- callee */

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
int find_atmqos_priogroup_number(){
	int i=0;
	int nRet = 0;
	UINT32 ulGroup = 0;
	UINT32 ulSize = NUM_ENTRIES_PER_PRI_PKT_GROUP;
	ATM_PRIORITY_PACKET_ENTRY PriPkts[NUM_ENTRIES_PER_PRI_PKT_GROUP + 1];

#if defined(SUPPORT_RVSIP)
	for (i=7; i <= NUM_PRI_PKT_GROUPS ; i++){
#else
	for (i=2; i <= NUM_PRI_PKT_GROUPS ; i++){
#endif	
		PriPkts[0].ulStructureId = ID_ATM_PRIORITY_PACKET_ENTRY;
		ulGroup = i;
		nRet = (int)BcmAtm_GetPriorityPacketGroup(ulGroup, PriPkts, &ulSize);
		if (nRet==0 && ulSize ==0)
			break;	
	}
	return ulGroup;	
}

void add_atmqos_priogroup_rule(u_int16_t group, u_int16_t rtpport){
	UINT32 ulGroup=0;
	int nRet=0,i=0;
	ATM_PRIORITY_PACKET_ENTRY PriPkts[NUM_ENTRIES_PER_PRI_PKT_GROUP + 1];
	if( (group < 2) || protocol == 5)
		return;
	ulGroup=group;
	PriPkts[0].ulStructureId = ID_ATM_PRIORITY_PACKET_ENTRY;
	PriPkts[0].usPacketValue = rtpport;
	PriPkts[0].usValueMask = 0xFFFF;
	/* Add a new entry to the group. */
	if (protocol == 1){//PPPOE
		PriPkts[0].ulPacketOffset = 54;
	} else if (protocol == 2){//PPPOA
		if (encapmode ==0){//VCMUX
			PriPkts[0].ulPacketOffset = 24;
		} else {//LLC/SNAP
			PriPkts[0].ulPacketOffset = 28;
		}
	} else if (protocol == 3){//MER
		if (encapmode ==0){//LLC/SNAP
			PriPkts[0].ulPacketOffset = 46;
		} else {//VCMUX
			PriPkts[0].ulPacketOffset = 46;
		}
	} else {//IPOA
		if (encapmode ==0){//LLC/SNAP
			PriPkts[0].ulPacketOffset = 30;
		} else {//VCMUX
			PriPkts[0].ulPacketOffset = 22;
		}
	}
    nRet = (int) BcmAtm_SetPriorityPacketGroup(ulGroup, PriPkts, 1);
	//Setup pvc to priority packet group
	ATM_VCC_ADDR Addr;
	ATM_VCC_CFG Cfg;
	memset(&Addr, 0, sizeof(ATM_VCC_ADDR));
    memset( &Cfg, 0x00, sizeof(Cfg) );
 	Addr.usVpi=vpi;
	Addr.usVci=vci;
    Cfg.ulStructureId = ID_ATM_VCC_CFG;
	nRet = (int) BcmAtm_GetVccCfg( &Addr, &Cfg );
	if( ulGroup >= 2 && ulGroup <= NUM_PRI_PKT_GROUPS ){
		for( i = 0; i < NUM_PRI_PKT_GROUPS; i++ ){
           	if( Cfg.ulPriorityPacketGroupNumbers[i] == 0 ){
				Cfg.ulPriorityPacketGroupNumbers[i] = ulGroup;
				/* Set the updated vcc configuration. */
                nRet = (int) BcmAtm_SetVccCfg( &Addr, &Cfg );
				if( nRet != 0 )
					printk("add_atmqos_priogroup_rule(): updated vcc config fail\n");
				break;
			}
    	}
	}
}
#endif
/* Return 1 for match, 0 for accept, -1 for partial. */
static int find_pattern(const char *data, size_t dlen,
			const char *pattern, size_t plen,
			char skip, char term,
			unsigned int *numoff,
			unsigned int *numlen
			)
{
	size_t i, j, k;

	//DEBUGP("find_pattern `%s': dlen = %u\n", pattern, dlen);
	if (dlen == 0)
		return 0;

	*numoff = 0;
	*numlen = 0;

	if (dlen <= plen) {
		/* Short packet: try for partial? */
		if (strnicmp(data, pattern, dlen) == 0)
			return -1;
		else return 0;
	}

	for(i=0; i<= (dlen - plen); i++){
		if( memcmp(data + i, pattern, plen ) != 0 ) continue;	

		/* patten match !! */	
		*numoff=i + plen;
		for (j=*numoff, k=0; data[j] != term; j++, k++)
			if( j > dlen ) return -1 ;	/* no terminal char */

		*numlen = k;
		return 1;
	}

	return 0;
}

static unsigned int
sip_nat_expected(struct sk_buff **pskb,
		 unsigned int hooknum,
		 struct ip_conntrack *ct,
		 struct ip_nat_info *info)
{
	struct ip_nat_multi_range mr;
	u_int32_t newdstip, newsrcip, newip;
	struct ip_ct_sip_expect *exp_sip_info;
	struct ip_conntrack *master = master_ct(ct);

	IP_NF_ASSERT(info);
	IP_NF_ASSERT(master);
	IP_NF_ASSERT(!(info->initialized & (1<<HOOK2MANIP(hooknum))));

	exp_sip_info = &ct->master->help.exp_sip_info;

	LOCK_BH(&ip_sip_lock);

	/* Outer machine IP */
	newsrcip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
	/* Client (virtual) IP under NAT */
	newdstip = master->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;

	UNLOCK_BH(&ip_sip_lock);

	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_SRC)
		newip = newsrcip;
	else
		newip = newdstip;

	DEBUGP("sip_nat_expected: IP to %u.%u.%u.%u:%u\n", NIPQUAD(newip),
			htons(exp_sip_info->port));

	mr.rangesize = 1;
	/* We don't want to manip the per-protocol, just the IPs... */
	mr.range[0].flags = IP_NAT_RANGE_MAP_IPS;
	mr.range[0].min_ip = mr.range[0].max_ip = newip;

	/* ... unless we're doing a MANIP_DST, in which case, make
	   sure we map to the correct port */
	if (HOOK2MANIP(hooknum) == IP_NAT_MANIP_DST) {
		mr.range[0].flags |= IP_NAT_RANGE_PROTO_SPECIFIED;
		mr.range[0].min.udp.port = mr.range[0].max.udp.port	= exp_sip_info->port;
	}

	return ip_nat_setup_info(ct, &mr, hooknum);
}

static int _find_sip_via_addrport(const char *data, size_t dlen,
			unsigned int *numoff, unsigned int *numlen)
{
	const char *addr, *p = data;
	const char *limit = data + dlen;

	*numoff = 0;
	*numlen = 0;

	while (p < limit) {
		/* Find the topmost via tag */
		if (strnicmp(p, "\nvia:",5) && strnicmp(p, "\nv:",3) &&
		    strnicmp(p, "\rvia:",5) && strnicmp(p, "\rv:",3)) {
			p++;
			continue;
		}

		/* Look for UDP */
		while (*p!='U' && *p!='u') {
			if (p == limit)
				return 0;
			p++;
		}
		p+= 3;

		if (p >= limit)
			return 0;

		/* Skip a space */
		while (*p == ' '){
			if (p == limit)
				return 0;
			p++;
		}

		/* IP address */
		addr = p;

		/* FQDNs or dotted quads */
		while (isalpha(*p) || isdigit(*p) || (*p=='.') || (*p=='-')) {
			p++;
			if (p == limit)
				return 0;
		}

		/* If there is a port number, skip it */
		if (*p == ':') {
			p++;
			if (p == limit)
				return 0;

			while (isdigit(*p)) {
				p++;
				if (p == limit)
					return 0;
			}
		}

		*numoff = addr - data;
		*numlen = p - addr;

		return 1;
	}

	return 0;	/* Not found */
}

static int _mangle_sip_via(struct ip_conntrack *ct, enum ip_conntrack_info ctinfo,
		struct sk_buff **pskb, u_int32_t newip, u_int16_t newport,
		unsigned int numoff, unsigned int numlen)
{
	int buflen;
	char buffer[sizeof("nnn.nnn.nnn.nnn:nnnnn")];

	sprintf(buffer, "%u.%u.%u.%u:%u", NIPQUAD(newip), newport);
	buflen = strlen(buffer);

	MUST_BE_LOCKED(&ip_sip_lock);

	if(numlen == 0)
		return 0;

	if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, numoff, 
				numlen, buffer, buflen) )
		return 0;

	DEBUGP("(SIP) Via is changed to %s.\n", buffer);

	return buflen - numlen;
}

static int mangle_sip_via(struct ip_conntrack *ct, enum ip_conntrack_info ctinfo,
		struct sk_buff **pskb, u_int32_t newip, u_int16_t newport)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	unsigned int udplen = ntohs(iph->tot_len) - (iph->ihl * 4);
	unsigned int datalen = udplen - 8;
	unsigned int matchoff, matchlen;

	 /* Find the topmost via tag */
	_find_sip_via_addrport(data , datalen, &matchoff, &matchlen);
	return _mangle_sip_via(ct, ctinfo, pskb, newip, newport,
			matchoff, matchlen);
}

static int mangle_sip_contact_in(struct ip_conntrack *ct, enum ip_conntrack_info ctinfo,
		struct sk_buff **pskb, u_int32_t newip, u_int16_t newport)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	unsigned int udplen = ntohs(iph->tot_len) - (iph->ihl * 4);
	unsigned int datalen = udplen - 8;
	int buflen, diff_len = 0;
	char buffer[sizeof("nnn.nnn.nnn.nnn:nnnnn")];
	const char *uri, *addr, *p = data;
	const char *limit = data + datalen;
	u_int32_t wanip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;     // NAT wan ip
	char buffer1[16];
	char buffer2[16];

	memset(buffer2,0,16);
	sprintf(buffer2,"%u.%u.%u.%u",NIPQUAD(wanip));
	while (p < limit) {
		if (strnicmp(p, "\ncontact:", 9) && strnicmp(p, "\nm:", 3) &&
		    strnicmp(p, "\rcontact:", 9) && strnicmp(p, "\rm:", 3)) {
			p++;
			continue;
		}

		while (strnicmp(p, "sip:", 4)) {
			if (p == limit)
				return 0;
			p++;
		}
		p += 4;

		/* If there is user info in the contact */
		uri = p;

		while (*p!='@' && *p!='>' && *p!=';' && *p!='\n' && *p!='\r' && *p!='?' && *p!=',') {
			if (p == limit)
				return 0;
			p++;
		}

		if (*p=='@')
			p++;
		else
			p = uri;	/* back to previous URI pointer */

		/* IP address */
		addr = p;
		while (*p!=':' && *p!='>' && *p!=';') {
			if (p == limit)
				return 0;
			p++;
		}
		if ((p-addr) > 16)
			return 0;
		memset(buffer1,0,16);		
		memcpy(buffer1,addr,p-addr);
		if (strcmp(buffer1, buffer2)!=0){
			continue;			
		}
		/* FQDNs or dotted quads */
		while (isalpha(*p) || isdigit(*p) || (*p=='.') || (*p=='-')) {
			p++;
			if (p == limit)
				return 0;
		}

		/* If there is a port number, skip it */
		if (*p==':') {
			p++;
			while (isdigit(*p))
				p++;
		}

		sprintf(buffer, "%u.%u.%u.%u:%u", NIPQUAD(newip), newport);
		buflen = strlen(buffer);

		MUST_BE_LOCKED(&ip_sip_lock);

		if( p == addr)
			return 0;

		if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, addr - data, 
					p - addr, buffer, buflen) )
			return 0;
	
		diff_len = buflen - (p - addr);
		DEBUGP("(SIP) Contact is changed to %s.\n", buffer);
		break;
	}

	return diff_len;
}

static int mangle_sip_contact(struct ip_conntrack *ct, enum ip_conntrack_info ctinfo,
		struct sk_buff **pskb, u_int32_t newip, u_int16_t newport)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	unsigned int udplen = ntohs(iph->tot_len) - (iph->ihl * 4);
	unsigned int datalen = udplen - 8;

	int buflen, diff_len = 0;
	char buffer[sizeof("nnn.nnn.nnn.nnn:nnnnn")];
	const char *uri, *addr, *p = data;
	const char *limit = data + datalen;


	while (p < limit) {
		if (strnicmp(p, "\ncontact:", 9) && strnicmp(p, "\nm:", 3) &&
		    strnicmp(p, "\rcontact:", 9) && strnicmp(p, "\rm:", 3)) {
			p++;
			continue;
		}

		while (strnicmp(p, "sip:", 4)) {
			if (p == limit)
				return 0;
			p++;
		}
		p += 4;

		/* If there is user info in the contact */
		uri = p;

		while (*p!='@' && *p!='>' && *p!=';' && *p!='\n' && *p!='\r' && *p!='?' && *p!=',') {
			if (p == limit)
				return 0;
			p++;
		}

		if (*p=='@')
			p++;
		else
			p = uri;	/* back to previous URI pointer */

		/* IP address */
		addr = p;

		/* FQDNs or dotted quads */
		while (isalpha(*p) || isdigit(*p) || (*p=='.') || (*p=='-')) {
			p++;
			if (p == limit)
				return 0;
		}

		/* If there is a port number, skip it */
		if (*p==':') {
			p++;
			while (isdigit(*p))
				p++;
		}

		sprintf(buffer, "%u.%u.%u.%u:%u", NIPQUAD(newip), newport);
		buflen = strlen(buffer);

		MUST_BE_LOCKED(&ip_sip_lock);

		if( p == addr)
			return 0;

		if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, addr - data, 
					p - addr, buffer, buflen) )
			return 0;
	
		diff_len = buflen - (p - addr);
		DEBUGP("(SIP) Contact is changed to %s.\n", buffer);
		break;
	}

	return diff_len;
}

static int mangle_sip_requestline(struct ip_conntrack *ct, enum ip_conntrack_info ctinfo,
		struct sk_buff **pskb, u_int32_t newip, u_int16_t newport)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	unsigned int udplen = ntohs(iph->tot_len) - (iph->ihl * 4);
	unsigned int datalen = udplen - 8;

	int buflen, diff_len = 0;
	char buffer[sizeof("nnn.nnn.nnn.nnn:nnnnn")];
	const char *uri, *addr, *p = data;
	const char *limit = data + datalen;

	while (p < limit) {
		while (strnicmp(p, "sip:", 4)) {
			if (p == limit)
				return 0;
			p++;
		}
		p += 4;

		/* If there is user info in the contact */
		uri = p;

		while (*p!='@' && *p!='>' && *p!=';' && *p!='\n' && *p!='\r' && *p!='?' && *p!=',') {
			if (p == limit)
				return 0;
			p++;
		}

		if (*p=='@')
			p++;
		else
			p = uri;	/* back to previous URI pointer */

		/* IP address */
		addr = p;

		/* FQDNs or dotted quads */
		while (isalpha(*p) || isdigit(*p) || (*p=='.') || (*p=='-')) {
			p++;
			if (p == limit)
				return 0;
		}

		/* If there is a port number, skip it */
		if (*p==':') {
			p++;
			while (isdigit(*p))
				p++;
		}

		sprintf(buffer, "%u.%u.%u.%u:%u", NIPQUAD(newip), newport);
		buflen = strlen(buffer);

		MUST_BE_LOCKED(&ip_sip_lock);

		if( p == addr)
			return 0;

		if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, addr - data, p - addr, buffer, buflen) )
			return 0;
	
		diff_len = buflen - (p - addr);
		break;
	}

	return diff_len;
}

static int _find_sip_content_length_size(const char *data, size_t dlen,
			unsigned int *numoff, unsigned int *numlen)
{
	char *st, *p = (char *)data;
	const char *limit = data + dlen;
	int size = 0;

	*numoff = 0;
	*numlen = 0;

	while (p < limit) {
		if (strnicmp(p, "\nContent-Length:", 16) && strnicmp(p, "\nl:", 3) &&
		    strnicmp(p, "\rContent-Length:", 16) && strnicmp(p, "\rm:", 3)) {
			p++;
			continue;
		}

		/* Go through the string above */
		while (*p != ':') {
			if (p == limit)
				return 0;
			p++;
		}
		p++;

		while (*p == ' ') {
			if (p == limit)
				return 0;
			p++;
		}

		st = p;
		size = simple_strtoul(p, &p, 10);

		*numoff = st - data;
		*numlen = p - st;

		return size;
	}

	return 0;
}

static int mangle_sip_content_length(struct ip_conntrack *ct, enum ip_conntrack_info ctinfo,
		struct sk_buff **pskb, int diff_len)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	unsigned int udplen = ntohs(iph->tot_len) - (iph->ihl * 4);
	unsigned int datalen = udplen - 8;

	unsigned int matchlen, matchoff;
	int size, buflen;
	char buffer[sizeof("nnnnn")];

	/* original legth */
	size = _find_sip_content_length_size(data, datalen, &matchoff, &matchlen);

	/* new legth */
	sprintf(buffer, "%u", size + diff_len);
	buflen = strlen(buffer);

	if (matchlen == 0)
		return 0;

	if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, matchoff, 
				matchlen, buffer, buflen) )
		return 0;

	DEBUGP("(SDP) Content-Length is changed %d->%s.\n", size, buffer);
	return buflen - matchlen;

}

static int mangle_sip_sdp_content(struct ip_conntrack *ct, enum ip_conntrack_info ctinfo,
		struct sk_buff **pskb, u_int32_t newip, u_int16_t newport, u_int16_t isvideoport)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	unsigned int udplen = ntohs(iph->tot_len) - (iph->ihl * 4);
	unsigned int datalen = udplen - 8;

	unsigned int matchlen, matchoff;
	int found, buflen, diff_len = 0;
	char buffer[sizeof("nnn.nnn.nnn.nnn")];
	char *addr, *p;
	char *limit = (char *)data + datalen;
	u_int32_t ipaddr = 0;
	u_int16_t getport;
	int dir;

	/* Find RTP address */
	found = find_sdp_rtp_addr(data, datalen, &matchoff, &matchlen, &ipaddr);
	if (found && matchlen != 0) {
		/* If it's a null address, then the call is on hold */
		if (found == 2 && ipaddr == 0)
			return 0;
			
		sprintf(buffer, "%u.%u.%u.%u", NIPQUAD(newip));
		buflen = strlen(buffer);

		MUST_BE_LOCKED(&ip_sip_lock);

		if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, matchoff, 
					matchlen, buffer, buflen) )
			return 0;
	
		diff_len += (buflen - matchlen);
		DEBUGP("(SDP) RTP address is changed to %s.\n", buffer);
	}

	/* Find audio port */
	if (isvideoport == 0){
		getport = find_sdp_audio_port(data, datalen, &matchoff, &matchlen);
		if (getport != newport && matchlen != 0) {
			sprintf(buffer, "%d", newport);
			buflen = strlen(buffer);

			MUST_BE_LOCKED(&ip_sip_lock);
			matchoff = matchoff+diff_len;  
			if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, matchoff, 
						matchlen, buffer, buflen) )
				return 0;

			diff_len += (buflen - matchlen);
			DEBUGP("(SDP) audio port is changed to %d.\n", newport);
		}
	}	

	/* Find video port */
	if (isvideoport == 1){
		getport = find_sdp_video_port(data, datalen, &matchoff, &matchlen);
		if (getport != newport && matchlen != 0) {
			sprintf(buffer, "%d", newport);
			buflen = strlen(buffer);

			MUST_BE_LOCKED(&ip_sip_lock);
			matchoff = matchoff+diff_len;  
			if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, matchoff, 
						matchlen, buffer, buflen) )
				return 0;

			diff_len += (buflen - matchlen);
			DEBUGP("(SDP) video port is changed to %d.\n", newport);
		}
	}

	dir = CTINFO2DIR(ctinfo);
	if (dir == IP_CT_DIR_ORIGINAL)
		return diff_len;

	/* Find Session ID address */
	found = find_pattern(data, datalen, " IN IP4 ", 8, ' ', '\r',
			&matchoff, &matchlen);
	if (found && matchlen != 0) {
		p = addr = (char *)data + matchoff;

		/* FQDNs or dotted quads */
		while (isalpha(*p) || isdigit(*p) || (*p=='.') || (*p=='-')) {
			p++;
			if (p == limit)
				return 0;
		}

		sprintf(buffer, "%u.%u.%u.%u", NIPQUAD(newip));
		buflen = strlen(buffer);

		MUST_BE_LOCKED(&ip_sip_lock);

		if (!ip_nat_mangle_udp_packet(pskb, ct, ctinfo, addr - data, 
					p - addr, buffer, buflen) )
			return 0;
	
		diff_len += (buflen - (p - addr));
		DEBUGP("(SDP) Session ID is changed to %s.\n", buffer);
	}

	return diff_len;
}

static int mangle_sip_packet(struct ip_conntrack *ct, enum ip_conntrack_info ctinfo,
		struct sk_buff **pskb, u_int32_t newip,  u_int16_t isvideoport)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	int diff_len = 0;
	struct ip_ct_sip_master *ct_sip_info = &ct->help.ct_sip_info;
	u_int16_t natport = ntohs(ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.u.udp.port); 

	DEBUGP("nat_sip: %s:(%d)\n", __FUNCTION__, __LINE__);
	if (isvideoport == 0){
	ct_sip_info->mangled = 1;
	}

	/* Changes the via, if this is a request */
	if (strnicmp(data,"SIP/2.0",7) != 0) {
		mangle_sip_via(ct, ctinfo, pskb, newip, natport);
	}

	/*SIP ALG can't change contact field in 3xx response packets*/
	if (strnicmp(data, "SIP/2.0 3", 9) != 0) {//This packet is not 3xx response
		mangle_sip_contact(ct, ctinfo, pskb, newip, natport);
	}

	if ((diff_len = mangle_sip_sdp_content(ct, ctinfo, pskb, newip, ct_sip_info->rtpport, isvideoport)) != 0)
		mangle_sip_content_length(ct, ctinfo, pskb, diff_len);

	return 1;
}

static int sip_out_data_fixup(struct ip_conntrack *ct,
		struct sk_buff **pskb,
		enum ip_conntrack_info ctinfo,
		struct ip_conntrack_expect *expect)
{
	struct ip_conntrack_tuple newtuple;
	struct ip_ct_sip_master *ct_sip_info = &ct->help.ct_sip_info;
	struct ip_ct_sip_expect *exp_sip_info;
	u_int32_t wanip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;     // NAT wan ip
	u_int16_t port = 0;
	u_int16_t videoport = 0;
	u_int16_t isvideoport = 2;
#ifdef RTCP_SUPPORT
	struct ip_conntrack_expect *rtcp_exp;
#endif
#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	u_int16_t domangle = 1;
#endif

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (stunaddr!=0 && stunmask !=0 ){			
		if (((stunaddr & stunmask) != (ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip & stunmask)) || (stunaddr == ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip)){//public IP
			domangle = 0;
		}
	}
#endif

	MUST_BE_LOCKED(&ip_sip_lock);

	if (expect) {
		DEBUGP("nat_sip: %s: There is a exp %p.\n", __FUNCTION__, expect);

		exp_sip_info = &expect->help.exp_sip_info;
		if (exp_sip_info->nated) {
			DEBUGP("nat_sip: %s: The exp %p had been changed.\n", __FUNCTION__, expect);
#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
			if (exp_sip_info->type == CONN_RTP && ct_sip_info->rtpport > 1024 && ct_sip_info->groupadded ==0){
				if (vci != 0 && protocol != 0){
					if (stunaddr!=0 && stunmask !=0 ){			
						if (domangle){//private IP
							ct_sip_info->rtpport_priogroup = find_atmqos_priogroup_number();
							add_atmqos_priogroup_rule(ct_sip_info->rtpport_priogroup, ct_sip_info->rtpport);
							ct_sip_info->groupadded = 1;
						}
					}
				}
			}
#endif		
			goto mangle;
		}

		DEBUGP("nat_sip: %s: The exp type is %d.\n", __FUNCTION__, exp_sip_info->type);

		if (exp_sip_info->type == CONN_SIP) {
			exp_sip_info->nated = 1;
			DEBUGP("nat_sip: %s: There is a SIP exp %p.\n", __FUNCTION__, expect);
		} else if (exp_sip_info->type == CONN_RTP) {
			DEBUGP("nat_sip: %s: There is a RTP exp %p.\n", __FUNCTION__, expect);
			/* RTP expect */
			isvideoport = 0;
			port = ntohs(expect->tuple.dst.u.udp.port); 
#ifdef RTCP_SUPPORT
			rtcp_exp = expect_find(ct, expect->tuple.dst.ip, port + 1); 
#endif
			/* RFC1889 - If an application is supplied with an odd number
			 * for use as the RTP port, it should replace this number with
			 * the next lower (even) number. */
			if (port & 0x01) 
				port++;

			/* fullfill newtuple */
			memset(&newtuple, 0, sizeof(struct ip_conntrack_tuple));
			newtuple.dst.ip = wanip;
			newtuple.src.ip = expect->tuple.src.ip;
			newtuple.src.u.all = 0;
			newtuple.dst.protonum = expect->tuple.dst.protonum;
			/* Try to get same port: if not, try to change it. */
			for (; port != 0; port += 2) {
				newtuple.dst.u.udp.port = htons(port);
				if (ip_conntrack_change_expect(expect, &newtuple) == 0) {
#ifdef RTCP_SUPPORT
					/* Change RTCP */
					if (rtcp_exp) {
						DEBUGP("nat_sip: %s: RTCP exp %p found.\n",
								__FUNCTION__, rtcp_exp);
						newtuple.dst.u.udp.port = htons(port + 1);
						if (ip_conntrack_change_expect(rtcp_exp, &newtuple) != 0) {
							DEBUGP("nat_sip: %s: Can't change RTCP exp %p.\n",
									__FUNCTION__, rtcp_exp);
							continue;
						}
						rtcp_exp->help.exp_sip_info.nated = 1;
					}
#endif
					break;
				}
			}
			if (port == 0)
				return 0;

			exp_sip_info->nated = 1;
			ct_sip_info->rtpport = port;
#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
			if (vci != 0 && protocol != 0){
				if (stunaddr!=0 && stunmask !=0 ){			
					if (domangle && ct_sip_info->groupadded == 0 && ct_sip_info->rtpport > 1024){//private IP
						ct_sip_info->rtpport_priogroup = find_atmqos_priogroup_number();
						add_atmqos_priogroup_rule(ct_sip_info->rtpport_priogroup, ct_sip_info->rtpport);
						ct_sip_info->groupadded= 1;
					}
				}
			}
#endif			
			DEBUGP("nat_sip: %s: RTP exp %p, masq port=%d\n", __FUNCTION__, expect, port);
		} else if (exp_sip_info->type == CONN_RTPVIDEO){
			DEBUGP("nat_sip: %s: There is a RTP exp %p.\n", __FUNCTION__, expect);
			/* RTP Video expect */
			isvideoport = 1;
			videoport = ntohs(expect->tuple.dst.u.udp.port); 
#ifdef RTCP_SUPPORT
			rtcp_exp = expect_find(ct, expect->tuple.dst.ip, port + 1);
#endif
			/* RFC1889 - If an application is supplied with an odd number
			 * for use as the RTP port, it should replace this number with
			 * the next lower (even) number. */
			if (videoport & 0x01) 
				videoport++;
			/* fullfill newtuple */
			memset(&newtuple, 0, sizeof(struct ip_conntrack_tuple));
			newtuple.dst.ip = wanip;
			newtuple.src.ip = expect->tuple.src.ip;
			newtuple.src.u.all = 0;
			newtuple.dst.protonum = expect->tuple.dst.protonum;
			/* Try to get same port: if not, try to change it. */
			for (; videoport != 0; videoport += 2) {
				newtuple.dst.u.udp.port = htons(videoport);
				if (ip_conntrack_change_expect(expect, &newtuple) == 0) {
#ifdef RTCP_SUPPORT
					/* Change RTCP */
					if (rtcp_exp) {
						DEBUGP("nat_sip: %s: RTCP exp %p found.\n",
								__FUNCTION__, rtcp_exp);
						newtuple.dst.u.udp.port = htons(videoport + 1);
						if (ip_conntrack_change_expect(rtcp_exp, &newtuple) != 0) {
							DEBUGP("nat_sip: %s: Can't change RTCP exp %p.\n",
									__FUNCTION__, rtcp_exp);
							continue;
						}
						rtcp_exp->help.exp_sip_info.nated = 1;
					}
#endif
					break;
				}
			}
			if (videoport == 0)
				return 0;

			exp_sip_info->nated = 1;
			ct_sip_info->rtpport = videoport;
			DEBUGP("nat_sip: %s: RTP video exp %p, masq port=%d\n", __FUNCTION__, expect, videoport);				
		}
#ifdef RTCP_SUPPORT
		else {
			/* We ignore the RTCP expect, and will adjust it later
			 * during RTP expect */
			DEBUGP("nat_sip: %s: There is a RTCP exp %p, by-pass.\n", __FUNCTION__, expect);
			return 1;
		}
#endif
	}

mangle:
	/* Change address inside packet to match way we're mapping
	   this connection. */
#if defined(VOICE_QUALITY_CONTROL_BY_ATM)	   
	if (!ct_sip_info->mangled && domangle){
#else
	if (!ct_sip_info->mangled){
#endif		
		if (!mangle_sip_packet(ct, ctinfo, pskb, wanip, isvideoport))
			return 0;
	}		

	return 1;
}

static int sip_in_data_fixup(struct ip_conntrack *ct,
		struct sk_buff **pskb,
		enum ip_conntrack_info ctinfo,
		struct ip_conntrack_expect *expect)
{
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;
	unsigned int udplen = ntohs(iph->tot_len) - (iph->ihl * 4);
	unsigned int datalen = udplen - 8;
	struct ip_ct_sip_master *ct_sip_info = &ct->help.ct_sip_info;
	u_int32_t wanip = ct->tuplehash[IP_CT_DIR_REPLY].tuple.dst.ip;     // NAT wan ip
	unsigned int matchlen, matchoff;
	int found, diff_len = 0;
	u_int32_t ipaddr = 0, vip = 0;
	u_int16_t port = 0, vport = 0, aport = 0;
	u_int16_t videoport = 0, rport = 0;
	struct ip_conntrack_expect *exp;
#ifdef RTCP_SUPPORT
	struct ip_conntrack_expect *rtcpexp;
#endif
#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	u_int16_t domangle = 1;
#endif

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (stunaddr!=0 && stunmask !=0 ){			
		if (((stunaddr & stunmask) != (ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip & stunmask)) || (stunaddr == ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip)){//public IP
			domangle = 0;
		}
	}
#endif

	if (expect){
		DEBUGP("nat_sip: %s: There is a exp %p.\n", __FUNCTION__, expect);
	} else {
		return 1;
	}
	DEBUGP("nat_sip: ct_sip_info->mangled=[%d]\n", ct_sip_info->mangled);

	/* Prevent from mangling the packet or expect twice */
	if (ct_sip_info->mangled) {
		DEBUGP("nat_sip: %s: the conntrack %p had been mangled.\n", __FUNCTION__, ct);
		return 1;
	}
        //u_int32_t srcaddr = iph->daddr;//source address of ip field in packet

	ct_sip_info->mangled = 1;
#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (strnicmp(data, "SIP/2.0 ", 8) != 0 && domangle) {//This packet is request
#else
	if (strnicmp(data, "SIP/2.0 ", 8) != 0) {//This packet is request
#endif
		found = 0;
		/* Find CSeq field */
		found = find_pattern(data, datalen, "CSeq: ", 6, ' ', '\r',
			&matchoff, &matchlen);
		if (found && matchlen != 0) {
			vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
			vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
			mangle_sip_requestline(ct, ctinfo, pskb, vip, vport);
			//return 1;
		}
	}

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (strnicmp(data, "SIP/2.0 200", 11) == 0 && domangle) {
#else	
	if (strnicmp(data, "SIP/2.0 200", 11) == 0) {
#endif	
		/* Find CSeq field */
		found = find_pattern(data, datalen, "CSeq: ", 6, ' ', '\r',
				&matchoff, &matchlen);
		if (found && matchlen != 0) {
			char *p = (char *)data + matchoff;

			simple_strtoul(p, &p, 10);
			if (strnicmp(p, " REGISTER", 9) == 0) {
				DEBUGP("nat_sip: 200 OK - REGISTER\n");
				vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
				vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
				mangle_sip_via(ct, ctinfo, pskb, vip, vport);
				mangle_sip_contact_in(ct, ctinfo, pskb, vip, vport);
				return 1;
			} else if (strnicmp(p, " SUBSCRIBE", 10) == 0){//SUBSCRIBE for Voice mail
				vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
				vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
				mangle_sip_via(ct, ctinfo, pskb, vip, vport);
				mangle_sip_contact_in(ct, ctinfo, pskb, vip, vport);
				return 1;					
			} else if (strnicmp(p, " INVITE", 7) == 0){
				DEBUGP("nat_sip: 200 OK - INVITE\n");
				vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
				vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
				mangle_sip_via(ct, ctinfo, pskb, vip, vport);
				//return 1;
			} else {//Don't care SDP session
				vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
				vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
				mangle_sip_via(ct, ctinfo, pskb, vip, vport);
				return 1;
			}
		}
	}

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (strnicmp(data, "SIP/2.0 1", 9) == 0 && domangle) {//This packet is 1xx response
#else	
	if (strnicmp(data, "SIP/2.0 1", 9) == 0) {//This packet is 1xx response	
#endif	
		found = 0;
		/* Find CSeq field */
		found = find_pattern(data, datalen, "CSeq: ", 6, ' ', '\r',
			&matchoff, &matchlen);
		if (found && matchlen != 0) {
			//char *p = (char *)data + matchoff;
			//simple_strtoul(p, &p, 8);
			DEBUGP("nat_sip: pre change via and contact field\n");
			vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
			vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
			mangle_sip_via(ct, ctinfo, pskb, vip, vport);
			mangle_sip_contact(ct, ctinfo, pskb, vip, vport);
			return 1;
		}
	}

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (strnicmp(data, "SIP/2.0 3", 9) == 0 && domangle) {//This packet is 3xx response
#else	
	if (strnicmp(data, "SIP/2.0 3", 9) == 0) {//This packet is 3xx response	
#endif	
		found = 0;
		/* Find CSeq field */
		found = find_pattern(data, datalen, "CSeq: ", 6, ' ', '\r',
				&matchoff, &matchlen);
		if (found && matchlen != 0) {
			//char *p = (char *)data + matchoff;
			//simple_strtoul(p, &p, 8);
			DEBUGP("nat_sip: pre change via and contact field\n");
			vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
			vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
			mangle_sip_via(ct, ctinfo, pskb, vip, vport);
			return 1;
		}
	}

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (strnicmp(data, "SIP/2.0 4", 9) == 0 && domangle) {//This packet is 4xx response
#else
	if (strnicmp(data, "SIP/2.0 4", 9) == 0) {//This packet is 4xx response	
#endif	
		found = 0;
		/* Find CSeq field */
		found = find_pattern(data, datalen, "CSeq: ", 6, ' ', '\r',
				&matchoff, &matchlen);
		if (found && matchlen != 0) {
			//char *p = (char *)data + matchoff;
			//simple_strtoul(p, &p, 8);
			DEBUGP("nat_sip: pre change via and contact field\n");
			vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
			vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
			mangle_sip_via(ct, ctinfo, pskb, vip, vport);
			mangle_sip_contact(ct, ctinfo, pskb, vip, vport);
			return 1;
		}
	}	

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (strnicmp(data, "SIP/2.0 5", 9) == 0 && domangle) {//This packet is 5xx response
#else		
	if (strnicmp(data, "SIP/2.0 5", 9) == 0) {//This packet is 5xx response	
#endif	
		found = 0;
		/* Find CSeq field */
		found = find_pattern(data, datalen, "CSeq: ", 6, ' ', '\r',
			&matchoff, &matchlen);
		if (found && matchlen != 0) {
			//char *p = (char *)data + matchoff;
			//simple_strtoul(p, &p, 8);
			DEBUGP("nat_sip: pre change via and contact field\n");
			vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
			vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
			mangle_sip_via(ct, ctinfo, pskb, vip, vport);
			mangle_sip_contact(ct, ctinfo, pskb, vip, vport);
			return 1;
		}
	}

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if (strnicmp(data, "SIP/2.0 6", 9) == 0 && domangle) {//This packet is 6xx response
#else		
	if (strnicmp(data, "SIP/2.0 6", 9) == 0) {//This packet is 6xx response	
#endif	
		found = 0;
		/* Find CSeq field */
		found = find_pattern(data, datalen, "CSeq: ", 6, ' ', '\r',
			&matchoff, &matchlen);
		if (found && matchlen != 0) {
			//char *p = (char *)data + matchoff;
			//simple_strtoul(p, &p, 8);
			DEBUGP("nat_sip: pre change via and contact field\n");
			vip = ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
			vport = ntohs(ct->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.u.udp.port);
			mangle_sip_via(ct, ctinfo, pskb, vip, vport);
			mangle_sip_contact(ct, ctinfo, pskb, vip, vport);
			return 1;
		}
	}

#if defined(VOICE_QUALITY_CONTROL_BY_ATM)
	if ( strnicmp(data, "BYE", 3) == 0){
		if (vci != 0 && protocol != 0 && ct_sip_info->rtpport_priogroup > 0){
			BcmAtm_SetPriorityPacketGroup(ct_sip_info->rtpport_priogroup,NULL,0);
			ct_sip_info->groupadded = 0;
			ct_sip_info->rtpport = 0;
		}
		return 1;
	}	
#endif
	/* Only interesting the SDP content. */
	if (strnicmp(data, "INVITE", 6) != 0 && strnicmp(data, "SIP/2.0 200", 11) != 0)
		return 1;
	
	/* Find RTP address */
	found = find_sdp_rtp_addr(data, datalen, &matchoff, &matchlen, &ipaddr);

	DEBUGP("nat_sip: sdp address found = %d, ipaddr = %u.%u.%u.%u\n", found, NIPQUAD(ipaddr));
	if (found < 2)
		return 1;

	DEBUGP("nat_sip: wanip = %u.%u.%u.%u\n", NIPQUAD(wanip));
	
	/* Is it a null address or our WAN address? */
	if (ipaddr == 0 || (htonl(ipaddr) != wanip))
		return 1;

	DEBUGP("nat_sip: %s: This is a loopback RTP connection.\n", __FUNCTION__);

	/* Find audio port, and we don't like the well-known ports,
	 * which is less than 1024 */
	port = find_sdp_audio_port(data, datalen, &matchoff, &matchlen);
	if (port < 1024)
		return 0;
	
	DEBUGP("nat_sip: RTP audio port = %u\n", port);
	
	exp = expect_find(ct, wanip, port);
	if (exp) {
		DEBUGP("nat_sip: %s: Found exp %p, tuple.dst=%u.%u.%u.%u:%u.\n",
				__FUNCTION__, exp, NIPQUAD(ipaddr), port);

		/* Restore masq-ed SDP */
		vip = exp->expectant->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
		aport = exp->help.exp_sip_info.port;
		if ((diff_len = mangle_sip_sdp_content(ct, ctinfo, pskb, vip, aport, 0)) != 0)
			mangle_sip_content_length(ct, ctinfo, pskb, diff_len);
		/*************************************************************
		 * linux kernel 2.4 need to unexpect the related expectation *
		 * but in kernel 2.6, the function seem don't need anymore   *
		 * I still need to find why? but anyway, comment it now      *
		 *									Andrew (2005/08/01)      *
		 *************************************************************/
#if 1
		/* Unset RTP, RTCP expect, respectively */
		//ip_conntrack_unexpect_related(exp);
#ifdef RTCP_SUPPORT
		rtcpexp = expect_find(ct, wanip, port + 1);
		if (rtcpexp && ct != rtcpexp->expectant)
			ip_conntrack_unexpect_related(rtcpexp);
#endif //#ifdef RTCP_SUPPORT
#endif //#if 0
	} else {
		DEBUGP("nat_sip: %s: Can't find exp for tuple.dst=%u.%u.%u.%u:%u.\n",
				__FUNCTION__, NIPQUAD(ipaddr), port);
	}
	
	
	/* Find video port, and we don't like the well-known ports,
	 * which is less than 1024 */
	videoport = find_sdp_video_port(data, datalen, &matchoff, &matchlen);
	if (videoport < 1024)
		return 0;
	DEBUGP("nat_sip: RTP video port = %u\n", videoport);
	if (videoport != port){
		exp = expect_find(ct, wanip, videoport);
		if (exp) {
			DEBUGP("nat_sip: %s: Found video exp %p, tuple.dst=%u.%u.%u.%u:%u.\n",
					__FUNCTION__, exp, NIPQUAD(ipaddr), videoport);

			/* Restore masq-ed SDP */
			vip = exp->expectant->tuplehash[IP_CT_DIR_ORIGINAL].tuple.src.ip;
			rport = exp->help.exp_sip_info.port;
			if ((diff_len = mangle_sip_sdp_content(ct, ctinfo, pskb, vip, rport, 1)) != 0)
				mangle_sip_content_length(ct, ctinfo, pskb, diff_len);
			/* Unset RTP, RTCP expect, respectively */
			//ip_conntrack_unexpect_related(exp);
#ifdef RTCP_SUPPORT
			rtcpexp = expect_find(ct, wanip, videoport + 1);
			if (rtcpexp && ct != rtcpexp->expectant)
				ip_conntrack_unexpect_related(rtcpexp);
#endif //#ifdef RTCP_SUPPORT
		} else {
			DEBUGP("nat_sip: %s: Can't find exp for tuple.dst=%u.%u.%u.%u:%u.\n",
					__FUNCTION__, NIPQUAD(ipaddr), videoport);
		}		
				
	}	
	

	return 1;
}

static unsigned int nat_help(struct ip_conntrack *ct,
			 struct ip_conntrack_expect *exp,
			 struct ip_nat_info *info,
			 enum ip_conntrack_info ctinfo,
			 unsigned int hooknum,
			 struct sk_buff **pskb)
{
	int dir;

	/* Only mangle things once: original direction in POST_ROUTING
	   and reply direction on PRE_ROUTING. */
	dir = CTINFO2DIR(ctinfo);
	if (!((hooknum == NF_IP_POST_ROUTING && dir == IP_CT_DIR_ORIGINAL)
	      || (hooknum == NF_IP_PRE_ROUTING && dir == IP_CT_DIR_REPLY))) {
		DEBUGP("nat_sip: Not touching dir %s at hook %s\n",
		       dir == IP_CT_DIR_ORIGINAL ? "ORIG" : "REPLY",
		       hooknum == NF_IP_POST_ROUTING ? "POSTROUTING"
		       : hooknum == NF_IP_PRE_ROUTING ? "PREROUTING"
		       : hooknum == NF_IP_LOCAL_OUT ? "OUTPUT" : "???");
		return NF_ACCEPT;
	}

#ifdef DEBUG_PRINTK
	struct iphdr *iph = (*pskb)->nh.iph;
	struct udphdr *udph = (void *)iph + iph->ihl * 4;
	const char *data = (const char *)udph + 8;

	if (strnicmp(data, "REGISTER" , 8) == 0)
		DEBUGP("nat_sip: REGISTER\n");
	else if (strnicmp(data, "INVITE" , 6) == 0)
		DEBUGP("nat_sip: INVITE\n");
	else if (strnicmp(data, "ACK" , 3) == 0)
		DEBUGP("nat_sip: ACK\n");
	else if (strnicmp(data, "BYE", 3) == 0)
		DEBUGP("nat_sip: BYE\n");
	else if (strnicmp(data, "CANCEL", 6) == 0)
		DEBUGP("nat_sip: CANCEL\n");
	else if (strnicmp(data, "SIP/2.0 200", 11) == 0)
		DEBUGP("nat_sip: 200 OK\n");
	else if (strnicmp(data, "SIP/2.0 100", 11) == 0)
		DEBUGP("nat_sip: 100 Trying\n");
	else if (strnicmp(data, "SIP/2.0 180", 11) == 0)
		DEBUGP("nat_sip: 180 Ringing\n");
	else if (strnicmp(data, "SIP/2.0 401", 11) == 0)
		DEBUGP("nat_sip: 401 Unauthorized\n");	
#endif

	LOCK_BH(&ip_sip_lock);

	if (dir == IP_CT_DIR_ORIGINAL) {
		DEBUGP("call sip_out_data_fixup\n");
		sip_out_data_fixup(ct, pskb, ctinfo, exp);
		DEBUGP("exit sip_out_data_fixup\n");
	} else {
		DEBUGP("call sip_in_data_fixup\n");
		sip_in_data_fixup(ct, pskb, ctinfo, exp);
		DEBUGP("exit sip_in_data_fixup\n");
	}

	UNLOCK_BH(&ip_sip_lock);

	return NF_ACCEPT;
}

static struct ip_nat_helper sip[MAX_PORTS];
static char sip_names[MAX_PORTS][6];

/* Not __exit: called from init() */
static void fini(void)
{
	int i;
	for (i = 0; (i < MAX_PORTS) && ports[i]; i++) {
		DEBUGP("ip_nat_sip: unregistering port %d\n", ports[i]);
		ip_nat_helper_unregister(&sip[i]);
	}
}

static int __init init(void)
{
	int i, ret=0;
	char *tmpname;

	//printk("ip_nat_sip v0.01 loading\n");
#if defined(VOICE_QUALITY_CONTROL_BY_ATM)	
	stunaddr=(stunaddr_first<<16) | (stunaddr_end);
	stunmask=(stunmask_first<<16) | (stunmask_end);
#endif
	if (ports[0] == 0)
		ports[0] = SIP_PORT;

	for (i = 0; (i < MAX_PORTS) && ports[i]; i++) {

		memset(&sip[i], 0, sizeof(struct ip_nat_helper));

		sip[i].tuple.src.u.udp.port = htons(ports[i]);
		sip[i].tuple.dst.protonum = IPPROTO_UDP;
		sip[i].mask.src.u.udp.port = 0xFFFF;
		sip[i].mask.dst.protonum = 0xFFFF;
		sip[i].help = nat_help;
		sip[i].expect = sip_nat_expected;
		sip[i].flags = IP_NAT_HELPER_F_ALWAYS;

		tmpname = &sip_names[i][0];
		sprintf(tmpname, "sip%2.2d", i);
		sip[i].name = tmpname;

		DEBUGP("ip_nat_sip: Trying to register for port %d\n",
				ports[i]);
		ret = ip_nat_helper_register(&sip[i]);

		if (ret) {
			printk("ip_nat_sip: error registering "
			       "helper for port %d\n", ports[i]);
			fini();
			return ret;
		}
		ports_c++;
	}
	return ret;
}

module_init(init);
module_exit(fini);
