/***************************************************************************
# <:copyright-gpl 
# Copyright 2004 Broadcom Corp. All Rights Reserved. 
# This program is free software; you can distribute it and/or modify it 
# under the terms of the GNU General Public License (Version 2) as 
# published by the Free Software Foundation. 
# This program is distributed in the hope it will be useful, but WITHOUT 
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
# for more details. 
# You should have received a copy of the GNU General Public License along 
# with this program; if not, write to the Free Software Foundation, Inc., 
# 59 Temple Place - Suite 330, Boston MA 02111-1307, USA. 
# :>
 *
 ***************************************************************************
 * File Name  : board_api.c
 *
 * Description: interface for board level calls: flash,
 *              led, soft reset, free memory page, and memory dump.
 *              Adapted form flash_api.c by Yen Tran.
 *
 * Created on : 02/20/2002  seanl
 *
 ***************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>      /* open */
#include <unistd.h>     /* exit */
#include <sys/ioctl.h>  /* ioctl */
#include <memory.h>

#include <bcmtypes.h>
#include <board_api.h>

int boardIoctl(int board_ioctl, BOARD_IOCTL_ACTION action, char *string, int strLen, int offset, char *buf)
{
    BOARD_IOCTL_PARMS IoctlParms;
    int boardFd = 0;

    boardFd = open("/dev/brcmboard", O_RDWR);
    if ( boardFd != -1 ) {
        IoctlParms.string = string;
        IoctlParms.strLen = strLen;
        IoctlParms.offset = offset;
        IoctlParms.action = action;
        IoctlParms.buf    = buf;
        ioctl(boardFd, board_ioctl, &IoctlParms);
        close(boardFd);
        boardFd = IoctlParms.result;
    } else
        printf("Unable to open device /dev/brcmboard.\n");

    return boardFd;
}
#ifdef USE_ALL


/*****************************************************************************
* sysScratchPadGet - get the contents of persistent stratch pad flash memory
* INPUT:   tokenId -- token id, ASCIZ tokBuf (up to 15 char)
*          tokBuf  -- buf 
*          bufLen  -- buf len
* RETURNS: 0 - ok, -1 fail.
*/
int sysScratchPadGet(char *tokenId, char *tokBuf, int bufLen)
{
    return (boardIoctl(BOARD_IOCTL_FLASH_READ, SCRATCH_PAD, tokenId, bufLen, bufLen, tokBuf));
}

/*****************************************************************************
* sysScratchPadSet - write the contents of persistent scratch pad flash memory
* INPUT:   tokenId -- token id, ASCIZ tokBuf (up to 15 char)
*          tokBuf  -- buf 
*          bufLen  -- buf len
* RETURNS: 0 - ok, -1 fail.
*/
int sysScratchPadSet(char *tokenId, char *tokBuf, int bufLen)
{
    return (boardIoctl(BOARD_IOCTL_FLASH_WRITE, SCRATCH_PAD, tokenId, bufLen, bufLen, tokBuf));
}


/*****************************************************************************
* sysPersistentGet - get the contents of non-volatile RAM,
* RETURNS: OK, always.
*/
int sysNvRamGet(char *string, int strLen, int offset)
{
    return (boardIoctl(BOARD_IOCTL_FLASH_READ, NVRAM, string, strLen, offset, ""));
}

/*****************************************************************************
* sysPersistentSet - write the contents of non-volatile RAM
* RETURNS: OK, always.
*/
int sysNvRamSet(char *string, int strLen, int offset)
{
    return (boardIoctl(BOARD_IOCTL_FLASH_WRITE, NVRAM, string, strLen, offset, ""));
}

/*****************************************************************************
* sysNrPagesGet - returns number of free system pages.  Each page is 4K bytes.
* RETURNS: Number of 4K pages.
*/
int sysNrPagesGet(void)
{
    return (boardIoctl(BOARD_IOCTL_GET_NR_PAGES, 0, "", 0, 0, ""));
}

/*****************************************************************************
* sysDumpAddr - Dump kernel memory.
* RETURNS: OK, always.
*/
int sysDumpAddr(char *addr, int len)
{
    return (boardIoctl(BOARD_IOCTL_DUMP_ADDR, 0, addr, len, 0, ""));
}

/*****************************************************************************
* sysDumpAddr - Set kernel memory.
* RETURNS: OK, always.
*/
int sysSetMemory(char *addr, int size, unsigned long value )
{
    return (boardIoctl(BOARD_IOCTL_SET_MEMORY, 0, addr, size, (int) value, ""));
}

/*****************************************************************************
 * image points to image to be programmed to flash; size is the size (in bytes)
 * of the image.
 * if error, return -1; otherwise return 0
 */

#endif // USE_ALL

/*****************************************************************************
* sysPersistentGet - get the contents of persistent flash memory
* RETURNS: OK, always.
*/
int sysPersistentGet(char *string, int strLen, int offset)
{
    return (boardIoctl(BOARD_IOCTL_FLASH_READ, PERSISTENT, string, strLen, offset, ""));
}

/*****************************************************************************
* sysPersistenSet - write the contents of persistent Scrach Pad flash memory
* RETURNS: OK, always.
*/
int sysPersistentSet(char *string, int strLen, int offset)
{
    return (boardIoctl(BOARD_IOCTL_FLASH_WRITE, PERSISTENT, string, strLen, offset, ""));
}

#ifdef NNN_POT
//********************************************************************************
// Get PSI size
//********************************************************************************
unsigned int sysGetPot( int flag )
{
    return( boardIoctl(BOARD_IOCTL_GET_POT, 0, "", 0, flag, "") );
}
#endif
//********************************************************************************
// Get PSI size
//********************************************************************************
int sysGetPsiSize( void )
{
    return( boardIoctl(BOARD_IOCTL_GET_PSI_SIZE, 0, "", 0, 0, "") );
}

int sysFlashImageSet(void *image, int size, int addr,
    BOARD_IOCTL_ACTION imageType)
{
    int result;

    result = boardIoctl(BOARD_IOCTL_FLASH_WRITE, imageType, image, size, addr, "");

    return(result);
}

/*****************************************************************************
 * Get flash size 
 * return int flash size
 */
int sysFlashSizeGet(void)
{
    return (boardIoctl(BOARD_IOCTL_FLASH_READ, FLASH_SIZE, "", 0, 0, ""));
}

/*****************************************************************************
* kerSysMipsSoftReset - soft reset the mips. (reboot, go to 0xbfc00000)
* RETURNS: NEVER
*/
void sysMipsSoftReset(void)
{  
    boardIoctl(BOARD_IOCTL_MIPS_SOFT_RESET, 0, "", 0, 0, "");
}

//********************************************************************************
// Get Chip Id
//********************************************************************************
int sysGetChipId( void )
{
#if BCM6348A
return (0x6348A);
#elif BCM6348B
return (0x6348B);
#elif BCM6338A
return (0x6338A);
#elif BCM6338B
return (0x6338B);
#elif BCM6345A
return (0x6345A);
#elif BCM6345A2
return (0x6345A2);
#elif BCM6345B
return (0x6345B);
#else
return (0x6348);
#endif
//Odm Michael    return( boardIoctl(BOARD_IOCTL_GET_CHIP_ID, 0, "", 0, 0, "") );
}

/*==== add by Andrew (2004/09/14) ====*/
#if defined(CFG_XFER_TO_FACDEFLT)
int sysFactDefltGet(char *string, int strLen, int offset)
{
   return (boardIoctl(BOARD_IOCTL_FLASH_READ, FACTDEFLT, string, strLen, offset, ""));
}

int sysFactDefltSet(char *string, int strLen, int offset)
{
    return (boardIoctl(BOARD_IOCTL_FLASH_WRITE, FACTDEFLT, string, strLen, offset, ""));
}
#endif
/*==== end ====*/

//********************************************************************************
// Wakeup monitor task
//********************************************************************************
void sysWakeupMonitorTask(void)
{
    return (boardIoctl(BOARD_IOCTL_WAKEUP_MONITOR_TASK, 0, "", 0, 0, ""));
}

#ifdef USE_ALL
//********************************************************************************
// LED status display:  ADSL link: DOWN/UP, PPP: DOWN/STARTING/UP
//********************************************************************************
void sysLedCtrl(BOARD_LED_NAME ledName, BOARD_LED_STATE ledState)
{
    boardIoctl(BOARD_IOCTL_LED_CTRL, 0, "", (int)ledName, (int)ledState, "");
}


//********************************************************************************
// Get board id
//********************************************************************************
int sysGetBoardIdName(char *name, int length)
{
    return( boardIoctl(BOARD_IOCTL_GET_ID, 0, name, length, 0, "") );
}

//swda add,02/04/2005
//get serial number
#if SERIAL_NUMBER
int swdasysGetSerialNo(char *serial, int length)
{
    return( boardIoctl(BOARD_IOCTL_GET_SERIAL_NUMBER, 0, serial, length, 0, "") );
}
#endif
//swda add end

//swda add,04/08/2005
#if	DEFAULT_WLAN_WEP128
int swdasysGetWEP128Key(char *WEP128key, int length)
{
    return( boardIoctl(BOARD_IOCTL_GET_WEP128_KEY, 0, WEP128key, length, 0, "") );
}
#endif
//swda add end

//swda add,09/12/2005
int swdasysInfoBootComplete(void)
{
	return( boardIoctl(BOARD_IOCTL_INFO_BOOT_COMPLETE, 0, "", 0, 0, "") );
}
//swda add end

//swda add,09/13/2005
#if ODM_AUTO_PROVISION_LAN
int swdasysGetAutoProvisionGPIOstatus(void)
{
	return( boardIoctl(BOARD_IOCTL_AUTO_PROVISION_LAN, 0, "", 0, 0, "") );
}
#endif
//swda add end

//swda add,11/29/2006
int swdasysSetVar(char *name, int value)
{
    return( boardIoctl(BOARD_IOCTL_SET_VAR, 0, name, value, 0, "") );
}
//swda add end

//swda add,11/02/2005
/***************************************************************
Function: swdaSetAolTestLedState()
Description: Set AAA Test led state
Arguments: state - 0:off  
                   1:on for 30 seconds 
                   2:blinking for 30 seconds        
Returns: 1 - TRUE  else - FALSE
Author: Swda, 11/02/2005
***************************************************************/
#if AAA_TEST_LED
int swdaSetAolTestLedState(int state)
{
	return( boardIoctl(BOARD_IOCTL_SET_AAA_TEST_LED, 0, "", 0, state, "") );
}
#endif
//swda add end

//swda add,11/03/2005
#if	WLAN_ENABLE_CTRL_BUTTON
#define GET_WLAN_ENABLE_GPIO_STATUS	0	//the same as in board.c

int swdaGetWlanEnableGPIOstatus(void)
{
	return(boardIoctl(BOARD_IOCTL_WLAN_ENABLE_CTRL_BUTTON, PERSISTENT, "", GET_WLAN_ENABLE_GPIO_STATUS, 0, ""));
}
#endif
//swda add end

//********************************************************************************
// Get MAC Address
//********************************************************************************
int sysGetMacAddress( unsigned char *pucaAddr, unsigned long ulId )
{
    return(boardIoctl(BOARD_IOCTL_GET_MAC_ADDRESS, 0, pucaAddr, 6, (int) ulId, ""));
}


//********************************************************************************
// Release MAC Address
//********************************************************************************
int sysReleaseMacAddress( unsigned char *pucaAddr )
{
    return( boardIoctl(BOARD_IOCTL_RELEASE_MAC_ADDRESS, 0, pucaAddr, 6, 0, "") );
}


//********************************************************************************
// Get SDRAM size
//********************************************************************************
int sysGetSdramSize( void )
{
    return( boardIoctl(BOARD_IOCTL_GET_SDRAM_SIZE, 0, "", 0, 0, "") );
}


/*****************************************************************************
* sysGetBooline - get bootline
* RETURNS: OK, always.
*/
int sysGetBootline(char *string, int strLen)
{
    return (boardIoctl(BOARD_IOCTL_GET_BOOTLINE, 0, string, strLen, 0, ""));
}

/*****************************************************************************
* sysSetBootline - write the bootline to nvram
* RETURNS: OK, always.
*/
int sysSetBootline(char *string, int strLen)
{
    return (boardIoctl(BOARD_IOCTL_SET_BOOTLINE, 0, string, strLen, 0, ""));
}

//********************************************************************************
// Get MAC Address
//********************************************************************************
int sysGetBaseMacAddress( unsigned char *pucaAddr )
{
    return(boardIoctl(BOARD_IOCTL_GET_BASE_MAC_ADDRESS, 0, pucaAddr, 6, 0, ""));
}

//********************************************************************************
// Get number of Ethernet phys
//********************************************************************************
int sysGetNumEnet(void)
{
    return(boardIoctl(BOARD_IOCTL_GET_NUM_ENET, 0, "", 0, 0, ""));
}

//********************************************************************************
// Get CFE vesion info
//********************************************************************************
int sysGetCFEVersion(char *string, int strLen)
{
    return (boardIoctl(BOARD_IOCTL_GET_CFE_VER, 0, string, strLen, 0, ""));
}

//********************************************************************************
// Get board Ethernet configuration
//********************************************************************************
int sysGetEnetCfg(char *string, int strLen)
{
    return(boardIoctl(BOARD_IOCTL_GET_ENET_CFG, 0, string, strLen, 0, ""));
}

//********************************************************************************
// Set monitor loop file descriptor
//********************************************************************************
int sysSetMonitorFd(int fd)
{
    return (boardIoctl(BOARD_IOCTL_SET_MONITOR_FD, 0, "", 0, fd, ""));
}
//********************************************************************************
// Get Chip REV
//********************************************************************************
int sysGetChipRev( void )
{
    return( boardIoctl(BOARD_IOCTL_GET_CHIP_REV, 0, "", 0, 0, "") );
}
#endif // USE_ALL

