/*
  **
  ** cache.c - cache handling routines
  **
  ** Part of the dproxy package by Matthew Pratt. 
  **
  ** Copyright 1999 Matthew Pratt <mattpratt@yahoo.com>
  **
  ** This software is licensed under the terms of the GNU General 
  ** Public License (GPL). Please see the file COPYING for details.
  ** 
  **
*/
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <sys/file.h>
#include <ctype.h>
#include <unistd.h>

#include "cache.h"
#include "conf.h"

/** function prototypes for private functions*/
//static int cache_byname(FILE * , char *, char *);
static int cache_byname(FILE * , char *, char *, char [][]);
static char hostname1[64];
static char hostname2[64];

/*****************************************************************************
 *  search for a host by its name.
 *  
 *    This function first searches the cache for an entry. If the entry 
 *    was not found there, we will look into a dhcp "leases" file.
 * 
 *  @arg name  - name to find.
 *  @arg ip    - pointer to a buffer where to put the ip adress.
 *  
 *  @return 0 if an entry was found, 1 if not.
*****************************************************************************/
int cache_lookup_name(char *name, char ip[BUF_SIZE], char optionip[7][20])
{
  FILE * fp;

  //debug( "cache_lookup_name(%s)\n", name);
  
  /** check the cache */
  if( (fp = fopen( config.cache_file , "r")) != NULL) {
	 int result = 0;
	 //result = cache_byname(fp,name, ip);
	 result = cache_byname(fp,name, ip, optionip); 
	 fclose(fp);   
	 if( result > 0 ) {
		return 1;
	 }
  }
  
  return 0;
}
/*****************************************************************************
 * lookup a hostname in the cache file.
 *
 * This function will not lock the cache ! 
 * 
 *  @arg fp    - open file pointer for the cache file.
 *  @arg name  - name to find.
 *  @arg ip    - pointer to a buffer where to put the ip adress.
 *  
 *  @return 0 if an entry was found, 1 if not.
 *****************************************************************************/
static int cache_byname(FILE * fp, char *name, char ip[BUF_SIZE],char optionip[7][20])
{
	char line[BUF_SIZE];
	char *token;
	int i = 0;
	ip[0] = 0;
	//char optionip[7][20];

	/** make shure we are at the start of the cache */
  	rewind(fp);

	while( fgets(line, BUF_SIZE, fp) ){
  		//debug("cache file = %s\n",line);  	
#if 0	 	
	 	token = strtok( line, " ");
	 	if( !strcasecmp( token, name) ){
			token = strtok( NULL, " ");
			while( isalnum(*token) || (*token=='.') )ip[i++] = *token++;
			ip[i] = 0;
			return 1;
	 	}
#else
		token = strtok( line, " ");
		//debug("1. token = %s\n",token);
		if( !strcasecmp( token, name) ){
			token = strtok( NULL, " ");
			//debug("2. token = %s\n",token);
			strcpy(ip,token);
			//debug("ip = %s\n",ip);
			for (i=0;i<OPTION_IP;i++){
				if ((token = strtok( NULL, " "))){
					strcpy(optionip[i], token);
					//debug("optionip[%d] = %s\n",i,optionip[i]);
					if (strstr(optionip[i],".")==NULL){
						//debug("the fiels is time clear it\n");
						strcpy(optionip[i],"");
					}
				}
			}
			return 1;
		}
#endif	 	
	}

	return 0;
}

/*****************************************************************************/
int cache_lookup_ip(char *ip, char result[BUF_SIZE])
{
	FILE *fp;
  	char line[BUF_SIZE];
  	char *token;
  	int i = 0;
  	if( ip[0] == 0 ){
  		return 0;
  	}	
  	result[0] = 0;
  
  	fp = fopen( config.cache_file , "r");
  	if(!fp){
  		return 0;
  	}	
  	while( fgets(line, BUF_SIZE, fp) ){
#if 0  		
		strtok( line, " ");
	 	token = strtok( NULL, " ");
	 		if( !strncasecmp( token, ip, strlen(ip) ) ){
				while( isalnum(line[i]) || (line[i]=='.') )result[i] = line[i++];
					result[i] = 0;
				fclose(fp);
				return 1;
	 		}
#else
		strtok( line, " ");
	 	while((token = strtok( NULL, " "))!=NULL){
	 		//if( !strncasecmp( token, ip, strlen(ip) ) ){
	 		if( !strcasecmp( token, ip ) ){
				while( isalnum(line[i]) || (line[i]=='.') || (line[i]=='-'))result[i] = line[i++];
					result[i] = 0;
				fclose(fp);
				return 1;
	 		}
	 	}	
#endif	 	
  	}
  	fclose(fp);
  	return 0;
}
/*****************************************************************************
* save the name to the list.
* 
*
*****************************************************************************/
void cache_name_append(char *name, char *ip, char *ipstr)
{

	FILE *fp;
	char dummy[BUF_SIZE];
	char temp[7][20];
	//debug("start cache_name_append\n");
	fp = fopen( config.cache_file, "a");
	if(!fp){
		debug("*cache_name_append, Could not open cache file '%s' for writing",
			      config.cache_file);
	 	return;
  	}

  	/** check if another process already added this host to the cache */
  	//if( cache_byname(fp, name, dummy) != 0 ) {
  	if( cache_byname(fp, name, dummy, temp) != 0 ) {
		fclose(fp);
	  	return;
  	}

  	/** make shure that we at the end of the file. */
  	fseek(fp,0,SEEK_END);
  	//debug("*write new entry, name = %s ip = %s\n", name,ip);
  	//debug("ipstr=%sandytest\n",ipstr);
  	//debug("strlen(ipstr)=%d\n",strlen(ipstr));
  	ipstr[strlen(ipstr)-1]='\0'; // clear last space character
  	//debug("ipstr=%sandytest\n",ipstr);
  	/** write new entry */
	if (strstr(ipstr,".")){ //check ipstr is null or not
  		//debug("ipstr is not null\n");
  		fprintf( fp, "%s %s %s %ld\n", name, ip, ipstr ,time(NULL) );
  	} else{
  		//debug("ipstr is null\n");
  		fprintf( fp, "%s %s %ld\n", name, ip, time(NULL) );
	}
	fclose(fp);
  	//debug("end cache_name_append\n");  
}
/*****************************************************************************/
void cache_purge(int older_than)
{
	FILE *in_fp, *out_fp;
	char line[BUF_SIZE];
	char old_cache[1024];
	char *name, *ip=NULL, *time_made;
	int i=0;
	char tempip[1024];
	char temptime[20];
	
	tempip[0]='\0';
	temptime[0]='\0';
	
	in_fp = fopen( config.cache_file , "r");
	if(!in_fp){
	 debug_perror("Could not open old cache file");
	 /*return;*/
	}
	
	if( in_fp ) {
		sprintf( old_cache, "%s.old", config.cache_file );
	  	if( rename( config.cache_file, old_cache ) < 0 ){
	 		debug_perror("Could not move cache file");
	 		fclose(in_fp);
	 		return;
	  	}
  	}
  	out_fp = fopen( config.cache_file , "w");
  	if(!out_fp){
		if( in_fp ) {
	 		fclose(in_fp);
	 	}
	 	debug_perror("Could not open new cache file");
	 	return;
  	}

  	cache_add_hosts_entries(out_fp);
  	//debug("older_than=%d\n",older_than);
  	if( in_fp ) {
    	while( fgets(line, BUF_SIZE, in_fp) ){
	 		tempip[0]='\0';
	 		//debug("line=%s\n",line);
#if 1
			name = strtok( line, " ");
			//debug("name=%s\n",name);
			i=0;
			while((ip = strtok( NULL, " "))){
				//debug("ip=%s\n",ip);
				if (strstr(ip,".")){
					if (i>0)
						strcat(tempip," ");
					strcat(tempip,ip);
					//debug("1. tempip=%s\n",tempip);
					i++;
				} else {
					strcpy(temptime,ip);
					//debug("1. temptime=%s\n",temptime);
					break;
				} 
			}
			//debug("tempip=%s\n",tempip);
			//debug("temptime=%s\n",temptime);
			if( temptime[0]=='\0')continue;
                     if (strtoull(temptime, NULL, 10)==0)continue;
	 			//debug("temptime=%d\n",atoi( temptime ));
	 			//debug("time(NULL)=%d\n",time(NULL));
	 			if( time(NULL) - atoi( temptime ) < older_than ){
	 				fprintf( out_fp, "%s %s %s", name, tempip, temptime );
				}
//	}
#else	 		
	 		name = strtok( line, " ");
	 		ip = strtok( NULL, " ");
	 		time_made = strtok( NULL, " ");
	 		if(!time_made)continue;
			if ( (strcmp(hostname1,name) != 0) && (strcmp(hostname2,name) != 0)){
	 			//debug("time_made=%d\n",atoi( time_made ));
	 			//debug("time(NULL)=%d\n",time(NULL));
	 			if( time(NULL) - atoi( time_made ) < older_than ){
	 				fprintf( out_fp, "%s %s %s", name, ip, time_made );
				}
			}
#endif	 	
	 				
		}

		fclose(in_fp);
		unlink(old_cache);
	}

	fclose(out_fp);
}
/*****************************************************************************/
void cache_add_hosts_entries(FILE *cache_file)
{
	FILE *hosts_fp;
	char line[BUF_SIZE];
	//char *ip, *name;
	//char *ip2, *name2;
	char ip[16], name[64];
	char ip2[16], name2[64];
	char ip3[16], name3[64];
	int counter = 0;

	hosts_fp = fopen( config.hosts_file , "r");
	if( !hosts_fp ) {
		debug_perror("can not open 'hosts'-file ");
		return;
  	}
#if 1
	while( fgets(line, BUF_SIZE, hosts_fp) ){
		line[strlen(line) - 1] = 0; /* get rid of '\n' */
		sscanf(line,"%s\t%s", ip, name);
		if (counter != 1 && counter != 2){
			fprintf( cache_file, "%s %s %ld\n", name, ip, 0L );
		}
		if (counter == 1){
			strcpy(ip2, ip);
                     strcpy(name2, name);
		}	
		if (counter == 2){
			strcpy(ip3, ip);
                     strcpy(name3, name);
		}	
		counter++;
	}
	if (counter >= 3){
		if (strcmp(name2, name3) == 0){
			fprintf( cache_file, "%s %s %s %ld\n", name2, ip2, ip3, 0L );
		} else {
			fprintf( cache_file, "%s %s %ld\n", name2, ip2, 0L );
			fprintf( cache_file, "%s %s %ld\n", name3, ip3, 0L );
		}	
	} else {
		fprintf( cache_file, "%s %s %ld\n", name2, ip2, 0L );
	}
#else
	while( fgets(line, BUF_SIZE, hosts_fp) ){
		line[strlen(line) - 1] = 0; /* get rid of '\n' */
	 	ip = strtok( line, " \t");
	 	if( ip == NULL ) continue;  /* ignore blank lines */
	 	if( ip[0] == '#' )continue; /* ignore comments */
	 	while( (name = strtok( NULL, " \t" )) ){
	   		if(name[0] == '#')break;
			fprintf( cache_file, "%s %s %ld\n", name, ip, 0L );
		}
  	}
#endif
#if defined(CONFIG_LANG_VOYAGER)
	fprintf( cache_file, "api.home %s %ld\n", ip, 0L );
#if VOYAGER220V_MGCP_BBV || VOYAGER220V_SIP_BBV || VOYAGER220V_SIP_GEN || VOYAGER220V_MGCP_BTR || VOYAGER220V_SIP_BTR
	fprintf( cache_file, "voyager220v.home %s %ld\n", ip, 0L );
#else
	fprintf( cache_file, "voyager.home %s %ld\n", ip, 0L );
#endif
#endif	
	fclose(hosts_fp);
}

void get_hostname(){
	FILE *hosts_fp;
	char line[BUF_SIZE];
	char *ip, *name;
	int i=0;
	
	hostname1[0]='\0';
	hostname2[0]='\0';
	
	hosts_fp = fopen( config.hosts_file , "r");
	if( !hosts_fp ) {
		debug_perror("can not open 'hosts'-file ");
		return;
  	}

	while( fgets(line, BUF_SIZE, hosts_fp) ){
		line[strlen(line) - 1] = 0; /* get rid of '\n' */
	 	ip = strtok( line, " \t");
	 	if( ip == NULL ) continue;  /* ignore blank lines */
	 	if( ip[0] == '#' )continue; /* ignore comments */
	 	while( (name = strtok( NULL, " \t" )) ){
	   		if(name[0] == '#')break;
			if (i == 0)
				strcpy(hostname1,name);
			if (i == 1)
				strcpy(hostname2,name);
		}
		i++;
  	}
	fclose(hosts_fp);
}
