#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "dns_construct.h"

#define SET_UINT16_TO_N(buf, val, count) *(uint16*)buf = htons(val);count += 2; buf += 2
#define SET_UINT32_TO_N(buf, val, count) *(uint32*)buf = htonl(val);count += 4; buf += 4
/*****************************************************************************/
/* this function encode the plain string in name to the domain name encoding 
 * see decode_domain_name for more details on what this function does. */
int dns_construct_name(char *name, char *encoded_name)
{
  int i,j,k,n;

  k = 0; /* k is the index to temp */
  i = 0; /* i is the index to name */
  while( name[i] ){

	 /* find the dist to the next '.' or the end of the string and add it*/
	 for( j = 0; name[i+j] && name[i+j] != '.'; j++);
	 encoded_name[k++] = j;

	 /* now copy the text till the next dot */
	 for( n = 0; n < j; n++)
		encoded_name[k++] = name[i+n];
	
	 /* now move to the next dot */ 
	 i += j + 1;

	 /* check to see if last dot was not the end of the string */
	 if(!name[i-1])break;
  }
  encoded_name[k++] = 0;
  return k;
}
/*****************************************************************************/
int dns_construct_header(dns_request_t *m)
{
  char *ptr = m->original_buf;
  int dummy;
  SET_UINT16_TO_N( ptr, m->message.header.id, dummy );

  //Paul add 12/23/2003: Set the header flags to response
  #define DNS_QR         0x8100
  m->message.header.flags.flags = DNS_QR;
  
  SET_UINT16_TO_N( ptr, m->message.header.flags.flags, dummy );
  debug("dns_construct, (Response)m->message.header.flags.flags = %x\n",m->message.header.flags.flags);
  SET_UINT16_TO_N( ptr, m->message.header.qdcount, dummy );
  SET_UINT16_TO_N( ptr, m->message.header.ancount, dummy );
  SET_UINT16_TO_N( ptr, m->message.header.nscount, dummy );
  SET_UINT16_TO_N( ptr, m->message.header.arcount, dummy );
  
  return 0;
}

int dns_construct_header_error_name(dns_request_t *m)
{
  char *ptr = m->original_buf;
  int dummy;
  SET_UINT16_TO_N( ptr, m->message.header.id, dummy );

  m->message.header.flags.flags = 0x8103;
  
  SET_UINT16_TO_N( ptr, m->message.header.flags.flags, dummy );
  debug("dns_construct, (Response)m->message.header.flags.flags = %x\n",m->message.header.flags.flags);
  SET_UINT16_TO_N( ptr, m->message.header.qdcount, dummy );
  SET_UINT16_TO_N( ptr, m->message.header.ancount, dummy );
  SET_UINT16_TO_N( ptr, m->message.header.nscount, dummy );
  SET_UINT16_TO_N( ptr, m->message.header.arcount, dummy );
  
  return 0;
}

/*****************************************************************************/
void dns_construct_reply( dns_request_t *m )
{
	int len;
  	int i=0;

  	/* point to end of orginal packet */ 
 	m->here = &m->original_buf[m->numread];

  	m->message.header.ancount = 1;
  	/*andy lin add*/	
	for (i=0 ; i < OPTION_IP ;i++){
		//debug("1. m->ip_option[%d]=%s\n",i,m->ip_option[i]); 		
		if (strcmp(m->ip_option[i],"")!=0){
			m->message.header.ancount++;
		}
	}
	//debug("m->message.header.ancount=%d\n",m->message.header.ancount);	
	/*End andy lin add*/  
	m->message.header.flags.f.question = 1;
     
	dns_construct_header( m );

	if( m->message.question[0].type == A || m->message.question[0].type == ANY){
    	/* standard lookup so return an IP */
    	struct in_addr in;

    	inet_aton( m->ip, &in );
    	SET_UINT16_TO_N( m->here, 0xc00c, m->numread ); /* pointer to name */
    	SET_UINT16_TO_N( m->here, A, m->numread );      /* type */
    	SET_UINT16_TO_N( m->here, IN, m->numread );     /* class */
    	SET_UINT32_TO_N( m->here, 10000, m->numread );  /* ttl */
    	SET_UINT16_TO_N( m->here, 4, m->numread );      /* datalen */
    	memcpy( m->here, &in.s_addr, sizeof(in.s_addr) ); /* data */
    	m->numread += sizeof( in.s_addr);
    	/*andy lin add*/    
    	for (i=0 ; i < OPTION_IP ;i++){
			//debug("2. m->ip_option[%d]=%s\n",i,m->ip_option[i]);    		
			if (strcmp(m->ip_option[i],"")!=0){
				m->here=m->here+sizeof( in.s_addr);
    			inet_aton( m->ip_option[i], &in );
    			SET_UINT16_TO_N( m->here, 0xc00c, m->numread ); /* pointer to name */
    			SET_UINT16_TO_N( m->here, A, m->numread );      /* type */
    			SET_UINT16_TO_N( m->here, IN, m->numread );     /* class */
    			SET_UINT32_TO_N( m->here, 10000, m->numread );  /* ttl */
   				SET_UINT16_TO_N( m->here, 4, m->numread );      /* datalen */
    			memcpy( m->here, &in.s_addr, sizeof(in.s_addr) ); /* data */
    			m->numread += sizeof( in.s_addr);
			}
		}
		/*end andy lin add*/ 
	}else if ( m->message.question[0].type == PTR ){
    	/* reverse look up so we are returning a name */
    	SET_UINT16_TO_N( m->here, 0xc00c, m->numread ); /* pointer to name */
    	SET_UINT16_TO_N( m->here, PTR, m->numread );    /* type */
    	SET_UINT16_TO_N( m->here, IN, m->numread );     /* class */
    	SET_UINT32_TO_N( m->here, 10000, m->numread );  /* ttl */
    	len = dns_construct_name( m->cname, m->here + 2 );
    	SET_UINT16_TO_N( m->here, len, m->numread );      /* datalen */
    	m->numread += len;
	}
}
/*****************************************************************************/
void dns_construct_error_reply(dns_request_t *m)
{
  /* point to end of orginal packet */ 
  m->here = m->original_buf;

  m->message.header.flags.f.question = 1;
  m->message.header.flags.f.rcode = 2;
  dns_construct_header( m );
}

void dns_construct_error_name_reply(dns_request_t *m)
{
	/* point to end of orginal packet */ 
  	m->here = m->original_buf;

 	dns_construct_header_error_name( m );
}


int dns_construct_query(dns_request_t *m)
{
	char *ptr = m->original_buf;
  	int dummy;
  	int i=0;
  	debug("dns_construct_query_header, (Query)m->src_port=%d\n",m->src_port);
  	debug("dns_construct_query_header, (Query)m->numread=%d\n",m->numread);
  	
  	SET_UINT16_TO_N( ptr, m->message.header.id, dummy );

  	#define DNS_QUERY         0x0100
  	m->message.header.flags.flags = DNS_QUERY;
  
  	SET_UINT16_TO_N( ptr, m->message.header.flags.flags, dummy );
  	debug("dns_construct_query_header, (Query)m->message.header.flags.flags = %x\n",m->message.header.flags.flags);
  	m->message.header.qdcount =1;
  	SET_UINT16_TO_N( ptr, m->message.header.qdcount, dummy );
  	m->message.header.ancount =0;
  	SET_UINT16_TO_N( ptr, m->message.header.ancount, dummy );
  	m->message.header.nscount =0;
  	SET_UINT16_TO_N( ptr, m->message.header.nscount, dummy );
  	m->message.header.arcount =0;
  	SET_UINT16_TO_N( ptr, m->message.header.arcount, dummy );
  
  	return 0;
}