#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <arpa/inet.h>
#include <errno.h>
#include <stdarg.h>
#include <signal.h>
#include <syslog.h>

#include "dproxy.h"
#include "dns_decode.h"
#include "cache.h"
#include "conf.h"
#include "dns_list.h"
#include "dns_construct.h"
#include "dns_io.h"
/*****************************************************************************/
/*****************************************************************************/
int next_server_index = 0;
int dns_main_quit;
int dns_sock;
int glbretry = 0;
int glbpureBridge = 0;
fd_set rfds;
dns_request_t *dns_request_list;
/*****************************************************************************/
int is_connected()
{
  FILE *fp;

  if(!config.ppp_detect)return 1;

  fp = fopen( config.ppp_device_file, "r" );
  if(!fp)return 0;
  fclose(fp);
  return 1;
}
/*****************************************************************************/
int dns_init()
{
	struct sockaddr_in sa;
	struct in_addr ip;
	
	/* Clear it out */
	memset((void *)&sa, 0, sizeof(sa));
	  
	dns_sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	
	/* Error */
	if( dns_sock < 0 ){
		debug_perror("Could not create socket");
		exit(1);
	} 
	
	ip.s_addr = INADDR_ANY;
	sa.sin_family = AF_INET;
	memcpy((void *)&sa.sin_addr, (void *)&ip, sizeof(struct in_addr));
	sa.sin_port = htons(PORT);
	
	/* bind() the socket to the interface */
	if (bind(dns_sock, (struct sockaddr *)&sa, sizeof(struct sockaddr)) < 0){
		debug_perror("dns_init: bind: Could not bind to port");
		exit(1);
	}
	
	dns_main_quit = 0;
	
	FD_ZERO( &rfds );
	FD_SET( dns_sock, &rfds );
	
	dns_request_list = NULL;
	
	cache_purge( config.purge_time );
	
	return 1;
}
int getdnslive(){
	FILE* fs = fopen("/var/dnslive", "r");
	int dnslive=-1;
	
	if ( fs != NULL ) {
    	fscanf(fs,"%d",&dnslive);
    	fclose(fs);
   	}

	return dnslive;
}

int isWanUp() 
{
   char data[10];
   int res = 0,wanStatus = 0;

   FILE* fp = fopen("/var/wanStatus", "r");
   if ( fp != NULL ) {
      //fgets(data, 10, fp);
      fscanf(fp,"%d",&wanStatus);
      fclose(fp);
   }

   //if(data[0] == '6') res = 1;
   if(wanStatus == 6) res = 1;
   return res;
}

/*****************************************************************************/
void dns_handle_new_query(dns_request_t *m)
{
	struct in_addr in;
	int retval = -1;
	FILE *resolv_fp;
	char line[64];
	char buff_name[32], buff_ip[32];
	int count =0;
	int dnslive=0;
	int i=0;
	//char optionip[7][20];

	for (i=0 ; i < OPTION_IP ;i++){ //clear option ip
		strcpy(m->ip_option[i],"");
	}	
  	//if( m->message.question[0].type == A || m->message.question[0].type == AAA){
  	if( m->message.question[0].type == A || m->message.question[0].type == ANY){
    	/* standard query */
    	//retval = cache_lookup_name( m->cname, m->ip );
    	retval = cache_lookup_name( m->cname, m->ip, m->ip_option );
  	}else if( m->message.question[0].type == PTR ){
    	/* reverse lookup */
    	retval = cache_lookup_ip( m->ip, m->cname );
  	}
  	/*else if (m->message.question[0].type == SRV || m->message.question[0].type == MX
  			|| m->message.question[0].type == AAA || m->message.question[0].type == NAPTR
  			|| m->message.question[0].type == NS){*/
  	else if (m->message.question[0].type != A && m->message.question[0].type != PTR
  			&& m->message.question[0].type != ANY){		
  		retval = 0;
  	}
  	//debug("dns_handle_new_query, (Query) m->message.header.flags.flags = %x\n",m->message.header.flags.flags);
  	//debug("dns_handle_new_query, m->cname = %s ............... m->ip = %s\n",m->cname, m->ip);
  	//debug("dns_handle_new_query, retval=%d\n",retval);
  	switch( retval ){
    	case 0:
			//Paul add 12/26/2003: Check dns file exist or not, if not sent out fake dns packet
			if (glbpureBridge == 0){//routing mode
				resolv_fp = fopen( "/var/fyi/sys/dns" , "r");
				if( !resolv_fp ) {
					/*!!! Using fake dns */
					strcpy(buff_ip, "128.9.0.107");
					inet_aton(buff_ip, &in );
					debug("dns_handle_new_query, Sent Request To %s\n",buff_ip);
					dns_write_packet( dns_sock, in, PORT, m );
					return;
				} else {
					debug("dns_handle_new_query, Adding %s to list, id: %d\n", m->cname,m->message.header.id);
					dns_request_list = dns_list_add( dns_request_list, m );
					/*!!! relay the query untouched */	
					while( fgets(line, 64, resolv_fp) ){
						line[strlen(line) - 1] = 0; // get rid of '\n'
						//debug("dns_handle_new_query, line=%s\n", line);
						sscanf(line,"%s %s",buff_name,buff_ip);
						//debug("dns_handle_new_query, buff_name = %s  buff_ip = %s\n",buff_name,buff_ip);
						strcpy(config.name_server[count], buff_ip);
						//debug("dns_handle_new_query, config.name_server[%d]: %s\n",count,config.name_server[count]);
						count++;	
					}
					inet_aton( config.name_server[next_server_index], &in );
					//inet_aton( buff_ip, &in );
					//debug("dns_handle_new_query, Sent Request To %s\n",buff_ip);
					debug("dns_handle_new_query, Sent Request To %s\n",config.name_server[next_server_index]);
					debug("m->message.header.flags.f.question = %d\n",m->message.header.flags.f.question);
					dns_write_packet( dns_sock, in, PORT, m );
				}	 
       			fclose(resolv_fp);
       		} else {//pure bridge mode
       			dns_construct_error_name_reply( m );
      			dns_write_packet( dns_sock, m->src_addr, m->src_port, m );
       		}	
		break;
    	case 1:
      		dns_construct_reply( m );
      		dns_write_packet( dns_sock, m->src_addr, m->src_port, m );
      		debug("dns_handle_new_query, Cache hit\n");
      	break;
    	default:
      	debug("dns_handle_new_query, Unknown query type: %d\n", m->message.question[0].type );
	}
}
/*****************************************************************************/
void dns_handle_request(dns_request_t *m)
{
	dns_request_t *ptr = NULL;
	struct in_addr hinetdns;
	 
  	/* request may be a new query or a answer from the upstream server */
	ptr = dns_list_find_by_id( dns_request_list, m );
	
  	if( ptr != NULL ){
		//Paul add 12/25/2003: Ignore the annoying flags
		debug("Found query in list\n");
		/* message may be a response */
		debug("*m->message.header.flags.f.question: %d\n", m->message.header.flags.f.question);
		debug("*m->message.header.flags.f.rcode: %d\n", m->message.header.flags.f.rcode);
		debug("*m->message.header.flags.f.recursion_avail: %d\n", m->message.header.flags.f.recursion_avail);
		debug("*m->message.header.flags.f.want_recursion: %d\n", m->message.header.flags.f.want_recursion);		
		debug("dns_handle_request m->ip=%s\n",m->ip);
		debug("dns_handle_request m->cname=%s\n",m->cname);
		//debug("m->message.header.qdcount=%d\n",m->message.header.qdcount);
		//if (m->message.header.qdcount >=1){
		//	debug("m->message.question[0].type=%d\n",m->message.question[0].type);
		//}
		if( m->message.header.flags.f.question == 0 ){
			if( m->message.header.flags.f.rcode != NULL ){ // lookup was succesful
#if 0			
				dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, m );
				debug("Replying with answer from %s\n", inet_ntoa( m->src_addr ));
				dns_request_list = dns_list_remove( dns_request_list, ptr );
				debug("dns_handle_request m->ip=%s\n",m->ip);
			
				if (strcmp(m->ip,"")==0){			
					debug("m->ip is null,can't write to cache file.\n");
					next_server_index++;
					if (next_server_index>=2)
						next_server_index =0;
				} else {
					debug("Cache append: %s ----> %s\n", m->cname, m->ip );
					cache_name_append( m->cname, m->ip, m->ipstr);
				}
#else			
				/*************************************************************
				* Dproxy will redirect NAPTR, AAA, MX, SRV type response     *
				* packet to client.                                          *
				* Andy (2005/10/03)                                          *
				***************************************************************/
				if (m->message.header.qdcount >= 1){
					/*if (m->message.question[0].type == NAPTR
						|| m->message.question[0].type == SRV
						|| m->message.question[0].type == AAA
						|| m->message.question[0].type == MX 
						|| m->message.question[0].type == NS){*/
					if (m->message.question[0].type != A
						&& m->message.question[0].type != PTR
						&& m->message.question[0].type != ANY){	
						dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, m );
						debug("Replying NAPTR,SRV,AAA,MX type with answer from %s by question 0\n", inet_ntoa( m->src_addr ));
						dns_request_list = dns_list_remove( dns_request_list, ptr );
						glbretry = 0;
						return;
					}
				}				
				if (strcmp(m->ip,"")==0){			
					debug("m->ip is null,can't write to cache file.\n");
					debug("ptr->src_addr=%s\n", inet_ntoa( ptr->src_addr ));
					debug("ptr->src_port=%d\n", ptr->src_port);
					debug("dns_handle_request m->message.header.arcount=%d\n",m->message.header.arcount);
				
					if (strcmp(config.name_server[next_server_index],"168.95.1.1")==0){
						debug("Because answer is null, and old query dns server is 168.95.1.1\n");
							//if (m->message.header.arcount > 0 && (m->message.arecord[0].type == A || m->message.arecord[0].type == AAA) )
						if (m->message.header.arcount > 0 && (m->message.arecord[0].type == A) ){
							struct in_addr *addr;
							if (glbretry==0){
								addr = (struct in_addr *)m->message.arecord[0].data;
								debug("Because answer is null, retry query to %s from additional records\n",inet_ntoa(addr[0]));
								dns_construct_query(m);
								dns_write_packet( dns_sock, addr[0], PORT, m );
								glbretry++;
							} else {
								glbretry = 0;
								debug("1......Just retry one time\n");
								dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, m );
								dns_request_list = dns_list_remove( dns_request_list, ptr );
							}
						} else {
							debug("Because answer is null, and additional records is null too\n");
							dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, m );
							dns_request_list = dns_list_remove( dns_request_list, ptr );
							glbretry = 0;
						}
					} else {
						debug("Because answer is null, and old query dns server is not 168.95.1.1\n");
						hinetdns.s_addr = 0xA85F0101;
						debug("Because answer is null, retry query to %s\n",inet_ntoa(hinetdns));
						if (glbretry==0){
							dns_construct_query(m);
							dns_write_packet( dns_sock, hinetdns, PORT, m );
							glbretry++;
						} else {
							glbretry = 0;
							debug("2......Just retry one time\n");
							dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, m );
							dns_request_list = dns_list_remove( dns_request_list, ptr );
						}	
					} 		
					next_server_index++;
					if (next_server_index >= 2 || (strcmp(config.name_server[1],"0.0.0.0")==0))
						next_server_index =0;		
				} else {
					dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, m );
					debug("Replying with answer from %s\n", inet_ntoa( m->src_addr ));
					dns_request_list = dns_list_remove( dns_request_list, ptr );
					/*************************************************************************
					* If client use dig tool, and run "dig @192.168.1.1 www.google.com PTR"  *
					* error command. m->cname value is null.                                 *
					* Andy (2005/10/03)                                                      *
					**************************************************************************/
					if (strcmp(m->cname,"")!=0){
						debug("Cache append: %s ----> %s\n", m->cname, m->ip );
						cache_name_append( m->cname, m->ip, m->ipstr);
					}	
					glbretry = 0;
			
				}
#endif			
			}
		} else if( m->message.header.flags.f.question == 1 ){
			if( m->message.header.flags.f.rcode != NULL ){
				/*************************************************************
				* Dproxy will redirect NAPTR, AAA, MX, SRV type response     *
				* packet to client.                                          *
				* Andy (2005/10/03)                                          *
				**************************************************************/
				if (m->message.header.qdcount >= 1){
					/*if (m->message.question[0].type == NAPTR
						|| m->message.question[0].type == SRV
						|| m->message.question[0].type == AAA
						|| m->message.question[0].type == MX 
						|| m->message.question[0].type == NS){*/
					if (m->message.question[0].type != A
						&& m->message.question[0].type != PTR
						&& m->message.question[0].type != ANY){	
						dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, m );
						debug("Replying NAPTR,SRV,AAA,MX type with answer from %s  by question 1\n", inet_ntoa( m->src_addr ));
						dns_request_list = dns_list_remove( dns_request_list, ptr );
						glbretry = 0;
						return;
					}
			}
				
			debug("*dns_handle_request, send error back\n");
			/* send error back */
			dns_construct_error_reply(ptr);
			dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, ptr );
			dns_request_list = dns_list_remove( dns_request_list, ptr );
			glbretry = 0;
		}
	}

    /*if( m->message.header.flags.f.question == 1 ){
      dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, m );
      debug("Replying with answer from %s\n", inet_ntoa( m->src_addr ));
      dns_request_list = dns_list_remove( dns_request_list, ptr );
      if( m->message.header.flags.f.rcode == 0 ){ // lookup was succesful
	debug("Cache append: %s ----> %s\n", m->cname, m->ip );
	cache_name_append( m->cname, m->ip );
      }*/

    //}else{
    //  debug("Duplicate query\n");
    //}
  	}else{
		dns_handle_new_query( m );
	}
	

}
/*****************************************************************************/

void setpdns (){
	next_server_index=0;
}

void setsdns (){
	next_server_index=1;
}


int dns_main_loop()
{
	struct timeval tv;
	fd_set active_rfds;
	int retval;
	dns_request_t m;
	dns_request_t *ptr, *next;
	int purge_time = config.purge_time / 60;
	struct in_addr in;

	FILE* fp = fopen("/var/pureBridge", "r");
	if ( fp != NULL ) {
		fscanf(fp,"%d",&glbpureBridge);
		fclose(fp);
	}

	while( !dns_main_quit ){
		/* set the one second time out */
    	tv.tv_sec = 1;
    	tv.tv_usec = 0;

    	/* now copy the main rfds in the active one as it gets modified by select*/
    	active_rfds = rfds;

    	retval = select( FD_SETSIZE, &active_rfds, NULL, NULL, &tv );
    	if (retval){
      		//debug("dns_main_loop, data is now available, retval = %d\n",retval);
      		dns_read_packet( dns_sock, &m );
      		m.time_pending = 0;
      		dns_handle_request( &m );
    	}else{
      		/* select time out */
      		ptr = dns_request_list;

      		while( ptr ){
				next = ptr->next;
				/* Resend query to a new nameserver if response
	 			* has not been received from the current nameserver */
	
				ptr->time_pending++;
	
				if( ptr->time_pending > DNS_TIMEOUT ){
	  				debug("dns_main_loop, Request timed out\n");
	  				/* send error back */
	  				dns_construct_error_reply(ptr);
	  				dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, ptr );
	  				dns_request_list = dns_list_remove( dns_request_list, ptr );
				}
				//Paul add 12/26/2003: Switching dns server
				debug("dns_main_loop, ptr->time_pending (%d) percent DNS_SERVER_TIMEOUT (%d) = %d\n",ptr->time_pending,DNS_SERVER_TIMEOUT,ptr->time_pending % DNS_SERVER_TIMEOUT);
				if( ( ptr->time_pending % DNS_SERVER_TIMEOUT == 0) && ( ptr->time_pending < DNS_TIMEOUT) ){
	   				debug("dns_main_loop, next_server_index(old) = %d\n",next_server_index);
          			/* Send a request to the next nameserver */
          			next_server_index = ptr->time_pending/DNS_SERVER_TIMEOUT;
          			debug("dns_main_loop, next_server_index(new) = %d\n",next_server_index);
	  				/* If already the maximum number of supported servers have
	    			been tried out then remove the request from the list and
	    			send a response  back */
	    			debug("config.name_server[%d] = %s\n",next_server_index,config.name_server[next_server_index]);
            		//if(next_server_index >= MAX_NAME_SERVER ||(config.name_server[next_server_index]  == NULL ))
	  				if( next_server_index >= MAX_NAME_SERVER ||(strcmp(config.name_server[next_server_index],"0.0.0.0")==0) ){
	  					/* send error back */
						debug("All Servers Tried: No Response\n");
	  					dns_construct_error_reply(ptr);
	  					dns_write_packet( dns_sock, ptr->src_addr, ptr->src_port, ptr );
	  					dns_request_list = dns_list_remove( dns_request_list, ptr );
               			debug("Sent a DNS error message to Client \n");
               			if(next_server_index >= 2 || (strcmp(config.name_server[next_server_index],"0.0.0.0")==0) )
                			next_server_index = 0;
               			debug("dns_main_loop, next_server_index(new) = %d\n",next_server_index);
	    			} else {
	      				/* Create the new server IP address and relay the query untouched */
              			inet_aton( config.name_server[next_server_index], &in );
	      				debug("Didn't get a response from last server\n");
	      				debug("Sending a new request to %s\n", config.name_server[next_server_index]);
	      				/* Send the new request to the next name server */
	      				dns_write_packet( dns_sock, in, PORT, ptr );	
	    			}

				}
				ptr = next;
      		} /* while(ptr) */

      		/* purge cache */
			purge_time--;
      		if( !purge_time ){
				cache_purge( config.purge_time );
				purge_time = config.purge_time / 60;
      		} 
		} /* if (retval) */
	}  
  	return 0;
}
/*****************************************************************************/
void debug_perror( char * msg ) {
	debug( "%s : %s\n" , msg , strerror(errno) );
}
/*****************************************************************************/
void debug(char *fmt, ...)
{
#define MAX_MESG_LEN 1024
  
  va_list args;
  char text[ MAX_MESG_LEN ];
  
  sprintf( text, "[ %d ]: ", getpid());
  va_start (args, fmt);
  vsnprintf( &text[strlen(text)], MAX_MESG_LEN - strlen(text), fmt, args);
  va_end (args);
  
  if( config.debug_file[0] ){
    FILE *fp;
    fp = fopen( config.debug_file, "a");
    if(!fp){
      syslog( LOG_ERR, "could not open log file %m" );
      return;
    }
    fprintf( fp, "%s", text);
    fclose(fp);
  }
  
  /** if not in daemon-mode stderr was not closed, use it. */
  if( ! config.daemon_mode ) {
    fprintf( stderr, "%s", text);
  }
}
/*****************************************************************************
 * print usage informations to stderr.
 * 
 *****************************************************************************/
void usage(char * program , char * message ) {
  fprintf(stderr,"%s\n" , message );
  fprintf(stderr,"usage : %s [-c <config-file>] [-d] [-h] [-P]\n", program );
  fprintf(stderr,"\t-c <config-file>\tread configuration from <config-file>\n");
  fprintf(stderr,"\t-d \t\trun in debug (=non-daemon) mode.\n");
  fprintf(stderr,"\t-P \t\tprint configuration on stdout and exit.\n");
  fprintf(stderr,"\t-h \t\tthis message.\n");
}
/*****************************************************************************
 * get commandline options.
 * 
 * @return 0 on success, < 0 on error.
 *****************************************************************************/
int get_options( int argc, char ** argv ) 
{
  char c = 0;
  int not_daemon = 0;
  int want_printout = 0;
  char * progname = argv[0];

  conf_defaults();

  while( (c = getopt( argc, argv, "c:dhP")) != EOF ) {
    switch(c) {
	 case 'c':
  		conf_load(optarg);
		break;
	 case 'd':
		not_daemon = 1;
		break;
	 case 'h':
		usage(progname,"");
		return -1;
	 case 'P':
		want_printout = 1;
		break;
	 default:
		usage(progname,"");
		return -1;
    }
  }
  
  /** unset daemon-mode if -d was given. */
  if( not_daemon ) {
	 config.daemon_mode = 0;
  }
  
  if( want_printout ) {
	 conf_print();
	 exit(0);
  }
  return 0;
}
/*****************************************************************************/
void sig_hup (int signo)
{
  signal(SIGHUP, sig_hup); /* set this for the next sighup */
  conf_load (config.config_file);
}
/*****************************************************************************/

#ifdef BUILD_STATIC
int dproxy_main(int argc, char **argv)
#else
int main(int argc, char **argv)
#endif
{
	system("mkdir /var/cache 2>dev/null");
	/* get commandline options, load config if needed. */
	if(get_options( argc, argv ) < 0 ) {
		exit(1);
		//exit(0);
  	}

	signal(SIGHUP, sig_hup);
	signal(SIGUSR1,setpdns);
	signal(SIGUSR2,setsdns);
		
	dns_init();
	get_hostname();
	if (config.daemon_mode) {
		/* Standard fork and background code */
    	switch (fork()) {
	 		case -1:	/* Oh shit, something went wrong */
				debug_perror("fork");
				exit(-1);
	 		case 0:	/* Child: close off stdout, stdin and stderr */
				close(0);
				close(1);
				close(2);
				break;
	 		default:	/* Parent: Just exit */
				exit(0);
    	}
	}

	dns_main_loop();

	return 0;
}
