/* HTTP CLIENT */
/* Andy modify (2005/05/26) */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/param.h>
#include <sys/uio.h>
#include <unistd.h>
#include <sys/poll.h>

#include "http-client.h"
#include "bcmtypes.h"
#include "board.h"
#include "board_api.h"
#include "bcmTag.h"
#include "syscall.h"

#define  BUF_SIZE          2048
#define  UPLOAD_OVERHEAD   256
#define  MIN_UPLOAD_LEN    2 * 1024
#define  WEB_UPLOAD_IMAGE    1       // for image upload
#define  WEB_UPLOAD_SETTINGS 2       // for configure file upload

char *curPtr = NULL;
char *imageBufPtrPtr = NULL;
int s; 

//#define     DEBUG_UPLOAD            // if uncommented,  enable debug display
#ifdef DEBUG_UPLOAD

static const char hextable[16] = "0123456789ABCDEF";
void dumpHex(unsigned char *start, int len)
{
    unsigned char *ptr = start,
    *end = start + len;

    while (ptr < end)
    {
        long offset = ptr - start;
        unsigned char text[120],
        *p = text;
        while (ptr < end && p < &text[16 * 3])
        {
            *p++ = hextable[*ptr >> 4];
            *p++ = hextable[*ptr++ & 0xF];
            *p++ = ' ';
        }
        p[-1] = 0;
        printf("%4lX %s\n", offset, text);
    }
}

#endif

static int imageLen = 0;

#ifdef INCLUDE_SIGNATURE

#define F1(x, y, z) (z ^ (x & (y ^ z)))
#define F2(x, y, z) F1(z, x, y)
#define F3(x, y, z) (x ^ y ^ z)
#define F4(x, y, z) (y ^ (x | ~z))

#define MD5STEP(f, w, x, y, z, data, s) ( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )

void BRCM_MD5Init(struct BRCM_MD5Context *ctx) {
    ctx->buf[0] = 0x67452301;
    ctx->buf[1] = 0xefcdab89;
    ctx->buf[2] = 0x98badcfe;
    ctx->buf[3] = 0x10325476;

    ctx->bits[0] = 0;
    ctx->bits[1] = 0;
}

void byteReverse (unsigned char *buf, unsigned longs) {
    unsigned int t;
    do {
        t = (unsigned int)((unsigned)buf[3]<<8 | buf[2]) << 16 | ((unsigned)buf[1]<<8 | buf[0]);
        *(unsigned int *)buf = t;
        buf += 4;
    } while (--longs);
}

void BRCM_MD5Transform(unsigned int buf[4], unsigned int const in[16])
{
    register unsigned int a, b, c, d;

    a = buf[0];
    b = buf[1];
    c = buf[2];
    d = buf[3];

    MD5STEP(F1, a, b, c, d, in[0 ]+0xd76aa478,  7);
    MD5STEP(F1, d, a, b, c, in[1 ]+0xe8c7b756, 12);
    MD5STEP(F1, c, d, a, b, in[2 ]+0x242070db, 17);
    MD5STEP(F1, b, c, d, a, in[3 ]+0xc1bdceee, 22);
    MD5STEP(F1, a, b, c, d, in[4 ]+0xf57c0faf,  7);
    MD5STEP(F1, d, a, b, c, in[5 ]+0x4787c62a, 12);
    MD5STEP(F1, c, d, a, b, in[6 ]+0xa8304613, 17);
    MD5STEP(F1, b, c, d, a, in[7 ]+0xfd469501, 22);
    MD5STEP(F1, a, b, c, d, in[8 ]+0x698098d8,  7);
    MD5STEP(F1, d, a, b, c, in[9 ]+0x8b44f7af, 12);
    MD5STEP(F1, c, d, a, b, in[10]+0xffff5bb1, 17);
    MD5STEP(F1, b, c, d, a, in[11]+0x895cd7be, 22);
    MD5STEP(F1, a, b, c, d, in[12]+0x6b901122,  7);
    MD5STEP(F1, d, a, b, c, in[13]+0xfd987193, 12);
    MD5STEP(F1, c, d, a, b, in[14]+0xa679438e, 17);
    MD5STEP(F1, b, c, d, a, in[15]+0x49b40821, 22);

    MD5STEP(F2, a, b, c, d, in[1 ]+0xf61e2562,  5);
    MD5STEP(F2, d, a, b, c, in[6 ]+0xc040b340,  9);
    MD5STEP(F2, c, d, a, b, in[11]+0x265e5a51, 14);
    MD5STEP(F2, b, c, d, a, in[0 ]+0xe9b6c7aa, 20);
    MD5STEP(F2, a, b, c, d, in[5 ]+0xd62f105d,  5);
    MD5STEP(F2, d, a, b, c, in[10]+0x02441453,  9);
    MD5STEP(F2, c, d, a, b, in[15]+0xd8a1e681, 14);
    MD5STEP(F2, b, c, d, a, in[4 ]+0xe7d3fbc8, 20);
    MD5STEP(F2, a, b, c, d, in[9 ]+0x21e1cde6,  5);
    MD5STEP(F2, d, a, b, c, in[14]+0xc33707d6,  9);
    MD5STEP(F2, c, d, a, b, in[3 ]+0xf4d50d87, 14);
    MD5STEP(F2, b, c, d, a, in[8 ]+0x455a14ed, 20);
    MD5STEP(F2, a, b, c, d, in[13]+0xa9e3e905,  5);
    MD5STEP(F2, d, a, b, c, in[2 ]+0xfcefa3f8,  9);
    MD5STEP(F2, c, d, a, b, in[7 ]+0x676f02d9, 14);
    MD5STEP(F2, b, c, d, a, in[12]+0x8d2a4c8a, 20);

    MD5STEP(F3, a, b, c, d, in[5 ]+0xfffa3942,  4);
    MD5STEP(F3, d, a, b, c, in[8 ]+0x8771f681, 11);
    MD5STEP(F3, c, d, a, b, in[11]+0x6d9d6122, 16);
    MD5STEP(F3, b, c, d, a, in[14]+0xfde5380c, 23);
    MD5STEP(F3, a, b, c, d, in[1 ]+0xa4beea44,  4);
    MD5STEP(F3, d, a, b, c, in[4 ]+0x4bdecfa9, 11);
    MD5STEP(F3, c, d, a, b, in[7 ]+0xf6bb4b60, 16);
    MD5STEP(F3, b, c, d, a, in[10]+0xbebfbc70, 23);
    MD5STEP(F3, a, b, c, d, in[13]+0x289b7ec6,  4);
    MD5STEP(F3, d, a, b, c, in[0 ]+0xeaa127fa, 11);
    MD5STEP(F3, c, d, a, b, in[3 ]+0xd4ef3085, 16);
    MD5STEP(F3, b, c, d, a, in[6 ]+0x04881d05, 23);
    MD5STEP(F3, a, b, c, d, in[9 ]+0xd9d4d039,  4);
    MD5STEP(F3, d, a, b, c, in[12]+0xe6db99e5, 11);
    MD5STEP(F3, c, d, a, b, in[15]+0x1fa27cf8, 16);
    MD5STEP(F3, b, c, d, a, in[2 ]+0xc4ac5665, 23);

    MD5STEP(F4, a, b, c, d, in[0 ]+0xf4292244,  6);
    MD5STEP(F4, d, a, b, c, in[7 ]+0x432aff97, 10);
    MD5STEP(F4, c, d, a, b, in[14]+0xab9423a7, 15);
    MD5STEP(F4, b, c, d, a, in[5 ]+0xfc93a039, 21);
    MD5STEP(F4, a, b, c, d, in[12]+0x655b59c3,  6);
    MD5STEP(F4, d, a, b, c, in[3 ]+0x8f0ccc92, 10);
    MD5STEP(F4, c, d, a, b, in[10]+0xffeff47d, 15);
    MD5STEP(F4, b, c, d, a, in[1 ]+0x85845dd1, 21);
    MD5STEP(F4, a, b, c, d, in[8 ]+0x6fa87e4f,  6);
    MD5STEP(F4, d, a, b, c, in[15]+0xfe2ce6e0, 10);
    MD5STEP(F4, c, d, a, b, in[6 ]+0xa3014314, 15);
    MD5STEP(F4, b, c, d, a, in[13]+0x4e0811a1, 21);
    MD5STEP(F4, a, b, c, d, in[4 ]+0xf7537e82,  6);
    MD5STEP(F4, d, a, b, c, in[11]+0xbd3af235, 10);
    MD5STEP(F4, c, d, a, b, in[2 ]+0x2ad7d2bb, 15);
    MD5STEP(F4, b, c, d, a, in[9 ]+0xeb86d391, 21);

    buf[0] += a;
    buf[1] += b;
    buf[2] += c;
    buf[3] += d;
}

void BRCM_MD5Update( struct BRCM_MD5Context *ctx, unsigned char const *buf, unsigned len)
{
    unsigned int t;
    /* Update bitcount */

    t = ctx->bits[0];
    if ((ctx->bits[0] = t + ((unsigned int)len << 3)) < t)
        ctx->bits[1]++; /* Carry from low to high */
    ctx->bits[1] += len >> 29;

    t = (t >> 3) & 0x3f;    /* Bytes already in shsInfo->data */

    /* Handle any leading odd-sized chunks */

    if ( t ) {
        unsigned char *p = (unsigned char *)ctx->in + t;

        t = 64-t;
        if (len < t) {
            memcpy(p, buf, len);
            return;
        }
        memcpy(p, buf, t);
        byteReverse(ctx->in, 16);
        BRCM_MD5Transform(ctx->buf, (unsigned int *)ctx->in);
        buf += t;
        len -= t;
    }

    /* Process data in 64-byte chunks */

    while (len >= 64) {
        memcpy(ctx->in, buf, 64);
        byteReverse(ctx->in, 16);
        BRCM_MD5Transform(ctx->buf, (unsigned int *)ctx->in);
        buf += 64;
        len -= 64;
    }

    /* Handle any remaining bytes of data. */

    memcpy(ctx->in, buf, len);

    return;
}

void BRCM_MD5Final(unsigned char digest[16], struct BRCM_MD5Context *ctx)
{
    unsigned count;
    unsigned char *p;

    /* Compute number of bytes mod 64 */
    count = (ctx->bits[0] >> 3) & 0x3F;

    /* Set the first char of padding to 0x80.  This is safe since there is
       always at least one byte free */
    p = ctx->in + count;
    *p++ = 0x80;

    /* Bytes of padding needed to make 64 bytes */
    count = 64 - 1 - count;

    /* Pad out to 56 mod 64 */
    if (count < 8) {
        /* Two lots of padding:  Pad the first block to 64 bytes */
        memset(p, 0, count);
        byteReverse(ctx->in, 16);
        BRCM_MD5Transform(ctx->buf, (unsigned int *)ctx->in);

        /* Now fill the next block with 56 bytes */
        memset(ctx->in, 0, 56);
    } else {
        /* Pad block to 56 bytes */
        memset(p, 0, count-8);
    }
    byteReverse(ctx->in, 14);

    /* Append length in bits and transform */
    ((unsigned int *)ctx->in)[14] = ctx->bits[0];
    ((unsigned int *)ctx->in)[15] = ctx->bits[1];

    BRCM_MD5Transform(ctx->buf, (unsigned int *)ctx->in);
    byteReverse((unsigned char *)ctx->buf, 4);
    memcpy(digest, ctx->buf, 16);
    memset(ctx, 0, sizeof(ctx));  /* In case it's sensitive */
    return;
}

int CheckSignature(char *imagePtr, char *digiKey)
{
    char digest[16];
    BRCM_MD5_CTX  context;
    PFILE_TAG pTag = (PFILE_TAG)imagePtr;
    int totalImageSize;

    BRCM_MD5Init(&context);
    BRCM_MD5Update(&context, digiKey, strlen(digiKey));
    totalImageSize = atoi(pTag->totalImageLen);
    BRCM_MD5Update(&context, imagePtr+TAG_LEN, totalImageSize);
    BRCM_MD5Final(digest, &context);

    if(memcmp(pTag->signiture_2, digest, 16))
        return -1;
	
    return 0;
}
#endif

#ifdef INCLUDE_SIGNATURE_RSA
int CheckSignature(char *imagePtr){
	int status;
    char signature[64];
    PFILE_TAG pTag = (PFILE_TAG)imagePtr;

    int totalImageSize = atoi(pTag->totalImageLen);
    memcpy(&signature[0], pTag->signiture_1, 20);
    memcpy(&signature[20], pTag->signiture_2, 20);
    memcpy(&signature[40], pTag->signiture_3, 24);

    status = DoVerifyData("/etc/public.key", signature, sizeof(signature), imagePtr+TAG_LEN, totalImageSize);

    return status;
}
#endif

typedef enum
{
    NO_IMAGE_FORMAT,
    BROADCOM_IMAGE_FORMAT,
    FLASH_IMAGE_FORMAT,
    PSI_TEXT_FORMAT,
    FAIL_CRC_ERROR,
} PARSE_RESULT;

typedef enum
{
    UPLOAD_OK,
    UPLOAD_FAIL_NO_MEM,
    UPLOAD_FAIL_NO_FILENAME,
    UPLOAD_FAIL_ILLEGAL_IMAGE,
    UPLOAD_FATAL,  // The state above this line will be mapped to a messge in uploadinfo.html
    UPLOAD_FAIL_RSA_ERROR,
    UPLOAD_FAIL_CRC_ERROR,
    UPLOAD_FAIL_FLASH = 20
} UPLOAD_RESULT;

static unsigned long Crc32_table[256] = {
    0x00000000, 0x77073096, 0xEE0E612C, 0x990951BA,
    0x076DC419, 0x706AF48F, 0xE963A535, 0x9E6495A3,
    0x0EDB8832, 0x79DCB8A4, 0xE0D5E91E, 0x97D2D988,
    0x09B64C2B, 0x7EB17CBD, 0xE7B82D07, 0x90BF1D91,
    0x1DB71064, 0x6AB020F2, 0xF3B97148, 0x84BE41DE,
    0x1ADAD47D, 0x6DDDE4EB, 0xF4D4B551, 0x83D385C7,
    0x136C9856, 0x646BA8C0, 0xFD62F97A, 0x8A65C9EC,
    0x14015C4F, 0x63066CD9, 0xFA0F3D63, 0x8D080DF5,
    0x3B6E20C8, 0x4C69105E, 0xD56041E4, 0xA2677172,
    0x3C03E4D1, 0x4B04D447, 0xD20D85FD, 0xA50AB56B,
    0x35B5A8FA, 0x42B2986C, 0xDBBBC9D6, 0xACBCF940,
    0x32D86CE3, 0x45DF5C75, 0xDCD60DCF, 0xABD13D59,
    0x26D930AC, 0x51DE003A, 0xC8D75180, 0xBFD06116,
    0x21B4F4B5, 0x56B3C423, 0xCFBA9599, 0xB8BDA50F,
    0x2802B89E, 0x5F058808, 0xC60CD9B2, 0xB10BE924,
    0x2F6F7C87, 0x58684C11, 0xC1611DAB, 0xB6662D3D,
    0x76DC4190, 0x01DB7106, 0x98D220BC, 0xEFD5102A,
    0x71B18589, 0x06B6B51F, 0x9FBFE4A5, 0xE8B8D433,
    0x7807C9A2, 0x0F00F934, 0x9609A88E, 0xE10E9818,
    0x7F6A0DBB, 0x086D3D2D, 0x91646C97, 0xE6635C01,
    0x6B6B51F4, 0x1C6C6162, 0x856530D8, 0xF262004E,
    0x6C0695ED, 0x1B01A57B, 0x8208F4C1, 0xF50FC457,
    0x65B0D9C6, 0x12B7E950, 0x8BBEB8EA, 0xFCB9887C,
    0x62DD1DDF, 0x15DA2D49, 0x8CD37CF3, 0xFBD44C65,
    0x4DB26158, 0x3AB551CE, 0xA3BC0074, 0xD4BB30E2,
    0x4ADFA541, 0x3DD895D7, 0xA4D1C46D, 0xD3D6F4FB,
    0x4369E96A, 0x346ED9FC, 0xAD678846, 0xDA60B8D0,
    0x44042D73, 0x33031DE5, 0xAA0A4C5F, 0xDD0D7CC9,
    0x5005713C, 0x270241AA, 0xBE0B1010, 0xC90C2086,
    0x5768B525, 0x206F85B3, 0xB966D409, 0xCE61E49F,
    0x5EDEF90E, 0x29D9C998, 0xB0D09822, 0xC7D7A8B4,
    0x59B33D17, 0x2EB40D81, 0xB7BD5C3B, 0xC0BA6CAD,
    0xEDB88320, 0x9ABFB3B6, 0x03B6E20C, 0x74B1D29A,
    0xEAD54739, 0x9DD277AF, 0x04DB2615, 0x73DC1683,
    0xE3630B12, 0x94643B84, 0x0D6D6A3E, 0x7A6A5AA8,
    0xE40ECF0B, 0x9309FF9D, 0x0A00AE27, 0x7D079EB1,
    0xF00F9344, 0x8708A3D2, 0x1E01F268, 0x6906C2FE,
    0xF762575D, 0x806567CB, 0x196C3671, 0x6E6B06E7,
    0xFED41B76, 0x89D32BE0, 0x10DA7A5A, 0x67DD4ACC,
    0xF9B9DF6F, 0x8EBEEFF9, 0x17B7BE43, 0x60B08ED5,
    0xD6D6A3E8, 0xA1D1937E, 0x38D8C2C4, 0x4FDFF252,
    0xD1BB67F1, 0xA6BC5767, 0x3FB506DD, 0x48B2364B,
    0xD80D2BDA, 0xAF0A1B4C, 0x36034AF6, 0x41047A60,
    0xDF60EFC3, 0xA867DF55, 0x316E8EEF, 0x4669BE79,
    0xCB61B38C, 0xBC66831A, 0x256FD2A0, 0x5268E236,
    0xCC0C7795, 0xBB0B4703, 0x220216B9, 0x5505262F,
    0xC5BA3BBE, 0xB2BD0B28, 0x2BB45A92, 0x5CB36A04,
    0xC2D7FFA7, 0xB5D0CF31, 0x2CD99E8B, 0x5BDEAE1D,
    0x9B64C2B0, 0xEC63F226, 0x756AA39C, 0x026D930A,
    0x9C0906A9, 0xEB0E363F, 0x72076785, 0x05005713,
    0x95BF4A82, 0xE2B87A14, 0x7BB12BAE, 0x0CB61B38,
    0x92D28E9B, 0xE5D5BE0D, 0x7CDCEFB7, 0x0BDBDF21,
    0x86D3D2D4, 0xF1D4E242, 0x68DDB3F8, 0x1FDA836E,
    0x81BE16CD, 0xF6B9265B, 0x6FB077E1, 0x18B74777,
    0x88085AE6, 0xFF0F6A70, 0x66063BCA, 0x11010B5C,
    0x8F659EFF, 0xF862AE69, 0x616BFFD3, 0x166CCF45,
    0xA00AE278, 0xD70DD2EE, 0x4E048354, 0x3903B3C2,
    0xA7672661, 0xD06016F7, 0x4969474D, 0x3E6E77DB,
    0xAED16A4A, 0xD9D65ADC, 0x40DF0B66, 0x37D83BF0,
    0xA9BCAE53, 0xDEBB9EC5, 0x47B2CF7F, 0x30B5FFE9,
    0xBDBDF21C, 0xCABAC28A, 0x53B39330, 0x24B4A3A6,
    0xBAD03605, 0xCDD70693, 0x54DE5729, 0x23D967BF,
    0xB3667A2E, 0xC4614AB8, 0x5D681B02, 0x2A6F2B94,
    0xB40BBE37, 0xC30C8EA1, 0x5A05DF1B, 0x2D02EF8D
};

/***************************************************************************
// Function Name: getCrc32
// Description  : caculate the CRC 32 of the given data.
// Parameters   : pdata - array of data.
//                size - number of input data bytes.
//                crc - either CRC32_INIT_VALUE or previous return value.
// Returns      : crc.
****************************************************************************/
UINT32 getCrc32(byte *pdata, UINT32 size, UINT32 crc)
{
    while (size-- > 0)
        crc = (crc >> 8) ^ Crc32_table[(crc ^ *pdata++) & 0xff];

    return crc;
}

//********************************************************************************
// Get board id
//********************************************************************************
int sysGetBoardIdName(char *name, int length)
{
    return( boardIoctl(BOARD_IOCTL_GET_ID, 0, name, length, 0, "") );
}

UPLOAD_RESULT verifyTag(PFILE_TAG pTag,int bufSize)
{
    UINT32 crc;
    int totalImageSize;
    int tagVer, curVer, FlashLayoutVer, curFlashLayoutVer, tagVerLast;
    char sysName[40] = "";
	int flashtotalSize=0;
	sysGetBoardIdName( sysName, sizeof(sysName) );
	flashtotalSize = sysFlashSizeGet();
    printf("Checking Image tag.....\n");

    //if (strncmp(sysName, pTag->boardId, strlen(sysName)) != 0)
    if (strcmp(sysName, pTag->boardId) != 0)
    {
        printf( "**** ERROR: Image with tag not the same as board is so can NOT be flashed onto board ID '%s'\n\n", sysName );
               return UPLOAD_FAIL_ILLEGAL_IMAGE;
    }
    printf("User request image with tag '%s' flash into board ID '%s'\n", pTag->boardId, sysName );

    // check tag validate token first
    crc = CRC32_INIT_VALUE;
    crc = getCrc32((char*) pTag, (UINT32)TAG_LEN-TOKEN_LEN, crc);
    if (crc != (UINT32)(*(UINT32*)(pTag->tagValidationToken))){
        printf("Illegal image ! Tag crc failed.\n");
        return UPLOAD_FAIL_CRC_ERROR;
    }
    // check imageCrc
    totalImageSize = bufSize - TAG_LEN;
    printf("totalImageSize=%d\n",totalImageSize);
    crc = CRC32_INIT_VALUE;
    crc = getCrc32(((char*)pTag + TAG_LEN), (UINT32) totalImageSize, crc);      
    if (crc != (UINT32) (*(UINT32*)(pTag->imageValidationToken))){
        printf("Illegal image ! image crc failed.\n");
        //return UPLOAD_FAIL_ILLEGAL_IMAGE;
        return UPLOAD_FAIL_CRC_ERROR;
    }    
    // only allow same or greater tag versions (up to tagVerLast - 26) to be uploaded
    tagVer = atoi(pTag->tagVersion);
    curVer = atoi(BCM_TAG_VER);
    FlashLayoutVer = atoi(pTag->FlashLayoutVersion);
    curFlashLayoutVer = atoi(BCM_FLASH_LAYOUT_VER);
    tagVerLast = atoi(BCM_TAG_VER_LAST);

    { 
     unsigned long cfeAddr;
     
//    #if defined(BCM6348A) || defined(BCM6348B) 

     int kernelSize;
     
     kernelSize =  atoi(pTag->kernelLen);  
     if (!kernelSize) {
       UPLOAD_RESULT status = UPLOAD_FAIL_CRC_ERROR;
       unsigned char* imgPtr;
       PFILE_TAG qTag; 
       int binfsSize;
       unsigned long binfsAddr;
            	
       binfsSize = atoi(pTag->rootfsLen);
       binfsAddr = (unsigned long) strtoul(pTag->rootfsAddress, NULL, 10);
       imgPtr=(unsigned char*)  pTag;
       imgPtr+=TAG_LEN+binfsSize;
       qTag = (PFILE_TAG) imgPtr;
       status = verifyTag(qTag,bufSize-TAG_LEN-binfsSize);
       if (status == UPLOAD_OK) printf("binfs image check OK\n"); 
       return status;
       
    }
 //   #endif    
     
    cfeAddr = (unsigned long) strtoul(pTag->cfeAddress, NULL, 10);
   //Not allow user to downgrade to 2.10.1 and upgrade to V2.20.x need include CFE since SPI is 24k instead 16k 
    if (FlashLayoutVer == 0) {
       printf("tagVer=%d   curVer=%d   tagVerLast=%d  FlashLayoutVer=%d  curFlashLayoutVer=%d \n\n\n\n\n",
       tagVer, curVer, tagVerLast, FlashLayoutVer, curFlashLayoutVer);
       printf(" ***** Not allowed to downgrade to 2.10 or earlier version ******\n\n");
       return UPLOAD_FAIL_ILLEGAL_IMAGE;
    }
    if ((FlashLayoutVer < curFlashLayoutVer)&&(!(cfeAddr))){
       printf("tagVer=%d   curVer=%d   tagVerLast=%d  FlashLayoutVer=%d  curFlashLayoutVer=%d \n\n\n\n\n",
       tagVer, curVer, tagVerLast, FlashLayoutVer, curFlashLayoutVer);
       printf(" ***** You are trying to downgrade to compatable flash layer out ********\n\n");
       printf(" ***** CFE must included ******\n\n");
       return UPLOAD_FAIL_ILLEGAL_IMAGE;
    }
    if ((FlashLayoutVer > curFlashLayoutVer)&&(!(cfeAddr))){ 
       printf("tagVer=%d   curVer=%d   tagVerLast=%d  FlashLayoutVer=%d  curFlashLayoutVer=%d \n\n\n\n\n",
       tagVer, curVer, tagVerLast, FlashLayoutVer, curFlashLayoutVer);
       printf(" ***** Your flash layout is different so you are not allowed to upgrade/downgrade ********\n\n");
       printf(" ***** CFE must included ******\n\n");
       return UPLOAD_FAIL_ILLEGAL_IMAGE;
    }
   }

    if (tagVer < curVer || tagVer > tagVerLast)
    {
       printf("Firmware tag version [%d] is not compatible with the current Tag version [%d]\n", tagVer, curVer);
       return UPLOAD_FAIL_ILLEGAL_IMAGE;
    }
    printf("flashtotalSize=[%d]\n", flashtotalSize);
#if	defined(CHIP_6348)
	printf("curFlashLayoutVer=[%d], FlashLayoutVer=[%d]\n", curFlashLayoutVer,FlashLayoutVer);
	if (flashtotalSize == 0x800000){//6348 is 8M
		if (curFlashLayoutVer > 4 && FlashLayoutVer < 5) { /* 5 start from 3.28u */
                        printf(" ***** You are not allowed to downgrade to the version not support flash/image size auto check on 8M bcm6348 ********\n\n");
			return UPLOAD_FAIL_ILLEGAL_IMAGE;
		}
	}
#elif defined(CHIP_6338)
	if (flashtotalSize == 0x400000){//6338 is 4M
		if (curFlashLayoutVer > 4 && FlashLayoutVer < 5) { /* 5 start from 3.28u */
                        printf(" ***** You are not allowed to downgrade to the version not support flash/image size auto check on 4M bcm6338 ********\n\n");
			return UPLOAD_FAIL_ILLEGAL_IMAGE;
		}
	}
#endif     
    
    
    return UPLOAD_OK;
}

// parseImageData
// parse the image data to see if it Broadcom flash format or flash image format file
// return: NO_IMAGE_FORMAT  
//         BROADCOM_IMAGE_FORMAT       
//         FLASH_IMAGE_FORMAT
//	   PSI_IMAGE_FORMAT          
 
PARSE_RESULT parseImageData(char *image_start_ptr, int bufSize, int upload_type)
{
	PARSE_RESULT result = NO_IMAGE_FORMAT;
	UPLOAD_RESULT status = UPLOAD_FATAL;
	/* If the image is PSI image */
	if (upload_type == WEB_UPLOAD_IMAGE) {
       	// Check if the first part of the image is the Broadcom defined TAG record.
    	status = verifyTag((FILE_TAG *) image_start_ptr, bufSize);
    	if (status == UPLOAD_OK){
        	result = BROADCOM_IMAGE_FORMAT;
           	printf("Odm firmware format verified.\n");
    	} else if ( status == UPLOAD_FAIL_CRC_ERROR){
    		result = FAIL_CRC_ERROR;
       	}
	}
	return result;
}

// depending on the image type, do the brcm image or whole flash image
// return: UPLOAD_RESULT
UPLOAD_RESULT flashImage(char *imagePtr, PARSE_RESULT imageType, int imageLen)
{
    PFILE_TAG pTag = (PFILE_TAG) imagePtr;
    int cfeSize, rootfsSize, kernelSize,binfsSize;
    unsigned long cfeAddr, rootfsAddr, kernelAddr,binfsAddr;
    UPLOAD_RESULT status = UPLOAD_OK;
    char* newImagePtr = imagePtr; /* Vic add for dual filesystem exist for 8M flash */   

    if (imageType == FLASH_IMAGE_FORMAT)  
    {
        printf("\nFlash whole image...");
        if ((sysFlashImageSet(imagePtr, imageLen-TOKEN_LEN, (int)FLASH_BASE, BCM_IMAGE_WHOLE)) == -1)
        {
            printf("Failed to flash whole image\n");
            return UPLOAD_FAIL_FLASH;
        }
        return UPLOAD_OK;
    }

	 /* Vic add begin for dual filesystem exist for 8M flash */   
     /* check kernel existence if kernel does not exist means there is only binfs partition */
    kernelAddr = (unsigned long) strtoul(pTag->kernelAddress, NULL, 10);
    kernelSize = atoi(pTag->kernelLen);   
    if (!kernelSize) {
       binfsSize = atoi(pTag->rootfsLen);
       binfsAddr = (unsigned long) strtoul(pTag->rootfsAddress, NULL, 10);
       printf("\nFlashing bin file system @ 0x%08x size 0x%x\n",binfsAddr-TAG_LEN,binfsSize);
       if ((sysFlashImageSet(imagePtr, binfsSize+TAG_LEN, (int) binfsAddr-TAG_LEN,BCM_IMAGE_BINFS)) == -1) {
            printf("Failed to flash bin file system\n");
            return UPLOAD_FAIL_FLASH;
       }  
       newImagePtr+=TAG_LEN+binfsSize;
       pTag = (PFILE_TAG) newImagePtr;
       /*call verifyTag too here*/
 	}
 	 /* From here newImagePtr will replace imagePtr by the above code */
     /* Vic add end for dual filesystem exist for 8M flash */   
     
    // check imageCrc
    cfeSize = rootfsSize = kernelSize = 0;

    // check cfe's existence
    cfeAddr = (unsigned long) strtoul(pTag->cfeAddress, NULL, 10);
    cfeSize = atoi(pTag->cfeLen);
    // check kernel existence
    kernelAddr = (unsigned long) strtoul(pTag->kernelAddress, NULL, 10);
    kernelSize = atoi(pTag->kernelLen);
    // check root filesystem existence
    rootfsAddr = (unsigned long) strtoul(pTag->rootfsAddress, NULL, 10);
    rootfsSize = atoi(pTag->rootfsLen);

    if (cfeAddr) 
    {
        printf("\nFlashing CFE...\n");
        if ((sysFlashImageSet(newImagePtr+TAG_LEN, cfeSize, (int) cfeAddr,
            BCM_IMAGE_CFE)) == -1)
        {
            printf("Failed to flash CFE\n");
            return UPLOAD_FAIL_FLASH;
        }
    }

    if (rootfsAddr && kernelAddr) 
    {
        char *tagFs = newImagePtr;

        // tag is alway at the sector start of fs
        if (cfeAddr)
        {
            tagFs = newImagePtr + cfeSize;       // will trash cfe memory, but cfe is already flashed
            memcpy(tagFs, newImagePtr, TAG_LEN);
        }

        printf("\nFlashing root file system and kernel(router will reboot after this).......");
        if ((sysFlashImageSet(tagFs, TAG_LEN+rootfsSize+kernelSize, (int)(rootfsAddr-TAG_LEN), BCM_IMAGE_FS)) == -1)
        {
            printf("Failed to flash root file system\n");
            return UPLOAD_FAIL_FLASH;
        }
    }

    if (status != UPLOAD_OK)
        printf("**** IMAGE FLASH FAILED ****\n");
    else 
        printf("Image flash done.\n");
    
    return status;
}

void bcmKillAllApps(void) {
   // NOTE: Add the app name before NULL. Order is important
   char *apps[]=
   {
      "bftpd",
      "telnetd",   
      "sshd",
      "snmp",
      "upnp",
      "sntp",
      "ddnsd",
      "reaim",
      "klogd",
      "syslogd",
      "tftpd",
      "ripd",
      "zebra ",
      "dhcpc",
#if defined(SUPPORT_MPD)
      "mpd"
#endif      
      "igmp",
      "dproxy",
#if ODM_KILL_DHCPD
      "dhcpd",
#endif
      NULL,
   };
   char cmd[SYS_CMD_LEN], app[SYS_CMD_LEN], buf[SYS_CMD_LEN];
   char pidStr[SYS_CMD_LEN];
   int pid = 0;
   int i = 0;
   FILE *fs;
   int socketPid = 0;
   int curPid = getpid();

   bcmSystemMute("ps > /var/pslist");
   fs = fopen("/var/pslist", "r");
   if (fs == NULL)
      return;
   bcmSystem("cat /var/pslist");

   //printf("curPid=%d, socketPid=%d, htpdPid=%d, inWeb=%d\n", curPid, socketPid, httpdPid, inWeb);
   // get app and read thru pslist according to the order of apps
   i = 0;
   do 
   {
      strcpy(app, apps[i]);
      while (fgets(buf, SYS_CMD_LEN, fs) > 0)  
      {
         if (strstr(buf, app) != NULL) // found command line with match app name
         {  
            // find pid string
            sscanf(buf, "%s\n", pidStr);
            pid = atoi(pidStr);
            if (pid != curPid && pid != socketPid)
            {
               printf("kill %s process...\n", app);
               sprintf(cmd, "kill -%d %d", 9, pid);
               bcmSystem(cmd);
            }
         }
      } 
      rewind(fs);    // start pslist over again
   } while (apps[++i] != NULL);

   fclose(fs);
   bcmSystemMute("rm /var/pslist");
#if defined(CHIP_6338)   
   bcmRemoveModules(0, 1);
#endif   
}

void termHttpc(void){

    if(imageBufPtrPtr)
        free(imageBufPtrPtr);

    if(s >= 0)
        close(s);
    
    exit(0);
}

void connectError(void){
    FILE *fp=NULL;
    
    fp=fopen("/var/oneclickstatus", "w");	
    if (fp){
        fprintf(fp,"%s","2");
        fclose(fp);
    }

    fp=fopen("var/imagestatus", "w");	
    if (fp){
        fprintf(fp,"%s","2");
        fclose(fp);
    }
}

int checkTag(PFILE_TAG pTag){
    char sysName[40] = "";

    sysGetBoardIdName(sysName, sizeof(sysName) );

    printf("Checking tag.....\n");

    if (strcmp(sysName, pTag->boardId) != 0)
    {
        printf( "**** ERROR: Image with tag not the same as board is so can NOT be flashed onto board ID '%s'\n\n", sysName );
               return UPLOAD_FAIL_ILLEGAL_IMAGE;
    }
    printf("User request image with tag '%s' flash into board ID '%s'\n", pTag->boardId, sysName );
    return UPLOAD_OK;
}

int main(int argc, char *argv[]){
    struct hostent *servhost;            
    struct sockaddr_in server;           
    struct servent *service;             
    FILE *fp;
    char buf[BUF_SIZE];
    int upload_len=0;
    int upload_type=WEB_UPLOAD_IMAGE;
    char *cp=NULL;                   
	int flashSize = 0;
	UPLOAD_RESULT result = UPLOAD_OK;
	int byteRd = 0;
	unsigned int uploadSize = 0;
	PARSE_RESULT imageType;
	FILE *fconfig;
	FILE *fconfigstatus;
       FILE *fimagestatus;
	FILE *fwebserver=NULL;
	int bytes=0; 
#ifdef INCLUDE_SIGNATURE_RSA
	int sigresult=-1;
#endif
    char host[BUF_SIZE] = "localhost";   
    char path[BUF_SIZE] = "/";            
    unsigned short port = 80;
    struct pollfd poll_fd;
    int charRead=0;
    int preCheckTag=-1;
    char *ptrTag=NULL;
    int moveimageprt = 0;

    signal(16, termHttpc); //register SIGUSR1 handler
    
    if ( argc > 1 ){                    
        char host_path[BUF_SIZE];

        if ( strlen(argv[1]) > BUF_SIZE-1 ){
            fprintf(stderr, "URL unknown\n");
            connectError();
            return 1;
        }
                                         
                                         
        if ( strstr(argv[1], "http://") &&
             sscanf(argv[1], "http://%s", host_path) &&
             strcmp(argv[1], "http://" ) ){
            char *p;

            p = strchr(host_path, '/');  
            if ( p != NULL ){
                strcpy(path, p);        
                *p = '\0';
                strcpy(host, host_path); 
            } else {                    
                strcpy(host, host_path); 
            }

            p = strchr(host, ':');       
            if ( p != NULL ){
                port = atoi(p+1);        
                if ( port <= 0 ){        
                    port = 80;           
                }
                *p = '\0';
            }
        } else {
            fprintf(stderr, "URL error http://host/path \n");
            connectError();
            return 1;
        }
    }

    //printf("http://%s%s \n\n",host, path);
    
    if (strstr(path,"btvoyager-one-click-fw-update")){
    	upload_type=WEB_UPLOAD_SETTINGS;
    }

	//printf("upload_type=%d\n",upload_type);
                                
    servhost = gethostbyname(host);
    if ( servhost == NULL ){
        fprintf(stderr, "[%s]\n", host);
        connectError();
        return 0;
    }

    bzero((char *)&server, sizeof(server));     

    server.sin_family = AF_INET;

                                               
    bcopy(servhost->h_addr, (char *)&server.sin_addr, servhost->h_length);

    if ( port != 0 ){                          
        server.sin_port = htons(port);
    } else {                                   
        service = getservbyname("http", "tcp");
        if ( service != NULL ){                
            server.sin_port = service->s_port;
        } else {                               
            server.sin_port = htons(80);
        }
    }
                                
    if ( ( s = socket(AF_INET, SOCK_STREAM, 0) ) < 0 ){
        fprintf(stderr, "can't create socket\n");
        connectError();
        return 1;
    }
                                
    if ( connect(s, (struct sockaddr *)&server, sizeof(server)) == -1 ){
        fprintf(stderr, "connect fail\n");
        connectError();
        return 1;
    }

    poll_fd.fd = s; /* We'll be polling shortly */
    poll_fd.events = POLLIN;
    
#if 0                                
    fp = fdopen(s, "r+");
    if ( fp == NULL ){
    	fprintf(stderr, "fdopen fail\n");
        return 1;
    }
    //setvbuf(fp, NULL, _IONBF, 0);

    fprintf(fp, "GET %s HTTP/1.0\r\n", path);
    fprintf(fp, "Host: %s:%d\r\n", host, port);
    fprintf(fp, "User-Agent: %s\r\n", "http_get");
    fprintf(fp, "Accept: */*\r\n");
    fprintf(fp, "Accept-Encoding: *\r\n");
    fprintf(fp, "Accept-Language: en\r\n");
    fprintf(fp, "Accept-Charset: iso-8859-1,*,utf-8\r\n");
    fprintf(fp, "\r\n");
#else
	/* Build request buffer, starting with the GET. */
	buf[0]='\0';
    bytes = snprintf( buf, sizeof(buf), "GET %s HTTP/1.0\r\n", path );
    //bytes += snprintf( &buf[bytes], sizeof(buf) - bytes, "Host: %s:%d\r\n", host, port );
    bytes += snprintf( &buf[bytes], sizeof(buf) - bytes, "Host: %s\r\n", host);
    bytes += snprintf( &buf[bytes], sizeof(buf) - bytes, "User-Agent: %s\r\n", "http_get" );
    bytes += snprintf( &buf[bytes], sizeof(buf) - bytes, "Accept: */*\r\n" );
    bytes += snprintf( &buf[bytes], sizeof(buf) - bytes, "Accept-Encoding: *\r\n" );
    bytes += snprintf( &buf[bytes], sizeof(buf) - bytes, "Accept-Language: en\r\n" );
    bytes += snprintf( &buf[bytes], sizeof(buf) - bytes, "Accept-Charset: iso-8859-1,*,utf-8\r\n" );
    bytes += snprintf( &buf[bytes], sizeof(buf) - bytes, "%s", "\r\n" );
	(void) write( s, buf, bytes );

	fp = fdopen(s, "r+");
    if ( fp == NULL ){
        fprintf(stderr, "fdopen fail\n");
        return 1;
    }
#endif

#ifdef INCLUDE_SIGNATURE    
	char sigkey[20];
	int sigresult=0;
#endif    


    buf[0]='\0';
    charRead = read(s, buf, sizeof(buf));
    if (charRead > 0){
        //printf("start to print http header, charRead = [%d]\n\n", charRead);
        //dumpHex(buf, charRead);
        cp = strstr(buf, "Content-Length:");
        if (cp != NULL) {    
            cp += strlen("Content-Length:");
            upload_len = atoi(cp);
            upload_len = upload_len + 2048;
            //printf("upload_len = %d\n",upload_len);            
        }//else
            //printf("Content-Length: not found.\n");
        cp = strstr(buf, "\r\n\r\n");
        if (cp != NULL){
            //printf("This line is http header last line\n");
            cp += strlen("\r\n\r\n");
        }//else
            //printf("Something wrong !!\n");
    }//else
        //printf("charRead = [%d]\n", charRead);
	   
    byteRd = cp - buf;
    //printf("byteRd = [%d]\n", byteRd);
    //dumpHex(cp, charRead - byteRd);

	if (upload_type==WEB_UPLOAD_SETTINGS)
		goto ReadConfigFile;

	flashSize = sysFlashSizeGet() + UPLOAD_OVERHEAD;
	if (upload_len > flashSize){
		printf("Size is over by %d\n", upload_len - flashSize);
		result = UPLOAD_FAIL_ILLEGAL_IMAGE;
	}else if (upload_len < MIN_UPLOAD_LEN && upload_type == WEB_UPLOAD_IMAGE){  // for fake file or just the path without file name.
		printf("Wrong file or illegal size [%d]\n", upload_len);
		result = UPLOAD_FAIL_ILLEGAL_IMAGE;
	}else if (result == UPLOAD_OK){
		printf("upload start with len = %d with flash size = %d ****\n", upload_len, flashSize);
		printf("Allocating %d byte buffer\n", upload_len);
              bcmKillAllApps();
		curPtr = (char *) malloc(upload_len);
		if( curPtr == NULL ){
			printf("Failed to allocate memory for the image. Size required: [%d]\n", upload_len);
			result = UPLOAD_FAIL_NO_MEM;
		}else{
			imageBufPtrPtr = curPtr;
                     ptrTag = imageBufPtrPtr;
			printf("Memory allocated\n");
		}
	}

    if(result == UPLOAD_OK){
    memcpy(curPtr, cp, charRead - byteRd);
    curPtr += (charRead - byteRd);
    uploadSize += (charRead - byteRd);
    if(uploadSize >= sizeof(FILE_TAG)+128){
        if ((ptrTag)[0] == 0x36 && (ptrTag)[1] == 0x00 && (ptrTag)[2] == 0x00 && (ptrTag)[3] == 0x00){
	 } else {
	     ptrTag = ptrTag + 128;
            moveimageprt = 1;
	 }
        preCheckTag = checkTag(ptrTag);
        if(preCheckTag!=UPLOAD_OK){
            printf("Image Tag is wrong.\n");
            result = UPLOAD_FAIL_ILLEGAL_IMAGE;
        }
    }
    }
    
    while( poll( &poll_fd, 1, 10000 ) > 0 ) {
	charRead = read(s, buf, BUF_SIZE);
	if(charRead > 0){
              byteRd = charRead;
		if (result != UPLOAD_OK)
			continue;               // just throw away the data
		else{  // move the data to the imageBuf
			uploadSize += byteRd;
			if (uploadSize > upload_len){
				uploadSize = upload_len;
                            printf("Failed on over sized image ?\n");
				result = UPLOAD_FAIL_ILLEGAL_IMAGE;
			}else {
			    memcpy(curPtr, buf, byteRd);
			    curPtr += byteRd;
                        if(uploadSize >= sizeof(FILE_TAG)+128 && preCheckTag==-1){
                            if ((ptrTag)[0] == 0x36 && (ptrTag)[1] == 0x00 && (ptrTag)[2] == 0x00 && (ptrTag)[3] == 0x00){
			       } else {
				    ptrTag = ptrTag + 128;
                                moveimageprt = 1;
			       }
                            preCheckTag = checkTag(ptrTag);
                            if(preCheckTag!=UPLOAD_OK){
                                printf("Image Tag is wrong.\n");
                                result = UPLOAD_FAIL_ILLEGAL_IMAGE;
                            }else {
                                bcmRemoveModules(0, 1);
                            }
                        }
			}
		}		
	}else if(charRead == -1){
            //printf("read from socket return error.\n");
            break;
        }else
            break;
    }

	//printf("upload size = %d\n", uploadSize);
	//printf("End get image file\n");
#ifdef DEBUG_UPLOAD
	if( imageBufPtrPtr )
		dumpHex(imageBufPtrPtr, uploadSize);
#endif
	// parse the image data
	if (result == UPLOAD_OK){
		if ( upload_type == WEB_UPLOAD_IMAGE){
			if ((imageBufPtrPtr)[0] == 0x36 && (imageBufPtrPtr)[1] == 0x00 && (imageBufPtrPtr)[2] == 0x00 && (imageBufPtrPtr)[3] == 0x00){
			} else {
				imageBufPtrPtr = imageBufPtrPtr + 128;
                            moveimageprt = 1;
			}
		}
		imageType = parseImageData(imageBufPtrPtr, uploadSize, upload_type);
		if (imageType == NO_IMAGE_FORMAT){
        	result = UPLOAD_FAIL_ILLEGAL_IMAGE;
		} else if (imageType == FAIL_CRC_ERROR ){
			result = UPLOAD_FAIL_CRC_ERROR;	
		} else{
        	imageLen = uploadSize;
#ifdef INCLUDE_SIGNATURE
			if (imageType == BROADCOM_IMAGE_FORMAT || imageType == FLASH_IMAGE_FORMAT){ 
				sigresult=CheckSignature(imageBufPtrPtr, sigkey);
				printf("MD5 sigresult=%d\n",sigresult);
				if (sigresult==-1)
					result = UPLOAD_FAIL_ILLEGAL_IMAGE;
			}		
#endif
#ifdef INCLUDE_SIGNATURE_RSA
			if (imageType == BROADCOM_IMAGE_FORMAT || imageType == FLASH_IMAGE_FORMAT){
				sigresult=CheckSignature(imageBufPtrPtr);
				printf("RSA sigresult=%d\n",sigresult);
				if (sigresult != 0)
					result = UPLOAD_FAIL_RSA_ERROR;
			}		
#endif
    	}
    	//printf("imageType=%d\n",imageType);
	}

//printf("UploadSize = [%d], UploadLen = [%d]\n", uploadSize, upload_len);

	if (result != UPLOAD_OK) {
		if (curPtr != NULL ){
			printf("free memory, because upload fail\n");
                     if(moveimageprt == 1)
                        imageBufPtrPtr = imageBufPtrPtr - 128;
			free(imageBufPtrPtr);
		}
		fclose(fp);
    	       close(s);
      	       fimagestatus = fopen("var/imagestatus", "w+");
              if(preCheckTag != UPLOAD_OK){
                  fprintf(fimagestatus, "%d", 0);
              }else if(upload_len - 2048 != uploadSize)
                  fprintf(fimagestatus, "%d", 2);
              else
                  fprintf(fimagestatus, "%d", 0);
              fclose(fimagestatus);

		return result;
	}  
	
	int flashcmd =0;
	int i=0;
	if (result == UPLOAD_OK && (imageType == BROADCOM_IMAGE_FORMAT || imageType == FLASH_IMAGE_FORMAT)){
              fimagestatus = fopen("var/imagestatus", "w+");
              fprintf(fimagestatus, "%d", 1);
              fclose(fimagestatus);
		for(i=0;i<90;i++){//waiting web server command to flash image
		    if(access("/var/flashimage", 0) == 0){
			fwebserver=fopen("/var/flashimage", "r");
			if (fwebserver){
				fscanf(fwebserver,"%d",&flashcmd);
				fclose(fwebserver);
				break;
			} 
		    }
			sleep(1);
		}
		if (flashcmd == 1)
			flashImage(imageBufPtrPtr, imageType, imageLen);
              else {
                  fimagestatus = fopen("var/imagestatus", "w+");
                  fprintf(fimagestatus, "%d", 0);
                  fclose(fimagestatus);
                  if(curPtr)
                      free(curPtr);
              }
	}else{
	    fimagestatus = fopen("var/imagestatus", "w+");
           fprintf(fimagestatus, "%d", 0);
           fclose(fimagestatus);
	}       

ReadConfigFile://Read one click file and write context to /var/oneclick.conf
	
	if (upload_type==WEB_UPLOAD_SETTINGS){
		fconfig=fopen("/var/oneclick.conf", "w"); 
		if (fconfig){
			if (result == UPLOAD_OK) {
                         fwrite(cp, sizeof(char), charRead - byteRd, fconfig);
                         while( poll( &poll_fd, 1, 10000 ) > 0) {
				charRead = read(s, buf, BUF_SIZE);
				if(charRead > 0){
					fwrite(buf, sizeof(char), charRead, fconfig);
				}else
				    break;
                         }
			}
			fclose(fconfig);
			fconfigstatus=fopen("/var/oneclickstatus", "w");	
			if (fconfigstatus){
				fprintf(fconfigstatus,"%s","1");
				fclose(fconfigstatus);
			}
		} else {
			fconfigstatus=fopen("/var/oneclickstatus", "w");	
			if (fconfigstatus){
				fprintf(fconfigstatus,"%s","0");
				fclose(fconfigstatus);
			}
			printf("open configrue file fail, Exit\n");
		}
		//printf("End get configure file\n");
	}		
	
    fclose(fp);
    close(s);

    return 0;
}
