/* monitor dproxy and igmp proccess daemon */

#include <stdio.h>
#include <signal.h>
#include <dirent.h>
#include <sys/types.h>
#include <unistd.h>

#include "mpd.h"

#define READ_BUF_SIZE		128

static int stopmpdmonitor = 0;	

void stopmonitor (){
	stopmpdmonitor=1;
}

void startmonitor (){
	stopmpdmonitor=0;
}

static void remove_delimitor( char *s)
{			   
	char *p1, *p2;	  

	p1 = p2 = s;
	while ( *p1 != '\0' || *(p1+1) != '\0') {
		if (*p1 != '\0') {
		   *p2 = *p1; 
		   p2++;		
		 }	  
		 p1++;  
	}				   
	*p2='\0';		   
				
}			   

static pid_t* find_pid_by_name( char* pidName)
{
	DIR *dir;
	struct dirent *next;
	pid_t* pidList=NULL;
	int i=0;

	dir = opendir("/proc");
	if (!dir) {
		printf("cfm:Cannot open /proc");
		return NULL;
	}

	while ((next = readdir(dir)) != NULL) {
		/*FILE *status */
		FILE *cmdline;
		char filename[READ_BUF_SIZE];
		char buffer[READ_BUF_SIZE];
		/* char name[READ_BUF_SIZE]; */

		/* Must skip ".." since that is outside /proc */
		if (strcmp(next->d_name, "..") == 0)
			continue;

		/* If it isn't a number, we don't want it */
		if (!isdigit(*next->d_name))
			continue;

		/* sprintf(filename, "/proc/%s/status", next->d_name); */
		/* read /porc/<pid>/cmdline instead to get full cmd line */
		sprintf(filename, "/proc/%s/cmdline", next->d_name);
		if (! (cmdline = fopen(filename, "r")) ) {
			continue;
		}
		if (fgets(buffer, READ_BUF_SIZE-1, cmdline) == NULL) {
			fclose(cmdline);
			continue;
		}
		fclose(cmdline);

		/* Buffer should contain a string like "Name:   binary_name" */
		/*sscanf(buffer, "%*s %s", name);*/
		/* buffer contains full commandline params separted by '\0' */
		remove_delimitor(buffer);
		if (strstr(buffer, pidName) != NULL) {
			pidList=realloc( pidList, sizeof(pid_t) * (i+2));
			if (!pidList) {
				printf("cfm: Out of memeory!\n");
			closedir(dir);
				return NULL;
			}
			pidList[i++]=strtol(next->d_name, NULL, 0);
		}
	}
	closedir(dir);

	if (pidList)
			pidList[i]=0;
	else if ( strcmp(pidName, "init")==0) {
		/* If we found nothing and they were trying to kill "init",
		 * guess PID 1 and call it good...  Perhaps we should simply
		 * exit if /proc isn't mounted, but this will do for now. */
		pidList=realloc( pidList, sizeof(pid_t));
		if (!pidList) {
			printf("cfm: Out of memeory!\n");
			return NULL;
		}
			pidList[0]=1;
	} else {
		pidList=realloc( pidList, sizeof(pid_t));
		if (!pidList) {
			printf("cfm: Out of memeory!\n");
			return NULL;
		}
		pidList[0]=-1;
	}
	return pidList;
}

int bcmGetPid(char * command)
{
	char cmdline[128], *p1, *p2;
	pid_t *pid = NULL;
	int ret = 0;

	p1 = command;
	p2 = cmdline;
	while ( *p1 != '\0') {
		if (*p1 != ' ') {
		   *p2 = *p1;
		   p2++;
		 }
		 p1++;
	}
	*p2='\0';

	pid = find_pid_by_name(cmdline);
	if ( pid != NULL ) {
	   ret = (int)(*pid);
	   free(pid);
	}

	return ret;
}


void andyCheckDproxy(void){
	int dproxy_pid=-1;
	char cmd[16];
   	dproxy_pid = bcmGetPid("dproxy");
   	if(dproxy_pid <= 0) {
		sprintf(cmd, "dproxy &");
		system(cmd);
	}
	return;	
}

#if defined(INCLUDE_IGMP)
void andympdCheckIgmp(void){
	int pid = 0;
   	char buf[256], cmd[256];
   	FILE* fs;
   	int igmplopid = 0;
   	
   	igmplopid = bcmGetPid("igmp lo");
   	if (igmplopid > 0){
   		return;
   	}
   	fs = fopen("/var/igmp.conf", "r");
   	if ( fs != NULL ) {
		// start new igmp
	  	fgets(buf, 256 - 1, fs);
	  	fclose(fs);
	  	if ( buf[0] == '\0' ) return;
	  	sprintf(cmd, "/bin/igmp %s ", buf);
	  	pid = bcmGetPid(cmd);
	  	
	  	if (pid <= 0){
	  		//printf("igmp not\n");
	  		sprintf(cmd, "/bin/igmp %s &", buf);
	  		system(cmd);
	  	} 
   	}
	return;	
}
#endif

void main(void){
#if defined(INCLUDE_IGMP)	
	int wanStatus;
#endif
	FILE* fp = NULL;
	
	
	signal(SIGUSR1,stopmonitor);
	signal(SIGUSR2,startmonitor);
	
	for(;;){
		sleep(5);
		if (stopmpdmonitor == 1)
			continue;
		andyCheckDproxy();
#if defined(INCLUDE_IGMP)
		fp = NULL;
		fp = fopen("/var/wanStatus", "r");
		if ( fp != NULL ) {
			fscanf(fp,"%d",&wanStatus);
			fclose(fp);
		}
		if(wanStatus == 6){
			andympdCheckIgmp();
		}
#endif		
	}	
}
