//*************************************************************************************
// Broadcom Corp. Confidential
// Copyright 2000, 2001, 2002, 2003 Broadcom Corp. All Rights Reserved.
//
// THIS SOFTWARE MAY ONLY BE USED SUBJECT TO AN EXECUTED
// SOFTWARE LICENSE AGREEMENT BETWEEN THE USER AND BROADCOM.
// YOU HAVE NO RIGHT TO USE OR EXPLOIT THIS MATERIAL EXCEPT
// SUBJECT TO THE TERMS OF SUCH AN AGREEMENT.
//
//**************************************************************************************
// File Name  : bcmImageBuilder.c
//
// Description: Build a image with a header (tag), a compressed kernel and
//              a compress JFFS2 root file system
//
// Created    : 04/23/2002  songw
//
// Modified   : 05/18/2002  seanl
//              Redefined the tag info.  Add imageType and getting all the flash address
//              from "board.h" which is shared by CFE and web uploading code.
//
//            : 04/18/2003 seanl
//              add the support for cramfs and new flash image layout:
//              1). minicfe : 64k
//              2). TAG:      256b
//              3). cramfs:   
//              4). kernel:
//              5). psi:      16k (default)
//              
//
//**************************************************************************************
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <getopt.h>
#include <netinet/in.h>

#define  BCMTAG_EXE_USE

#include "bcmTag.h"
#ifdef INCLUDE_SIGNATURE
   #include "version.h"
#endif

/* The BCM963xx flash memory map contains five sections: boot loader,
 * file system, kernel, nvram storage, persistent storage.  This program
 * defines the starting address for the boot loader, file system and kernel.
 * The run-time flash driver determines the starting address of the nvram
 * and persistent storage sections based on the flash part size.
 */
#define FLASH_BASE                      0xbfc00000

/* Typedefs.h */
typedef unsigned char byte;
typedef unsigned int UINT32;

#define BUF_SIZE        100 * 1024
static byte buffer[BUF_SIZE];

/*
     struct option {
             char *name;
             int has_arg;
             int *flag;
             int val;
     };

     The name field should contain the option name without the leading double
     dash.
     The has_arg field should be one of:
     no_argument        no argument to the option is expect.
     required_argument  an argument to the option is required.
     optional_argument  an argument to the option may be presented.
     If flag is non-NULL, then the integer pointed to by it will be set to the
     value in the val field. If the flag field is NULL, then the val field
     will be returned. Setting flag to NULL and setting val to the correspond-
     ing short option will make this function act just like getopt(3).
*/
static struct option longopts[] = {
    { "help", 0, 0, 'h' },
    { "version", 0, 0, 'v' },
    { "littleendian", 0, 0, 'l'},
    { "board", 1, 0, 0},
    { "blocksize", 1, 0, 0},
    { "output", 1, 0, 0},
    { "cfefile", 1, 0, 0},
    { "rootfsfile", 1, 0, 0},
    { "kernelfile", 1, 0, 0},
    { "include-cfe", 0, 0, 'i'},
    { 0, 0, 0, 0 }
};

static char version[] = "Broadcom Image Builder version 0.3";

static char usage[] = "Usage:\t%s {-h|--help|-v|--version}\n"
                    "\t[--littleendian]\n"
                    "\t--board <boardid> --blocksize {64|128}\n"
                    "\t--output <filename> --cfefile <filename>\n"
                    "\t[--rootfsfile <filename> --kernelfile <filename> [-i|--include-cfe]]\n";

static char *progname;

/*=========== Add by Andrew to produce MD5 signature (02/13/2004)==========*/
void print_digest(char *digest, int size)
{
    int i;
    printf("\tFirmware signature            : ");
    for(i=0;i<size;i++) printf("%02X", digest[i]&0xff);
    printf("\n");

    return;
}
//////////////////////////////////////////////////////////////////
#ifdef INCLUDE_SIGNATURE

#define BUFF_SIZE 256

typedef struct MD5Context {
    unsigned int buf[4];
    unsigned int bits[2];
    char         in[64];
}MD5_CTX;

#define F1(x, y, z) (z ^ (x & (y ^ z)))
#define F2(x, y, z) F1(z, x, y)
#define F3(x, y, z) (x ^ y ^ z)
#define F4(x, y, z) (y ^ (x | ~z))

#define MD5STEP(f, w, x, y, z, data, s) ( w += f(x, y, z) + data,  w = w<<s | w>>(32-s),  w += x )

void byteReverse (unsigned char *buf, unsigned longs) {
    unsigned int t;
    do {
        t = (unsigned int)((unsigned)buf[3]<<8 | buf[2]) << 16 | ((unsigned)buf[1]<<8 | buf[0]);
        *(unsigned int *)buf = t;
        buf += 4;
    } while (--longs);
}

//////////////////////////////////////////////////////////////////
void MD5Transform(unsigned int buf[4], unsigned int const in[16])
{
    register unsigned int a, b, c, d;

    a = buf[0];
    b = buf[1];
    c = buf[2];
    d = buf[3];

    MD5STEP(F1, a, b, c, d, in[0 ]+0xd76aa478,  7);
    MD5STEP(F1, d, a, b, c, in[1 ]+0xe8c7b756, 12);
    MD5STEP(F1, c, d, a, b, in[2 ]+0x242070db, 17);
    MD5STEP(F1, b, c, d, a, in[3 ]+0xc1bdceee, 22);
    MD5STEP(F1, a, b, c, d, in[4 ]+0xf57c0faf,  7);
    MD5STEP(F1, d, a, b, c, in[5 ]+0x4787c62a, 12);
    MD5STEP(F1, c, d, a, b, in[6 ]+0xa8304613, 17);
    MD5STEP(F1, b, c, d, a, in[7 ]+0xfd469501, 22);
    MD5STEP(F1, a, b, c, d, in[8 ]+0x698098d8,  7);
    MD5STEP(F1, d, a, b, c, in[9 ]+0x8b44f7af, 12);
    MD5STEP(F1, c, d, a, b, in[10]+0xffff5bb1, 17);
    MD5STEP(F1, b, c, d, a, in[11]+0x895cd7be, 22);
    MD5STEP(F1, a, b, c, d, in[12]+0x6b901122,  7);
    MD5STEP(F1, d, a, b, c, in[13]+0xfd987193, 12);
    MD5STEP(F1, c, d, a, b, in[14]+0xa679438e, 17);
    MD5STEP(F1, b, c, d, a, in[15]+0x49b40821, 22);

    MD5STEP(F2, a, b, c, d, in[1 ]+0xf61e2562,  5);
    MD5STEP(F2, d, a, b, c, in[6 ]+0xc040b340,  9);
    MD5STEP(F2, c, d, a, b, in[11]+0x265e5a51, 14);
    MD5STEP(F2, b, c, d, a, in[0 ]+0xe9b6c7aa, 20);
    MD5STEP(F2, a, b, c, d, in[5 ]+0xd62f105d,  5);
    MD5STEP(F2, d, a, b, c, in[10]+0x02441453,  9);
    MD5STEP(F2, c, d, a, b, in[15]+0xd8a1e681, 14);
    MD5STEP(F2, b, c, d, a, in[4 ]+0xe7d3fbc8, 20);
    MD5STEP(F2, a, b, c, d, in[9 ]+0x21e1cde6,  5);
    MD5STEP(F2, d, a, b, c, in[14]+0xc33707d6,  9);
    MD5STEP(F2, c, d, a, b, in[3 ]+0xf4d50d87, 14);
    MD5STEP(F2, b, c, d, a, in[8 ]+0x455a14ed, 20);
    MD5STEP(F2, a, b, c, d, in[13]+0xa9e3e905,  5);
    MD5STEP(F2, d, a, b, c, in[2 ]+0xfcefa3f8,  9);
    MD5STEP(F2, c, d, a, b, in[7 ]+0x676f02d9, 14);
    MD5STEP(F2, b, c, d, a, in[12]+0x8d2a4c8a, 20);

    MD5STEP(F3, a, b, c, d, in[5 ]+0xfffa3942,  4);
    MD5STEP(F3, d, a, b, c, in[8 ]+0x8771f681, 11);
    MD5STEP(F3, c, d, a, b, in[11]+0x6d9d6122, 16);
    MD5STEP(F3, b, c, d, a, in[14]+0xfde5380c, 23);
    MD5STEP(F3, a, b, c, d, in[1 ]+0xa4beea44,  4);
    MD5STEP(F3, d, a, b, c, in[4 ]+0x4bdecfa9, 11);
    MD5STEP(F3, c, d, a, b, in[7 ]+0xf6bb4b60, 16);
    MD5STEP(F3, b, c, d, a, in[10]+0xbebfbc70, 23);
    MD5STEP(F3, a, b, c, d, in[13]+0x289b7ec6,  4);
    MD5STEP(F3, d, a, b, c, in[0 ]+0xeaa127fa, 11);
    MD5STEP(F3, c, d, a, b, in[3 ]+0xd4ef3085, 16);
    MD5STEP(F3, b, c, d, a, in[6 ]+0x04881d05, 23);
    MD5STEP(F3, a, b, c, d, in[9 ]+0xd9d4d039,  4);
    MD5STEP(F3, d, a, b, c, in[12]+0xe6db99e5, 11);
    MD5STEP(F3, c, d, a, b, in[15]+0x1fa27cf8, 16);
    MD5STEP(F3, b, c, d, a, in[2 ]+0xc4ac5665, 23);

    MD5STEP(F4, a, b, c, d, in[0 ]+0xf4292244,  6);
    MD5STEP(F4, d, a, b, c, in[7 ]+0x432aff97, 10);
    MD5STEP(F4, c, d, a, b, in[14]+0xab9423a7, 15);
    MD5STEP(F4, b, c, d, a, in[5 ]+0xfc93a039, 21);
    MD5STEP(F4, a, b, c, d, in[12]+0x655b59c3,  6);
    MD5STEP(F4, d, a, b, c, in[3 ]+0x8f0ccc92, 10);
    MD5STEP(F4, c, d, a, b, in[10]+0xffeff47d, 15);
    MD5STEP(F4, b, c, d, a, in[1 ]+0x85845dd1, 21);
    MD5STEP(F4, a, b, c, d, in[8 ]+0x6fa87e4f,  6);
    MD5STEP(F4, d, a, b, c, in[15]+0xfe2ce6e0, 10);
    MD5STEP(F4, c, d, a, b, in[6 ]+0xa3014314, 15);
    MD5STEP(F4, b, c, d, a, in[13]+0x4e0811a1, 21);
    MD5STEP(F4, a, b, c, d, in[4 ]+0xf7537e82,  6);
    MD5STEP(F4, d, a, b, c, in[11]+0xbd3af235, 10);
    MD5STEP(F4, c, d, a, b, in[2 ]+0x2ad7d2bb, 15);
    MD5STEP(F4, b, c, d, a, in[9 ]+0xeb86d391, 21);

    buf[0] += a;
    buf[1] += b;
    buf[2] += c;
    buf[3] += d;
}

//////////////////////////////////////////////////////////////////
void MD5Init(struct MD5Context *ctx) {
    ctx->buf[0] = 0x67452301;
    ctx->buf[1] = 0xefcdab89;
    ctx->buf[2] = 0x98badcfe;
    ctx->buf[3] = 0x10325476;

    ctx->bits[0] = 0;
    ctx->bits[1] = 0;
}

//////////////////////////////////////////////////////////////////
void MD5Update( struct MD5Context *ctx, unsigned char const *buf, unsigned len)
{
    unsigned int t;

    /* Update bitcount */

    t = ctx->bits[0];
    if ((ctx->bits[0] = t + ((unsigned int)len << 3)) < t)
        ctx->bits[1]++; /* Carry from low to high */
    ctx->bits[1] += len >> 29;

    t = (t >> 3) & 0x3f;    /* Bytes already in shsInfo->data */

    /* Handle any leading odd-sized chunks */

    if ( t ) {
        unsigned char *p = (unsigned char *)ctx->in + t;

        t = 64-t;
        if (len < t) {
            memcpy(p, buf, len);
            return;
        }
        memcpy(p, buf, t);
        byteReverse(ctx->in, 16);
        MD5Transform(ctx->buf, (unsigned int *)ctx->in);
        buf += t;
        len -= t;
    }

    /* Process data in 64-byte chunks */

    while (len >= 64) {
        memcpy(ctx->in, buf, 64);
        byteReverse(ctx->in, 16);
        MD5Transform(ctx->buf, (unsigned int *)ctx->in);
        buf += 64;
        len -= 64;
    }

    /* Handle any remaining bytes of data. */

    memcpy(ctx->in, buf, len);

    return;
}

//////////////////////////////////////////////////////////////////
void MD5Final(unsigned char digest[16], struct MD5Context *ctx)
{
    unsigned count;
    unsigned char *p;

    /* Compute number of bytes mod 64 */
    count = (ctx->bits[0] >> 3) & 0x3F;

    /* Set the first char of padding to 0x80.  This is safe since there is
       always at least one byte free */
    p = ctx->in + count;
    *p++ = 0x80;

    /* Bytes of padding needed to make 64 bytes */
    count = 64 - 1 - count;

    /* Pad out to 56 mod 64 */
    if (count < 8) {
        /* Two lots of padding:  Pad the first block to 64 bytes */
        memset(p, 0, count);
        byteReverse(ctx->in, 16);
        MD5Transform(ctx->buf, (unsigned int *)ctx->in);

        /* Now fill the next block with 56 bytes */
        memset(ctx->in, 0, 56);
    } else {
        /* Pad block to 56 bytes */
        memset(p, 0, count-8);
    }
    byteReverse(ctx->in, 14);

    /* Append length in bits and transform */
    ((unsigned int *)ctx->in)[14] = ctx->bits[0];
    ((unsigned int *)ctx->in)[15] = ctx->bits[1];

    MD5Transform(ctx->buf, (unsigned int *)ctx->in);
    byteReverse((unsigned char *)ctx->buf, 4);
    memcpy(digest, ctx->buf, 16);
    memset(ctx, 0, sizeof(ctx));  /* In case it's sensitive */
    return;
}


//////////////////////////////////////////////////////////////////
UINT32 calculateSignature(char *digest, char *cfefile,    int cfe_size, 
                                        char *rootfsfile, int rootfs_size, 
                                        char *kernelfile, int kernel_size)
{
    MD5_CTX  context;
    char *imgPtr=NULL;
    FILE *stream_cfe = NULL;
    FILE *stream_rootfs = NULL;
    FILE *stream_kernel = NULL;
    char verion[30];
    int  total_size = cfe_size+rootfs_size+kernel_size;

    if ((stream_cfe = fopen(cfefile, "rb")) == NULL) {
        printf("bcmImageBuilder: Failed to open cfe file: %s\n", cfefile);
        fclose(stream_cfe);
        exit(1);
    }

    if (strlen(rootfsfile) != 0 && strlen(kernelfile) != 0) {
        if ((stream_rootfs = fopen(rootfsfile, "rb")) == NULL) {
            printf("bcmImageBuilder: Failed to open root file system file: %s\n", rootfsfile);
            fclose(stream_cfe);
            exit(1);
        }
        if ((stream_kernel = fopen(kernelfile, "rb")) == NULL) {
            printf("bcmImageBuilder: Filed to open kernel file: %s\n", kernelfile);
            fclose(stream_cfe);
            fclose(stream_rootfs);
            exit(1);
        }
    }

    imgPtr = (char *)calloc(1, total_size);

    fread( &imgPtr[0], sizeof(byte), cfe_size, stream_cfe);
    fread( &imgPtr[cfe_size], sizeof(byte), rootfs_size, stream_rootfs);
    fread( &imgPtr[cfe_size+rootfs_size], sizeof(byte), kernel_size, stream_kernel);

    MD5Init(&context);

#if RTA230_D29
    sprintf(verion, "ZXDSL831AB04");
#elif RTA230CN_D29
    sprintf(verion, "ZXDSL831AB04");
#elif RTA231_D29
    sprintf(verion, "ZXDSL831B04");
#elif RTA231CN_D29
    sprintf(verion, "ZXDSL831B04");
#elif RTA771_D29
    sprintf(verion, "ZXDSL831CB04");
#elif RTA771CN_D29
    sprintf(verion, "ZXDSL831CB04");
#else
    sprintf(verion, "alice-%s", SOFTWARE_VERSION);

#endif
    MD5Update(&context, (char *)verion, strlen(verion));

    MD5Update(&context, (char *)imgPtr, total_size);

    MD5Final(digest, &context);

    free(imgPtr);

    fclose(stream_cfe);
    fclose(stream_rootfs);
    fclose(stream_kernel);

    return 0;
}
#else
  #ifdef INCLUDE_SIGNATURE_RSA
/*=========== Add by Andrew to produce RSA signature (03/18/2004)==========*/
//////////////////////////////////////////////////////////////////
UINT32 calculateSignature(char *digest, char *cfefile,    int cfe_size,
                                        char *rootfsfile, int rootfs_size,
                                        char *kernelfile, int kernel_size)
{
    char *imgPtr=NULL;
    FILE *stream_cfe = NULL;
    FILE *stream_rootfs = NULL;
    FILE *stream_kernel = NULL;
    int  total_size = cfe_size+rootfs_size+kernel_size;

    if ((stream_cfe = fopen(cfefile, "rb")) == NULL) {
        printf("bcmImageBuilder: Failed to open cfe file: %s\n", cfefile);
        fclose(stream_cfe);
        exit(1);
    }

    if (strlen(rootfsfile) != 0 && strlen(kernelfile) != 0) {
        if ((stream_rootfs = fopen(rootfsfile, "rb")) == NULL) {
            printf("bcmImageBuilder: Failed to open root file system file: %s\n", rootfsfile);
            fclose(stream_cfe);
            exit(1);
        }
        if ((stream_kernel = fopen(kernelfile, "rb")) == NULL) {
            printf("bcmImageBuilder: Filed to open kernel file: %s\n", kernelfile);
            fclose(stream_cfe);
            fclose(stream_rootfs);
            exit(1);
        }
    }

    imgPtr = (char *)calloc(1, total_size);

    fread( &imgPtr[0], sizeof(byte), cfe_size, stream_cfe);
    fread( &imgPtr[cfe_size], sizeof(byte), rootfs_size, stream_rootfs);
    fread( &imgPtr[cfe_size+rootfs_size], sizeof(byte), kernel_size, stream_kernel);

    fclose(stream_cfe);
    fclose(stream_rootfs);
    fclose(stream_kernel);

    DoSignData("private.key", digest, imgPtr, total_size);

    free(imgPtr);

    return 0;
}
  #endif
#endif
/*=========== end of Add ==========*/

/***************************************************************************
// Function Name: getCrc32
// Description  : caculate the CRC 32 of the given data.
// Parameters   : pdata - array of data.
//                size - number of input data bytes.
//                crc - either CRC32_INIT_VALUE or previous return value.
// Returns      : crc.
****************************************************************************/
UINT32 getCrc32(byte *pdata, UINT32 size, UINT32 crc)
{
    while (size-- > 0)
        crc = (crc >> 8) ^ Crc32_table[(crc ^ *pdata++) & 0xff];

    return crc;
}

static void build_image(int littleendian, char *board, unsigned int blksize, 
    char *outputfile, char *cfefile, char *rootfsfile, char *kernelfile, int includecfe)
{
    FILE *stream_cfe = NULL;
    FILE *stream_rootfs = NULL;
    FILE *stream_kernel = NULL;
    FILE *stream_output = NULL;
    unsigned int total_size, cfe_size, rootfs_size, kernel_size, count, numWritten;
    unsigned int cfeaddr = FLASH_BASE;
    unsigned int fskerneladdr;
    FILE_TAG tag;
    UINT32 imageCrc, tagCrc , fskernelCrc;
#ifdef INCLUDE_SIGNATURE
    char digest[16];
#else
  #ifdef INCLUDE_SIGNATURE_RSA
    char digest[64];
  #endif
#endif

    if ((stream_cfe = fopen(cfefile, "rb")) == NULL) {
        printf("bcmImageBuilder: Failed to open cfe file: %s\n", cfefile);
        fclose(stream_cfe);
        exit(1);
    }

    if (strlen(rootfsfile) != 0 && strlen(kernelfile) != 0) {
        if ((stream_rootfs = fopen(rootfsfile, "rb")) == NULL) {
            printf("bcmImageBuilder: Failed to open root file system file: %s\n", rootfsfile);
            fclose(stream_cfe);
            exit(1);
        }
        if ((stream_kernel = fopen(kernelfile, "rb")) == NULL) {
            printf("bcmImageBuilder: Filed to open kernel file: %s\n", kernelfile);
            fclose(stream_cfe);
            fclose(stream_rootfs);
            exit(1);
        }
    }

    if((stream_output = fopen(outputfile, "wb")) == NULL) {
        printf("bcmImageBuilder: Failed to create output file: %s\n", outputfile);
        fclose(stream_cfe);
        fclose(stream_kernel);
        fclose(stream_rootfs);
        exit(1);
    }

    total_size = cfe_size  = rootfs_size = kernel_size = count = 0;
    imageCrc = CRC32_INIT_VALUE;

    while (!feof( stream_cfe )) {
        count = fread( buffer, sizeof(byte), BUF_SIZE, stream_cfe);
        if (ferror( stream_cfe)) {
            perror( "bcmImageBuilder: Read error" );
            exit(1);
        }
        imageCrc = getCrc32(buffer, count, imageCrc);
        /* Total up actual bytes read */
        total_size += count;
        cfe_size += count;
    }

    /* Calculate address after cfe in flash;  Tag is before fs now */
    fskerneladdr = cfeaddr + ((cfe_size + (blksize - 1)) & ~(blksize - 1));

    if (stream_rootfs && stream_kernel) {
        if (!includecfe) {
            /* CFE is not included in this image */
            total_size = 0;
            imageCrc = CRC32_INIT_VALUE;
            fclose(stream_cfe);
            stream_cfe = NULL;
        }
        fskernelCrc = CRC32_INIT_VALUE;
        while (!feof( stream_rootfs )) {
            count = fread( buffer, sizeof(byte), BUF_SIZE, stream_rootfs);
            if (ferror( stream_rootfs)) {
                perror( "bcmImageBuilder: Read error" );
                exit(1);
            }
            imageCrc = getCrc32(buffer, count, imageCrc);
            fskernelCrc = getCrc32(buffer, count, fskernelCrc);
            /* Total up actual bytes read */
            total_size += count;
            rootfs_size += count;
        }

        while (!feof( stream_kernel )) {
            count = fread( buffer, sizeof(byte), BUF_SIZE, stream_kernel );
            if (ferror( stream_kernel)) {
                perror( "bcmImageBuilder: Read error" );
                exit(1);
            }
            imageCrc = getCrc32(buffer, count, imageCrc);
            fskernelCrc = getCrc32(buffer, count, fskernelCrc);
            /* Total up actual bytes read */
            total_size += count;
            kernel_size += count;
        }
    }

    if (!stream_cfe) {
        cfeaddr = 0;
        cfe_size = 0;
    }

    if (!stream_rootfs || !stream_kernel) {
        fskerneladdr = 0;
        rootfs_size = 0;
        kernel_size = 0;
    }


    // fill the tag structure
    memset(&tag, 0, sizeof(FILE_TAG));

    strcpy(tag.tagVersion, BCM_TAG_VER);

#ifdef INCLUDE_SIGNATURE
    calculateSignature(digest, cfefile, cfe_size, rootfsfile, rootfs_size, kernelfile, kernel_size);
    strcpy(tag.signiture_1, BCM_SIG_1);
    memcpy(tag.signiture_2, digest, 16);
#else
  #ifdef INCLUDE_SIGNATURE_RSA
    calculateSignature(digest, cfefile, cfe_size, rootfsfile, rootfs_size, kernelfile, kernel_size);
    memcpy(tag.signiture_1, &digest[0], 20);
    memcpy(tag.signiture_2, &digest[20], 20);
    memcpy(tag.signiture_3, &digest[40], 24);
  #else
    strcpy(tag.signiture_1, BCM_SIG_1);
    strcpy(tag.signiture_2, BCM_SIG_2);
  #endif
#endif

    strcpy(tag.boardId, board);
    if (littleendian)
        strcpy(tag.bigEndian, "0");
    else
        strcpy(tag.bigEndian, "1");

    sprintf(tag.totalImageLen,"%d",total_size);

    sprintf(tag.cfeAddress, "%u", cfeaddr);
    sprintf(tag.cfeLen,"%d",cfe_size);

    if (!stream_rootfs || !stream_kernel) 
    {
        sprintf(tag.rootfsAddress,"%lu",(unsigned long) 0);
        sprintf(tag.rootfsLen,"%d",rootfs_size);
        sprintf(tag.kernelAddress,"%u", 0);
        sprintf(tag.kernelLen,"%d",kernel_size);
    }
    else
    {
        sprintf(tag.rootfsAddress,"%lu",(unsigned long) (fskerneladdr+TAG_LEN));
        sprintf(tag.rootfsLen,"%d",rootfs_size);
        sprintf(tag.kernelAddress,"%u", fskerneladdr + rootfs_size + TAG_LEN);
        sprintf(tag.kernelLen,"%d",kernel_size);
    }
    if (!littleendian) {
        imageCrc = htonl(imageCrc);
        fskernelCrc = htonl(fskernelCrc);
    }
    //printf("imageCrc 0x%08x fskernelCrc 0x%08x\n", imageCrc,fskernelCrc);
    memcpy(tag.imageValidationToken, (byte*)&imageCrc, CRC_LEN);

    /*Michael: add protection for user wanna downgrade to 2.10 */
    strcpy(tag.FlashLayoutVersion, BCM_FLASH_LAYOUT_VER);

    memcpy(tag.fskernelValidationToken, (byte*)&fskernelCrc, CRC_LEN);
    
    // get tag crc
    tagCrc = CRC32_INIT_VALUE;
    tagCrc = getCrc32((byte*)&tag, TAG_LEN-TOKEN_LEN, tagCrc);
    if (!littleendian)
        tagCrc = htonl(tagCrc);
    memcpy(tag.tagValidationToken, (byte*)&tagCrc, CRC_LEN);

#if 0
    printf ("tagVersion %s\n", tag.tagVersion);
    printf ("signiture_1 %s\n", tag.signiture_1);
    printf ("signiture_2 %s\n", tag.signiture_2);
    printf ("boardId %s\n", tag.boardId);
    printf ("bigEndian %s\n", tag.bigEndian);
    printf ("totalImageLen %s\n", tag.totalImageLen);
    printf ("cfeAddress %s, 0x%08X\n", tag.cfeAddress, (unsigned int)cfeaddr);
    printf ("cfeLen %s\n", tag.cfeLen);
    printf ("rootfsAddress %s, 0x%08X\n", tag.rootfsAddress, (unsigned int) fskerneladdr);
    printf ("rootfsLen %s\n", tag.rootfsLen);
    printf ("kernelAddress %s, 0x%08X\n", tag.kernelAddress, (unsigned int) (fskerneladdr + rootfs_size));
    printf ("kernelLen %s\n", tag.kernelLen);
    printf ("image ValidationToken 0x%08X\n", (UINT32)(*(UINT32*)tag.imageValidationToken));
    printf ("fskernel ValidationToken 0x%08X\n", (UINT32)(*(UINT32*)tag.fskernelValidationToken));
#endif

    //----------------Start to write image file---------------------------
    //Write tag first
    numWritten = fwrite((byte*)&tag, sizeof(byte), TAG_LEN, stream_output);
    if (numWritten != TAG_LEN) {
        printf("bcmImageBuilder: Failed to write tag: byte written: %d byte\n", numWritten);
        exit(1);
    }

    //Write the cfe if exist
    if (stream_cfe) {
        rewind(stream_cfe);
        while (!feof( stream_cfe )) {
            count = fread( buffer, sizeof(byte), BUF_SIZE, stream_cfe );
            if (ferror(stream_cfe)) {
                perror("bcmImageBuilder: Read cfe file error");
                exit(1);
            }
            numWritten = fwrite(buffer, sizeof(byte), count, stream_output);
            if (ferror(stream_output)) {
                perror("bcmImageBuilder: Write image file error");
                exit(1);
            }
        }
        fclose(stream_cfe);
    }

    //Write the rootfs
    if (stream_rootfs) {
        rewind(stream_rootfs);
        while (!feof( stream_rootfs )) {
            count = fread( buffer, sizeof(byte), BUF_SIZE, stream_rootfs );
            if (ferror(stream_rootfs)) {
                perror("bcmImageBuilder: Read root file system file error");
                exit(1);
            }
            numWritten = fwrite(buffer, sizeof(byte), count, stream_output);
            if (ferror(stream_output)) {
                perror("bcmImageBuilder: Write image file error");
                exit(1);
            }
        }
        fclose(stream_rootfs);
    }

    //Write the kernel
    if (stream_kernel) {
        rewind(stream_kernel);
        while (!feof( stream_kernel )) {
            count = fread( buffer, sizeof(byte), BUF_SIZE, stream_kernel );
            if (ferror(stream_kernel)) {
                perror("bcmImageBuilder: Read kerenl file error");
                exit(1);
            }
            numWritten = fwrite(buffer, sizeof(byte), count, stream_output);
            if (ferror(stream_output)) {
                perror("bcmImageBuilder: Write image file error");
                exit(1);
            }
        }
        fclose(stream_kernel);
    }

    fclose(stream_output);

    printf("bcmImageBuilder\n");

    if (stream_cfe)
        printf("\tCFE image size                : %u\n", cfe_size);
    printf("\tFile tag size                 : %d\n", TAG_LEN);    
    if (stream_rootfs)
        printf("\tRoot filesystem image size    : %u\n", rootfs_size);
    if (stream_kernel)
        printf("\tKernel image size             : %u\n", kernel_size);

    printf("\tCombined image file size      : %u\n", total_size+TAG_LEN);
#ifdef INCLUDE_SIGNATURE
    print_digest(digest, 16);
#else
  #ifdef INCLUDE_SIGNATURE_RSA
    print_digest(digest, 64);
  #endif
#endif
    printf("\n");
}


int main (int argc, char **argv)
{
  int optc;
  int h = 0, v = 0, littleendian = 0, includecfe = 0, lose = 0;
  int option_index = 0;
  char cfefile[256], rootfsfile[256], kernelfile[256], outputfile[256];
  char boardid[BOARD_ID_LEN];
  unsigned int blksize = 0;

  progname = argv[0];
  boardid[0] = cfefile[0] = kernelfile[0] = rootfsfile[0] = outputfile[0] = '\0';

  while ((optc = getopt_long (argc, argv, "hvli", longopts, &option_index)) != EOF) {
      switch (optc){
        case 0:
#ifdef DEBUG
          printf ("option %s", longopts[option_index].name);
          if (optarg)
             printf (" with arg %s", optarg);
          printf ("\n");
#endif
          if (strcmp(longopts[option_index].name, "cfefile") == 0) 
             strcpy(cfefile, optarg);
          else if (strcmp(longopts[option_index].name, "rootfsfile") == 0)
             strcpy(rootfsfile, optarg);
          else if (strcmp(longopts[option_index].name, "kernelfile") == 0)
             strcpy(kernelfile, optarg);
          else if (strcmp(longopts[option_index].name, "output") == 0)
             strcpy(outputfile, optarg);
          else if (strcmp(longopts[option_index].name, "board") == 0)
             strcpy(boardid, optarg);
          else if (strcmp(longopts[option_index].name, "blocksize") == 0)
             blksize = atoi(optarg) * 1024;
          break;
	case 'v':
	  v = 1;
	  break;
	case 'h':
	  h = 1;
	  break;
	case 'l':
	  littleendian = 1;
	  break;
	case 'i':
	  includecfe = 1;
	  break;
        default:
          lose = 1;
          break;
      }
  }

  if (lose || optind < argc || argc < 2) 
  {
      /* Print error message and exit.  */
      fprintf (stderr, usage, progname);
      exit (1);
  }

  if (v) {
      /* Print version number.  */
    fprintf (stderr, "%s\n", version);
    if (! h)
        exit (0);
  }

  if (h) {
      /* Print help info and exit.  */
      fprintf (stderr, "%s\n", version);
      fprintf (stderr, usage, progname);
      fputs ("\n", stderr);
      fputs ("  -h, --help                  Print a summary of the options\n", stderr);
      fputs ("  -v, --version               Print the version number\n", stderr);
      fputs ("  -l, --littleendian          Build little endian image\n", stderr);
      fputs ("  --board  <board name>       Board name\n", stderr);
      fputs ("  --blocksize  <block size>   Flash memory block size in KBytes\n", stderr);
      fputs ("  --output <filename>         Output image file name\n", stderr);
      fputs ("  --cfefile    <filename>     CFE imgage name\n", stderr);
      fputs ("  --kernelfile <filename>     Kernel image name\n", stderr);
      fputs ("  --rootfsfile <filename>     Root file system image name\n", stderr);
      fputs ("  -i, --include-cfe           Add CFE to kernel and rootfs image\n", stderr);
      exit (0);
  }


  if (strlen(boardid) == 0 || strlen(outputfile) == 0 
      || strlen(cfefile) == 0 || (blksize != 64*1024 && blksize !=128*1024))
  {
      fprintf (stderr, "Required input parameters are missing\n\n");
      fprintf (stderr, usage, progname);
      exit (1);
  }
  if ((strlen(rootfsfile) == 0 && strlen(kernelfile) != 0) ||
      (strlen(rootfsfile) != 0 && strlen(kernelfile) == 0) )
  {
      fprintf (stderr, "Options --rootfsfile and --kernelfile must be used together\n\n");
      fprintf (stderr, usage, progname);
      exit (1);
  }
  if (strlen(boardid) > 15)
  {
      fprintf (stderr, "Maximum board name length is 15 characters\n\n");
      fprintf (stderr, usage, progname);
      exit (1);
  }

  build_image(littleendian, boardid, blksize, outputfile, cfefile, rootfsfile, kernelfile, includecfe);

  exit(0);
}

