/* RDEMO.C - RSAREF demonstration program
 */

/* Copyright (C) 1991-4 RSA Laboratories, a division of RSA Data
   Security, Inc. All rights reserved.
 */

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "global.h"
#include "rsaref.h"
#include <netinet/in.h>

int main PROTO_LIST ((int, char **));
static void InitRandomStruct PROTO_LIST ((R_RANDOM_STRUCT *));
static void DoGenerateKeys PROTO_LIST ((R_RANDOM_STRUCT *));
static void WriteKeypair PROTO_LIST ((void));
static void PrintError PROTO_LIST ((char *, int));

R_RSA_PUBLIC_KEY PUBLIC_KEY;
R_RSA_PRIVATE_KEY PRIVATE_KEY;

int main (argc, argv)
int argc;
char *argv[];
{
  R_RANDOM_STRUCT randomStruct;

  InitRandomStruct (&randomStruct);

  DoGenerateKeys (&randomStruct);
  
  R_RandomFinal (&randomStruct);
  return (0);
}

/* Initialize the random structure with all zero seed bytes for test purposes.
   NOTE that this will cause the output of the "random" process to be
     the same every time.  To produce random bytes, the random struct
     needs random seeds!
 */
static void InitRandomStruct (randomStruct)
R_RANDOM_STRUCT *randomStruct;
{
  static long int seedByte;
  unsigned int bytesNeeded;

  srandom(time(0));
  seedByte = random();
  
  R_RandomInit (randomStruct);
  
  /* Initialize with all zero seed bytes, which will not yield an actual
       random number output.
   */
  while (1) {
    R_GetRandomBytesNeeded (&bytesNeeded, randomStruct);
    if (bytesNeeded == 0)
      break;
    
    R_RandomUpdate (randomStruct, (unsigned char *)&seedByte, 1);
  }
}

////////////////////////////////////////////////////////////////////////////////////
static void WritePublicKey2File(key)
  R_RSA_PUBLIC_KEY *key;
{
  FILE *fp;

  //because target board is little-endian, so need swap key->bits
  key->bits = htonl(key->bits);

  fp = fopen("key/public.key", "w");
  if(fp) {
    fwrite(key, sizeof(R_RSA_PUBLIC_KEY), 1, fp);
    fclose(fp);
  }

  return;
}

////////////////////////////////////////////////////////////////////////////////////
static void WritePrivateKey2File(key)
  R_RSA_PRIVATE_KEY *key;
{
  FILE *fp;

  fp = fopen("key/private.key", "w");
  if(fp) {
    fwrite(key, sizeof(R_RSA_PRIVATE_KEY), 1, fp);
    fclose(fp);
  }

  return;
}


static void DoGenerateKeys (randomStruct)
R_RANDOM_STRUCT *randomStruct;
{
  R_RSA_PROTO_KEY protoKey;
  int status;

  protoKey.bits = 512;
  protoKey.useFermat4 = 1;
  
  if((status = R_GeneratePEMKeys(&PUBLIC_KEY, &PRIVATE_KEY, &protoKey, randomStruct))) {
    PrintError ("generating keys", status);
    return;
  }

  fprintf(stderr, "Public key and private key are now ready to use.\n");
  
  WriteKeypair ();
}

static void WriteKeypair ()
{
  WritePublicKey2File(&PUBLIC_KEY);
  WritePrivateKey2File(&PRIVATE_KEY);
}

/* If type is zero, simply print the task string, otherwise convert the
     type to a string and print task and type.
 */
static void PrintError (task, type)
char *task;
int type;
{
  char *typeString, buf[80];

  if (type == 0) {
    puts (task);
    return;
  }
  
  /* Convert the type to a string if it is recognized.
   */
  switch (type) {
  case RE_KEY:
    typeString = "Recovered DES key cannot decrypt encrypted content";
    break;
  case RE_LEN:
    typeString = "Encrypted key length or signature length is out of range";
    break;
  case RE_MODULUS_LEN:
    typeString = "Modulus length is out of range";
    break;
  case RE_PRIVATE_KEY:
    typeString = "Private key cannot encrypt message digest, or cannot decrypt encrypted key";
    break;
  case RE_PUBLIC_KEY:
    typeString = "Public key cannot encrypt data encryption key, or cannot decrypt signature";
    break;
  case RE_SIGNATURE:
    typeString = "Signature is incorrect";
    break;
    
  default:
    sprintf (buf, "Code 0x%04x", type);
    typeString = buf;
  }

  printf ("ERROR: %s while %s\n", typeString, task);  
  fflush (stdout);
}
